﻿; ------------------------------------------------------------------------------------
; StringBuilder
; PB 4.5+ OS: Windows/Linux/MacOS Architecture: x86/x64
; Author: Alexander Aigner, cxAlex @ purebasic.fr/german
; ------------------------------------------------------------------------------------

; EnableExplicit

Structure _StringBuilder_Data
  *VTable
  *StringBuffer
  *StringBufferPointer
  *StringBufferPointerPointer
  StringSize.i
  BlockSize.i
  BlockCount.i
EndStructure

Interface StringBuilder ; StringBuilder Interface
  Add(*String) ; String hinzufügen (Pointer auf String)
  GetString.s() ; Totalen String laden
  Free() ; StringBuilder freigeben
EndInterface

; Neuer StringBuilder.
; BlockSize: mit welcher Blockgröße soll der Speicher erweitert werden?
Procedure StringBuilder_New(BlockSize = 1024)
  Protected *SB._StringBuilder_Data = AllocateMemory(SizeOf(_StringBuilder_Data))
  With *SB
    \VTable = ?_StringBuilder_VTable
    ; Speicher anlegen
    \BlockSize = BlockSize
    \StringBufferPointerPointer = @\StringBufferPointer ; Zeiger auf StringBufferPointer
  EndWith
  
  ProcedureReturn *SB
EndProcedure

Procedure _StringBuilder_Add(*SB._StringBuilder_Data, *String)
  Protected StringSize = MemoryStringLength(*String)

  With *SB
    ; Muss resizen
    While (\BlockCount*\BlockSize) < (\StringSize+StringSize+SizeOf(Character))
      \BlockCount + 1
      \StringBuffer = ReAllocateMemory(\StringBuffer, \BlockCount*\BlockSize)
      \StringBufferPointer = \StringBuffer + \StringSize
    Wend
    
    ; Kopieren
    ; CopyMemoryString Erhöht den \StringBufferPointer auf den \StringBufferPointerPointer zeigt selbst um StringSize,
    ; Deshabl müssen wir das nicht selbst machen.
    CopyMemoryString(*String, \StringBufferPointerPointer)
    
    ; Stringgröße erhöhen
    \StringSize+StringSize
  EndWith
EndProcedure

Procedure.s _StringBuilder_GetString(*SB._StringBuilder_Data)
  With *SB 
    If \StringBuffer
      ; Zurückgeben
      ProcedureReturn PeekS(\StringBuffer, \StringSize)
    EndIf
  EndWith
EndProcedure

Procedure _StringBuilder_Free(*SB._StringBuilder_Data)
  With *SB
    ; Freigeben
    If \StringBuffer
      FreeMemory(\StringBuffer)
    EndIf
    FreeMemory(*SB)
  EndWith
EndProcedure

DataSection
  _StringBuilder_VTable:
  Data.i @_StringBuilder_Add()
  Data.i @_StringBuilder_GetString()
  Data.i @_StringBuilder_Free()
EndDataSection

; IDE Options = PureBasic 4.60 Beta 3 (Windows - x64)
; CursorPosition = 26
; Folding = -
; EnableXP
; CPU = 1
; DisableDebugger