﻿; ------------------------------------------------------------------------------------
; Easy Network Manager
; PB 4.5+ OS: Windows/Linux/MacOS Architecture: x86/x64
; Author: Alexander Aigner, cxAlex @ purebasic.fr/german
; V# 1.3.3
; ------------------------------------------------------------------------------------

; If you like my work please donate to me: http://goo.gl/2Sjp0

; ------------------------------------------------------------------------------------
;{ Features:
;
; - vollständig Threadsicher
; - vollständiges Callback Managemant
; - einfach Multi-Server & Multi-Connections Verwaltung
; - einfacher Interface Zugriff
; - Client & Server Verwaltung
; - Client #PB_NetworkEvent_Disconnect
; - UDP #PB_NetworkEvent_Connect & #PB_NetworkEvent_Disconnect
; - Einfache Paket Verwaltung incl. Kompression
; - Einfaches versenden großer Daten via Streams
; - Einfaches Cache - Managemant
; - starke Parallelisierung für gute Performance
; - Protocol Manager zum einfachen entwickeln eigener Protokolle
; - Proxy Unterstüzung [SOCKS4/SOCKS5]
; - Log System zum Aufzeichnen sämtlicher Aktionen
;
;}
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
;{ FAQ:
;
; # 1:
; Beschreibung: ReceiveData/File können eventuell 1 mal zu viel aufgerufen werden,
; obwohl keine Daten anstehen. Der überflüssige Aufruf tritt immer knapp nach einem
; echten Aufruf auf. Die Ursache ist bekannt, die Lösung des Problems würde aber zu
; erheblichen Performance - Einbusen führen.
;
; Work-Around: Immer ENMP abfragen ob wirklich Daten empfangen wurden.
; 
; #2:
; Beschreibung: Versendete Pakete kommen nicht an, meistens in Server->Client
; Kommunikation.
;
; Work-Around: #ENM_Paket_ReloadDelay erhöhen, (ca. 10-30)
;
;
; #3:
; Beschreibung: plötzliche Dead-Locks.
;
; Work-Around: ENM\CloseConnection() sollte nicht auf gerade verwendete Verbindungen angewandt
; werden, eine Löung wird noch gesucht.
;
;
;}
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
;{ ToDo:
;
; - Linux/Max Ping
; - Performance weiter optimieren!
;}
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
;{ Changelog:
;
; - V# 1.3.3
; - Added: ENM_SimpleRPC Beispiel
; - BugFix: Unicode-Inkomatibiltät im Ping-Modul
;
; - V# 1.3.2
; - Added: Log-System
; - Added: AsyncFree für das Authentication-System
; - Added: Fortschittsanzeige-Proceduren für Streams auf Empfänger und Senderseite
; - Optimiert: StringBuilder integriert
; - Optimiert: Einige verbesserungen am Internen Stream-Handling, sollte nun stabiler laufen
; - Changed: Bessere Trennung zwischen Sende/Empfangsproceduren für Streams
; - Changed: Experimental Status von SafeNetwork aufgehoben, nun Standardmäßig eingebaut
; - Changed: Alle Samples an den neuen Syntax angepasst
; - BugFix: XIncludeFile unter Linux, Klein/Großschreibung berichtigt
;
; - V# 1.3.1
; - Added: Volle Dokumentation des AuthenticationSystem
; - Optimiert: Hashing im Authentication System verbessert
; - Optimiert: Kein Klartext - PW mehr zum erstellen eines Users nötig
; - BugFix: Kleiner Fehler in einer CompilerIF Sektion
; 
; - V# 1.3.0
; - Added: Authentication - System
; - Added: ENM\OpenProxyConnection() zum Herstellen von Verbindungen über Proxy - Server
; - Added: Windows-CloseConnection Workaround [CompilerSwitch]
; - Added: Default-Callback für alle Verbindungen, kann mit #ENM_Default_Callbacks
; - Added: Connect - Event wird auch auf der Client-Seite ausgelöst falls definiert
; - Optimiert: Async-Free schaltet sich nach ENM_DeInit() wieder auf Sync-Free um
; - Optimiert: sehr schnelle/viele Open/Close Events am Server
; - Optimiert: SafeNetwork System weiter überarbeitet
; - Optimiert: False-Receive EventCalls massiv vermindert
; - Optimiert: Senden/Empfangen beschleunigt
; - Optimiert: Stabilität bei korrupten Protokol-Daten erhöht
; - Optimiert: Durch die Bugfixes waren weitreichende Optimierungen in den Open/Close Proceduren möglich.
; - Changed: Add/Remove Server in Open/Close Server umbennant um der Syntax-Konvention treu zu bleiben
; - Changed: Interne Strukturen umbennant, bessere Lesbarkeit
; - BugFix: XData/InDeletion/SafeNetwork - Maps gelockt
; - BugFix: InitCon/CloseCon mussten syncronisiert werden
; - BugFix: Send/Receive/NCE - mussten Threadsicher gelockt werden
; - BugFix: sehr seltener Bug in der JQ im Zusammenhang mit Thread-Sleep
; - BugFix: Kleinen Bug in ENMS\WaitForEvent() behoben
; - BugFix: ENM\CloseConnection() konnte einen ungültigen Event-Call erzeugen
; - BugFix: gelegentlicher Fehler bei der 0-Terminierung exportierter Protokolle
; - BugFix: kleines Stabilitätsproblem im SafeNetwork System.
;  
; - V# 1.2.2
; - Added: Mistake Preventer um häufige Programmierfehler zu verhindern.
; - Added: Künstliche Limitierung der Client-Connections [Compiler Switch]
; - Added: Asyncrone Freigabe von dynamisch erstellten Objekten [Compiler Switch]
; - Optimiert: ENM_DeInit() beschleunigt.
; - Optimiert: ENM\CloseConnection() ruft nun auch auf der aufrufenden Seite ein Disconnect-Event hervor.
; - Optimiert: Transceiver weiter beschleunigt. Besonders Reset/Import/Export.
; - Optimiert: ConnectionData System benötigt nun keine Sync-Mutex mehr!
; - Optimiert: SafeNetwork benötigt keine globale Sync-Mutex mehr! [Experimentel][Compiler Switch]
; - Changed: Protocol Builder Proceduren umbennant
; - Removed: vID - System aus SafeNetwork [Experimentel]
; - Removed: *CB - Parameter von ENM\OpenConnection()
;
;
; - V# 1.2.1
; - Added: SendOnlyUsed-Mode für Protocol Transceiver
; - Added: Volle Dokumentation aller Protokol-Funktionen
; - Optimiert: Map-Nutzung in Protokol-Code
; - Optimiert: Protocol-Overhead verkleinert
; - Optimiert: Speichernutzung der Protokol-Verwaltung
; - Optimiert: Protocol-Code überarbeitet und kommentiert
; - Bugfix: Threadsicherheit der Protokol-Routinen verbessert
; - Bugfix: diverse kleine Fehler in den Protocol-Routinen
;
; - V# 1.2.0
; - Added: Protocol Builder
; - Added: Protocol Blueprint
; - Added: Protocol Transceiver
; - Optimiert: div. kleine Verbesserungen
;
; - V# 1.1.7
; - Added: Support for PB 4.6b2
; - Added: Settings-Checker um falsche Einstellugen abzufangen
; - Added: Bessere Auswertung von WSA - Fehlern [Windows only]
; - Changed: Include-Paths für OS X angepasst
; - BugFix: Allgemeiner Fehler bei der Implementierung des Pings
; - BugFix: Fehler bei der Implementation des Ping auf Nicht-Windows Systemen
; - BugFix: Fehler wenn #ENM_SN_PingTimeout auf Null gesetzt wurde
;
; - V# 1.1.6
; - Added: Support für PB 4.6b1
; - Removed: Cache\LoadFromFile(), Cache\SafeToFile()
; - BugFix: Fixed PB - Version Check
; - BugFix: Problem beim Reload und ungültigen Ping.
;
; - V# 1.1.5
; - Added: Minimalgröße für einen Cache-Eintrag einstellbar.
; - Added: Komprimierung lange nicht genutzter Einträge in einem Cache (in Cache\CleanUp())
; - Added: FastMem - Bibliothek (ASM optimierte Memory Befehle) [Internal] [CompilerSwitch]
; - Optimiert: Cache entscheided automatisch über minimale Größe eines Cache-Eintrags
; - Changed: Cache Funktionen waren versehentlich als ProcedureDLL deklariert
; - Changed: dynamische Reload-Timings per Compiler-Switch abschaltbar
; - Bugfix: Cachefile - Inkompatibilität zwischen Ascii und Unicode
;
; - V# 1.1.4
; - Optimiert: Packet-Reload-Timings beziehen nun die Ping-Zeiten mit ein
; - Added: GetClientPing() [Windows only]
; - Added: SafeNetwork deckt die Network Library nun vollständig ab
;
; - V# 1.1.3
; - Added: PrePing in ENM\OpenConnection() [Windows Only]
; - Optimiert: Cache Config Parameter vereinfacht
; - Optimiert: Cache ClearOld Procedure
; - Optimiert: Cache Verwaltungs - Overhead
; - Optimiert: Includes restrukturiert
; - Changed: CacheElement - Syntax angepasst
; - Changed: Reload-Timings verbessert
;
; - V# 1.1.2
; - Added: Cache nun voll funktionsfähig und threadsicher
; - Optimiert: Cache auf Hashtables umgestellt
; - Optimiert: JobQueue auf SingleThread-Only umgebaut
; - Bugfix: CloseConnection auf gerade aktive Verbindungen stabiler
;
; - V# 1.1.1 
; - Added: [Experimental]: Cache von ECluster portiert
; - Changed: Std - Timings auf Performance eingestellt.
; - Optimiert: Stream Code
; - BugFix: Partberechnung bei kleinen Streams
;
; - V# 1.1.0
; - Added: Stream-Managemant
; - Optimiert: Gesamtperformance und Speicherauslastung
; - Optimiert: Source aufgeräumt
; - Optimiert: EventCaller am Server benötigt weniger Mutex-Locks
; - Optimiert: EventCaller nun Performanter und Speicherschonender
; - Optimiert: RemoveServer() nun stabiler
; - Optimiert: div. kleine Verbesserungen am Code
; - Optimiert: ReceivePaket() und SendPaket() besser aufeinander abgestimmt
; - Optimiert: PaketReload nun weit effezienter
; - Optimiert: SafeNetwork, weniger Mutex-Locks
; - BugFix: PaketReload konnte Client in eine Endlosschleife führen
; - BugFix: ClientEventProcessor konnte den ClientCount unter Null bringen
; - BugFix: ClientEventProcessor konnte DeadLock erzeugen
; - BugFix: UDPWA konnte einen Eintrag doppelt anlegen
;
; - V# 1.0.4r2
; - Basiert auf Codebasis von V# 1.03
; - Added: ENM\LimitServerConnections() : Anzahl Verbindungen/Server begrenzen
; - Optimiert: kleinere Optimierungen und Bugfixes
; - Removed: Security Manager vorerst nicht implementiert
;
; - V# 1.0.4
; - Buggy/Unstable
; - Added: Security Manager
;
; - V# 1.0.3
; - Optimiert: Source aufgeräumt, Redundanzen vermindert
; - Optimiert: JobQueue Priority-Stacks entfernt
; - Optimiert: Paket-Verwaltung: Signatur verbessert
; - Optimiert: VTables werden nicht mehr dynamisch erzeugt
; - Optimiert: ClientEventProcessor - Performance verbessert
; - BugFix: UDP-Watcher im Mischbetrieb TCP/UDP
;
; - V# 1.0.2
; - XData - Verwaltung vereinfacht und Performance verbessert
; - UDP - Workaround verbessert/fehlerbereinigt
;
; - V# 1.0.1
; - UDP - Workaround
; - Anzahl der falschen Event-Calls reduziert
; - div. kleine Optimierungen am Code
; 
; - V# 1.0.0
; - Initial Release
;}
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
;{ Source - Quellen:

; ClientEvent - Disconnect:
; - http://www.purebasic.fr/english/viewtopic.php?t=42559

; Hasttable - Extension:
; - http://www.purebasic.fr/german/viewtopic.php?t=24030

; Icmp - Ping Source:
; - http://www.purebasic.fr/german/viewtopic.php?t=24004

; FastMem - Bibliothek:
; - http://www.agner.org/optimize/
;
; Windows - CloseConnection WorkAround:
; - http://www.purebasic.fr/english/viewtopic.php?t=46435

;}
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
;{ Settings:
;- Settings
; # Diverses
; Falscher Event-Filter. Bei Fehler beim Daten - Empfang testweise auf einen Wert 
; zwischen 8-128 setzen. (0=Disable)
#ENM_Sys_FalseEventFilter = 0
; Wie lange soll nach einem abgefangenen falschen Event gewartet werden?
#ENM_Sys_FalseEventFilterDelay = 0
; Wie lange soll nach einem Receive gewartet werden bis der
; Empfang wieder freigegebn wird? (Sorgt für weniger falsche Calls)
#ENM_Sys_ReceiveLockDelay = 0
; Wie lange soll gewartet werden wenn kein Event ansteht?
#ENM_Sys_NoEventDelay = 1
; Wie viele Clients werden ca. pro Server auftreten?
#ENM_Sys_ServerClientSlots = 1024
; Wie viele Verbindungen werden ca. hergestellt werden?
#ENM_Sys_ConnectionSlots = 1024
; FastMem - Bibliothek anstatt normaler Memory-Befehle verwenden?
#ENM_Sys_UseFastMem = #False;#True
; Soll versucht werden Programmierfehler zu verhindern?
#ENM_Sys_UseMistakePreventer = #True
; Limitiert die Anzahl der Verbindungen pro Anwendung (nur Client-seitig)
#ENM_Sys_LimitClientConnections = 0;250
; Asyncrones freigeben von Objekten:
#ENM_Sys_AsyncFree = #True
; CloseConnection - Workaround verwenden? (ACHTUNG: Verursacht ev. ein Memory-Leak!)
#ENM_Sys_WindowsCloseConnectionWA = #False
;
; # SafeNetwork - Einstellungen 
; WorkAround für UDP. Nur aktivieren wenn UDP benötigt wird!
#ENM_SN_UDPWorkAround = #False
; Connect/Disconnect Event aufrufen.
#ENM_SN_UDPWorkAroundFakeConnect = #True
; Nach wie vielen ms soll eine inaktive UDP - Verbindung beendet werden?
#ENM_SN_UDPWorkAroundInactiveKill = 5000
; Connection - Handles abstrahieren.
#ENM_SN_UseVId = #False
; PingTimeout für OpenNetworkConnection und GetClientPing(): senkt die
; Wartezeit bei nicht erreichbaren Servern: (< 0: Disabled, sonst Timeout Wert)
#ENM_SN_PingTimeout = 2000
; Wie lange ist ein Ping-Wert gültig bis ein neuer Ping ausgeführt wird? [ms]
#ENM_SN_PingCacheTime = 15000
; Wie lange soll gewartet werden wenn ein WSA-Block Error erkannt wird?
#ENM_SN_WSABlockDelay = 32
;
; # JobQueue - Thread Einstellungen
; Nach wie vielen ms soll ein inaktiver Thread schlafen gelegt werden? (0=Disable)
#ENM_JQ_SleepDelay = 50
; 
; # Paket Einstellungen
; Wie oft soll probiert werden fehlende Daten nachzuladen?
#ENM_Paket_MaxReloads = 5
; Wie oft soll maximal neu gesendet werten?
#ENM_Paket_MaxResends = 10
; Wie viele Ms soll zwischen den Versuchen gewartet werden?
#ENM_Paket_ReloadDelay = 1
; Wie viele Ms soll zwischen den Versuchen gewartet werden wenn nichts empfangen wurde?
#ENM_Paket_ZeroReloadDelay = 8
; Dynamisches Reload: Ping-Zeiten miteinbeziehen?
#ENM_Paket_DynamicReload = #True
; Was ist der maximale Multiplikator für den ReloadDelay?
#ENM_Paket_MaxReloadDelayMultiplicator = 3
; Wie groß ist ein Block im Empfangsbuffer? (8kB - 32kB erscheinen sinnvoll)
#ENM_Paket_ReceiveBlockSize = 24*1024
;
; # Stream Eintellungen
; Wie groß ist ein Stream-Part maximal?
#ENM_Stream_MaximumPartSize = 1024*1024
; In wie großen Schritten soll der Stream-Speicher vergößert werden?
; Sinnvoll ist ein vielfaches von #ENM_Stream_MaximumPartSize
#ENM_Stream_BufferBlockSize = 5*#ENM_Stream_MaximumPartSize
;
; # Protocol Einstellungen
; Protokol Blueprint-Mapgröße. Kann ruhig klein sein da nicht geschwindigkeitsrelevant
#ENM_ProtoBP_MapSize = 2
; Ursprungs-Blueprint im Transceiver dublizieren? (Schneller aber Speicherintensiver)
#ENM_ProtoT_DublicateBlueprint = #True

;}
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
;{ Befehls - Beschreibung:

XIncludeFile "ENM_CommandDescription.pbi"
;}
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
;- Internal
; ------------------------------------------------------------------------------------

EnableExplicit

; Event - Callback
Prototype ENM_Event_CB(Server, Client)

; Hilfsfunktionen
XIncludeFile "ENM_CommonIncludes.pbi"

; ------------------------------------------------------------------------------------
;{ Strukturen

; Struktur für die Callbacks
Structure ENM_CallBacks
  *Connect.ENM_Event_CB
  *Disconnect.ENM_Event_CB
  *Receive.ENM_Event_CB
  *ReceiveFile.ENM_Event_CB
EndStructure

Structure _ENM_ClientData_ReceiveTimeStamps
  RecFile.l ; Datei wird entfangen
  RecData.l ; Daten werden entfangen
EndStructure

; Daten eines Client an einem Server und als Connection
Structure _ENM_ClientData_ServerClient
  *ReceiveTS._ENM_ClientData_ReceiveTimeStamps
  ID.i ; Connection-ID
  SecureID.l ; "sichere ID" sorgt dafür um kopien dieser Struktur gegen Pointer auf Integrität zu prüfen
  JQ.i ; Job-Queue
  Used.a ; Struktur wird gerade benutz. Nötig für RemoveServer() und CloseConnection()
  Stream.i ; Connection befindet sich im Stream-Modus
EndStructure

; Gemeinsame Elemente für Server+Client Verwaltung
Structure _ENM_Data_ServerClient Extends _Thread_Std
  Count.i ; Anzahl Connections/Server
EndStructure

; Client an einem Server
Structure _ENM_ClientData_Server Extends _ENM_ClientData_ServerClient
  Server.i ; an welchem Server ist der Client?
EndStructure

; Connection - Structure
Structure _ENM_ClientData_Client Extends _ENM_ClientData_ServerClient
  CB.ENM_CallBacks ; Callbacks
  FakeConnectionEventSent.a ; FakeCB wurde gesendet
EndStructure

; Server-Struktur
Structure _ENM_Server
  ID.i ; Server ID
  Type.l ; Server Type
  Used.a ; Struktur wird gerade benutzt
  ConnectionLimit.i ; Wie viele Clients/Server sind erlaubt?
  Map Clients._ENM_ClientData_Server(2*#ENM_Sys_ServerClientSlots) ; Clients am Server
  CB.ENM_CallBacks ; Callbacks
EndStructure


; UDP - Disconnect Watcher
CompilerIf #ENM_SN_UDPWorkAround
  Structure _ENM_UDP_Watcher_Entry
    *Client._ENM_ClientData_Client
    SecureID.i
  EndStructure
  
  Structure _ENM_UDP_Watcher Extends _Thread_Std
    Map Clients._ENM_UDP_Watcher_Entry(2*#ENM_Sys_ServerClientSlots) 
  EndStructure
CompilerEndIf

; Server - Verwaltung
Structure _ENM_Data_Server Extends _ENM_Data_ServerClient
  Map Servers._ENM_Server() ; Server Liste
  ClientCount.i ; Client count
  CompilerIf #ENM_SN_UDPWorkAround
    UDP._ENM_UDP_Watcher
  CompilerEndIf
EndStructure

; Connection-Verwaltung
Structure _ENM_Data_Client Extends _ENM_Data_ServerClient
  Map Clients._ENM_ClientData_Client(2*#ENM_Sys_ConnectionSlots) ; Client Liste
EndStructure

; Extra-Daten
Structure _ENM_XData
  Map Connections.i(2*#ENM_Sys_ConnectionSlots)
  GMutex.i ; Schutzmutex
EndStructure

; Connection - Closer
Structure _ENM_ConCloser
  ConCloser.i ; JQ
  GMutex.i ; Schutzmutex
  Map InDeletion.i()
EndStructure

; Standard-Daten
Structure _ENM_DefaultSettings
  CB.ENM_CallBacks ; Callbacks
  CB_defined.a ; CB Definiert
EndStructure

; Daten für ENM
Structure _ENM_Data 
  *VTable ; VTable
  Type.l ; Typ (Server, Client, Client+Server)
  CompilerIf #ENM_Sys_AsyncFree
    GBCollector.i ; GarbageCollector
  CompilerEndIf
  Default._ENM_DefaultSettings ; DefaultSettings
  ConCloser._ENM_ConCloser ; Connection - Closer
  Server._ENM_Data_Server ; Server-Verwaltung
  Client._ENM_Data_Client ; Client-Verwaltung
  XData._ENM_XData ; Extra - Daten
EndStructure

; Event
Structure _ENM_Event
  *ENM._ENM_Data ; ENM-Struktur
  *CB.ENM_Event_CB ; Callback das aufgerufen wird
  *TS._ENM_ClientData_ReceiveTimeStamps ; Timestamps
  ConId.i ; Connection ID
  Server.i ; Server - ID
  Type.i ; Event - Typ
EndStructure

; Daten für den Garbage Collector
Structure _ENM_GBC_Data
  JQ.i ; JobQueue
  NC.i ; Network Connection
  ConId.i ; ConnectionID
  *ENM._ENM_Data ; ENM - Handle
EndStructure

;}
; ------------------------------------------------------------------------------------

; Zugriffs Interface

Interface _ENM_Interface
  OpenServer(Port.l, Type.l = #PB_Network_TCP)
  CloseServer(Server.i)
  LimitServerConnections(Server.i, Count.i = #Null)
  OpenConnection(ServerName$, Port.l, Type.i = #PB_Network_TCP)
  OpenProxyConnection(ServerName$, Port.l, ProxyType, ProxyHost$, ProxyPort, User$ = "", Pass$ = "", Timeout = 5000)
  CloseConnection(Connection.i, Server.i = #Null)
  SetCallbacks(Connection_Server.i, *CB.ENM_Callbacks)
  GetConnectionData(Connection.i)
  SetConnectionData(Connection.i, XData.i)
  SendPaket(Connection.i, *Mem, MemSize.i, Compression.i = #Null)
  ReceivePakets(Connection.i)
  OpenStream(Connection.i, Server.i = #Null)
  OpenCache(MaxSize = #Null)
  OpenProtocolBuilder(Name$, Version.a)
  OpenProtocolBlueprint(Blueprint$)
  OpenProtocolTransceiver(Blueprint)
  OpenAuthenticationSystem(MasterPassword$, MaximumLoginCount = 1, TokenTimeToLife = 604800)
  OpenLogger()
  OpenLoggerFileBackend(LogFile$)
EndInterface

; Zugriffs Objekt
Global ENM._ENM_Interface

; Enumerations: ClientInitialize
Enumeration
  #ENM_ToolsOnly = 0
  #ENM_Server = 2
  #ENM_Client = 1
  #ENM_Full = #ENM_Server|#ENM_Client
EndEnumeration

; Default
Enumeration
  #ENM_Default_CallBacks = 1
EndEnumeration

; Sichere Netzwerkfunktionen
XIncludeFile "SafeNetwork.pbi"
; Threadbasierte Jobverwaltung
XIncludeFile "JobQueue_MultiThread_LE_SingleThread.pbi"
; Log-System:
XIncludeFile "ENM_Logger.pbi"
; Proxy Unterstützung
XIncludeFile "ENM_ProxyConnect.pbi"
; Paketverwaltung
XIncludeFile "ENM_Pakets.pbi"
; Streamverwaltung
XIncludeFile "ENM_Stream.pbi"
; Cacheverwaltung
XIncludeFile "ENM_Cache.pbi"
; Protokol-Funktionen
XIncludeFile "ENM_ProtocolBuilder.pbi" ; Builder
XIncludeFile "ENM_ProtocolBlueprint.pbi" ; Blaupause
XIncludeFile "ENM_ProtocolTransceiver.pbi" ; Transceiver
; Authentication - System
XIncludeFile "ENM_Authenticate.pbi"

; ------------------------------------------------------------------------------------
;{ Hilfs - Proceduren/Macros

; Compiler Delay. Nur für Konstante Delays
Macro CDelay(_Delay)
  CompilerIf _Delay
    Delay(_Delay)
  CompilerEndIf
EndMacro

; sichere ID erzeugen
Macro _ENM_GenerateSecureID()
  (Random(2147483646)+1)
EndMacro

;}
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
;{ Connection - Managemant

; JobQues beenden
Procedure __ENM_QuitCon(*Quit._ENM_GBC_Data)
  With *Quit\ENM
    ; Netzwerkverbindung beenden
    If *Quit\NC
      CloseNetworkConnection(*Quit\NC)
    EndIf
    
    ; JobQueue beenden
    If *Quit\JQ
      ; JQ beenden
      JQ_WaitUntilFinish(*Quit\JQ)
      JQ_DeInit(*Quit\JQ)
    EndIf
    
    ; InDeletion austragen
    LockMutex(\ConCloser\GMutex)
    DeleteMapElement(\ConCloser\InDeletion(), Str(*Quit\ConId))
    UnlockMutex(\ConCloser\GMutex)
  EndWith
  
  FreeMemory(*Quit)
EndProcedure

; Beendet eine JobQueue parallel und ggf. auf die dazugehörige NetworkConnection und Maps
Procedure _ENM_QuitCon(*ENM._ENM_Data, JQ, NC, ConId)
  Protected *Quit._ENM_GBC_Data = AllocateMemory(SizeOf(_ENM_GBC_Data))
  
  With *ENM
    *Quit\JQ = JQ ; JobQueue
    *Quit\NC = NC ; Network Connection
    *Quit\ConId = ConId ; Network Connection
    *Quit\ENM = *ENM ; ENM - Handle
    
    ; InDeletion setzen
    LockMutex(\ConCloser\GMutex)
    AddMapElement(\ConCloser\InDeletion(), Str(ConId))
    UnlockMutex(\ConCloser\GMutex)
    
    ; Garbage cleanen ;)
    JQ_Add(*ENM\ConCloser\ConCloser, @__ENM_QuitCon(), *Quit)
  EndWith
EndProcedure

Macro _ENM_InitCon(_Con, ConId)
  ; Warten bis der Eintrag sicher ausgetragen ist...
  LockMutex(*ENM\ConCloser\GMutex)
  While FindMapElement(*ENM\ConCloser\InDeletion(), Str(ConId))
    UnlockMutex(*ENM\ConCloser\GMutex)
    Delay(1)
    LockMutex(*ENM\ConCloser\GMutex)
  Wend
  UnlockMutex(*ENM\ConCloser\GMutex)
  ; JobQueue initialisieren
  _Con#\ID = ConId
  _Con#\JQ = JQ_Init()
  _Con#\ReceiveTS = AllocateMemory(SizeOf(_ENM_ClientData_ReceiveTimeStamps))
  _Con#\ReceiveTS\RecData = ElapsedMilliseconds() ; Timestamps zum Empfangen laden
  _Con#\ReceiveTS\RecFile = _Con#\ReceiveTS\RecData
  _Con#\SecureID = _ENM_GenerateSecureID() ; SecureID erzeugen
EndMacro
;}
; ------------------------------------------------------------------------------------

Declare _ENM_SendClientEvent(*ENM._ENM_Data, *Client._ENM_ClientData_Client, *CB, Type)
Declare _ENM_SendServerEvent(*ENM._ENM_Data, *Client._ENM_ClientData_Server, *CB, Type)


; ENM - Proceduren

; Xtra Daten freigeben
Procedure _ENM_DeleteConnectionData(*ENM._ENM_Data, Connection)
  With *ENM\XData
    LockMutex(\GMutex)
    DeleteMapElement(\Connections(), Str(Connection))
    UnlockMutex(\GMutex)
  EndWith
EndProcedure

; Xtra Daten lesen
Procedure _ENM_GetConnectionData(*ENM._ENM_Data, Connection)
  Protected XData.i
  With *ENM\XData
    LockMutex(\GMutex)
    XData = \Connections(Str(Connection))
    UnlockMutex(\GMutex)
  EndWith
  ProcedureReturn XData
EndProcedure

; Xtra Daten setzen
Procedure _ENM_SetConnectionData(*ENM._ENM_Data, Connection, XData.i)
  With *ENM\XData
    LockMutex(\GMutex)
    \Connections(Str(Connection)) = XData
    UnlockMutex(\GMutex)
  EndWith
EndProcedure

; Macro zur Erstellung von verschiedenen Open-Proceduren (Normal, Proxy, ...)
Macro _ENM_OpenConnectionX(_CmdLine, _ConCommand)
  Procedure _CmdLine
    Protected Con, *ClientD._ENM_ClientData_Client
    
    ; Client initialisiert?
    If Not *ENM\Type & #ENM_Client
      ProcedureReturn #Null
    EndIf
    
    CompilerIf #ENM_Sys_LimitClientConnections
      If *ENM\Client\Count >= #ENM_Sys_LimitClientConnections ; Wir sind am Limit
        ProcedureReturn #Null
      EndIf
    CompilerEndIf
    
    Con = _ConCommand
    If Con ; Connection ok?
      LockMutex(*ENM\Client\GMutex)
      *ENM\Client\Count + 1
      Repeat
        *ClientD = AddMapElement(*ENM\Client\Clients(), Str(Con))
        If Not *ClientD : Continue : EndIf
        _ENM_InitCon(*ClientD, Con) ; Client Initialisieren
        ; STD - Callbacks eintragen falls vorhanden
        If *ENM\Default\CB_defined 
          CopyStructure(@*ENM\Default\CB, @*ClientD\CB, ENM_CallBacks)
          ; Fake-Event absenden
          If *ClientD\CB\Connect
            _ENM_SendClientEvent(*ENM, *ClientD, *ClientD\CB\Connect, #PB_NetworkEvent_Connect)
            *ClientD\FakeConnectionEventSent = #True ; Damit das Event nicht 2x ausgelöst wird
          EndIf
        EndIf
      Until *ClientD
      UnlockMutex(*ENM\Client\GMutex)
    EndIf
    
    ProcedureReturn Con
  EndProcedure
EndMacro

; Normale OpenConnection - Procedure
_ENM_OpenConnectionX(_ENM_OpenConnection(*ENM._ENM_Data, ServerName$, Port.l, Type = #PB_Network_TCP), OpenNetworkConnection(ServerName$, Port, Type))
; Proxy Connection - Procedure
_ENM_OpenConnectionX(_ENM_OpenProxyConnection(*ENM._ENM_Data, ServerName$, Port.l, proxy_type, proxy_host$, proxy_port, User$ = "", Pass$ = "", timeout = 5000), _ENM_ProxyConnect(ServerName$, Port.l, proxy_type, proxy_host$, proxy_port, User$, Pass$, timeout))

Procedure _ENM_CloseConnection(*ENM._ENM_Data, Connection.i, Server = #Null)
  Protected *ClientD._ENM_ClientData_Server, SecureID
  Protected *ClientDex._ENM_ClientData_Client, *ServerD._ENM_Server
  
  If Server ; Server Connection beenden
    ; Server initialisiert?
    If Not *ENM\Type & #ENM_Server
      ProcedureReturn #Null
    EndIf
    
    With *ENM\Server
      LockMutex(\GMutex)
      If \ClientCount ; Gibt es Clients?
        *ServerD = FindMapElement(\Servers(), Str(Server))
        If  *ServerD; Gültiger Server?
          *ClientD = FindMapElement(*ServerD\Clients(), Str(Connection))
          If *ClientD ; Gültiger Client?
            ; Nötige Daten laden
            SecureID = *ClientD\SecureID ; Secure ID laden
            *ClientD\SecureID = #Null ; Und Nullsetzen
            
            If *ClientD\Used ; Wird benutzt, warten...
              UnlockMutex(\GMutex)
              While *ClientD\Used
                CDelay(1)
              Wend
              LockMutex(\GMutex)
            EndIf
            
            ; JobQueue beenden, Connection schließen, ...
            If SecureID
              ; Fake-Event absenden.
              ; Xtra-Data wird nun im Event_MiddleLayer gelöscht.
              _ENM_SendServerEvent(*ENM, *ClientD, *ServerD\CB\Disconnect, #PB_NetworkEvent_Disconnect)
              ; Stream-Modus beenden
              If *ClientD\Stream
                SignalSemaphore(*ClientD\Stream)
              EndIf
              ; Verbindung beenden
              _ENM_QuitCon(*ENM, *ClientD\JQ, *ClientD\ID, *ClientD\ID)
              ; Map-Eintrag löschen
              DeleteMapElement(*ServerD\Clients(), Str(Connection))
              ; Ein Client weniger
              \ClientCount - 1
            EndIf
          EndIf
        EndIf
      EndIf
      UnlockMutex(\GMutex)
    EndWith
    
  Else ; Client Connection beenden
    
    ; Client initialisiert?
    If Not *ENM\Type & #ENM_Client
      ProcedureReturn #Null
    EndIf
    
    With *ENM\Client
      LockMutex(\GMutex)
      If \Count ; Gibt es Clients?
        *ClientD = FindMapElement(\Clients(), Str(Connection))
        If *ClientD ; Client gefunden?
          ; Nötige Daten laden
          *ClientDex = *ClientD ; Für mehr Funktionen
          SecureID = *ClientD\SecureID ; SecureID sichern ...
          *ClientD\SecureID = #Null ; ... und rücksetzen
          
          If *ClientD\Used ; Wird benutzt, warten...
            UnlockMutex(\GMutex)
            While *ClientD\Used
              CDelay(1)
            Wend
            LockMutex(\GMutex)
          EndIf
          
          ; JobQueue beenden, Connection schließen, ...
          If SecureID ; Sind wir eh die einzigen die beenden...?
            ; Fake-Event absenden
            ; XData löschen wird nun im Event Middle-Layer ausgeführt
            _ENM_SendClientEvent(*ENM, *ClientD, *ClientDex\CB\Disconnect, #PB_NetworkEvent_Disconnect)
             ; Stream-Modus beenden falls nötig
            If *ClientD\Stream
              SignalSemaphore(*ClientD\Stream)
            EndIf
            ; Alles aufräumen
            _ENM_QuitCon(*ENM, *ClientD\JQ, Connection, Connection)
            ; Map-Eintrag löschen
            DeleteMapElement(\Clients(), Str(Connection))
            ; ein Client weniger
            \Count - 1
          EndIf
        EndIf    
      EndIf
      UnlockMutex(\GMutex)
    EndWith
    
  EndIf

EndProcedure

; Server hinzufügen
Procedure _ENM_OpenServer(*ENM._ENM_Data, Port.l, Type.l)
  Protected *ServerD._ENM_Server
  
  ; Server initialisiert?
  If Not *ENM\Type & #ENM_Server
    ProcedureReturn #Null
  EndIf
  
  ; Server erstellen
  Protected Server = CreateNetworkServer(#PB_Any, Port, Type)
  
  With *ENM\Server
    If Server
      ;Server eintragen
      LockMutex(\GMutex)
      \Count + 1 ; Ein Server mehr :)
      *ServerD = AddMapElement(\Servers(), Str(Server))
      *ServerD\ID = Server
      *ServerD\Type = Type
      
      ; STD - Callbacks eintragen falls vorhanden
      If *ENM\Default\CB_defined
        CopyStructure(@*ENM\Default\CB, @*ServerD\CB, ENM_CallBacks) 
      EndIf
        
      UnlockMutex(\GMutex)
    EndIf
  EndWith
  
  ProcedureReturn Server
EndProcedure

; Server entfernen
Procedure _ENM_CloseServer(*ENM._ENM_Data, Server.i)  
  Protected *ServerD._ENM_Server, *ClientD._ENM_ClientData_Server
  ; Server initialisiert?
  If Not *ENM\Type & #ENM_Server
    ProcedureReturn #Null
  EndIf
  
  With *ENM\Server
    
    LockMutex(\GMutex)  
    *ServerD = FindMapElement(\Servers(), Str(Server))
    If *ServerD
      ; Ein Server weniger...
      \Count-1
       ; Strukturen werden benutzt, warten ...
      If *ServerD\Used
        UnlockMutex(\GMutex) 
        While *ServerD\Used
          CDelay(1)
        Wend
        LockMutex(\GMutex) 
      EndIf
      
      ForEach *ServerD\Clients() ; Alle Clients beenden
        *ClientD = *ServerD\Clients()
        If *ClientD\SecureID ; Wurde der Client eh noch nicht beendet?
          *ClientD\SecureID = #Null
          ; Fake-Event
          ; Xtra-Data wird nun im Event_MiddleLayer gelöscht.
           _ENM_SendServerEvent(*ENM, *ClientD, *ServerD\CB\Disconnect, #PB_NetworkEvent_Disconnect)
          ; List-Einträge müssen nicht einzeln gelöscht werden, da wird das nun alles auf einmal machen
          If *ClientD\Stream ; Stream-Modus beenden
            SignalSemaphore(*ClientD\Stream)
          EndIf
          ; JQ beenden
          _ENM_QuitCon(*ENM, *ClientD\JQ, #Null, *ClientD\ID)
        EndIf
      Next
      
      ; Warten bis die Verbindungen beendet wurden
      JQ_WaitUntilFinish(*ENM\ConCloser\ConCloser, #False)
      
      ; Map-Entry löschen
      ClearStructure(\Servers(Str(Server)), _ENM_Server)
      DeleteMapElement(\Servers(), Str(Server))
      
      ; Server beenden
      CloseNetworkServer(Server)
      
      UnlockMutex(\GMutex)
    Else
      UnlockMutex(\GMutex)
    EndIf
  EndWith
EndProcedure

; Anzahl verbindungen pro Server limitieren
Procedure _ENM_LimitServerConnections(*ENM._ENM_Data, Server, Count = #Null)
  ; Server initialisiert?
  If Not *ENM\Type & #ENM_Server
    ProcedureReturn #Null
  EndIf
  
  With *ENM\Server
    LockMutex(\GMutex)
    If FindMapElement(\Servers(), Str(Server)) ; Gibt es den Server?
      \Servers()\ConnectionLimit = Count
    EndIf
    UnlockMutex(\GMutex)
  EndWith
  
EndProcedure

; Callbacks einstellen
Procedure _ENM_SetCallbacks(*ENM._ENM_Data, Connection.i, *CB.ENM_Callbacks)
  Protected *ClientD._ENM_ClientData_Client
  Protected *ServerD._ENM_Server
  Protected CBFound = #False
  
  ; Defaults setzen
  If Connection = #ENM_Default_CallBacks
    CBFound = #True
    If *CB ; Callbacks kopieren
      CopyStructure(*CB, @*ENM\Default\CB, ENM_CallBacks)
      *ENM\Default\CB_defined = #True
    Else
      *ENM\Default\CB_defined = #False
    EndIf
  EndIf
  
  ; Client Callbacks ...
  With *ENM\Client
    If (*ENM\Type & #ENM_Client) And (Not CBFound)
      LockMutex(\GMutex)
      *ClientD = FindMapElement(\Clients(), Str(Connection))
      If *ClientD
        ; Callbacks einstellen
        If *CB ; Callbacks kopieren
          CopyStructure(*CB, @*ClientD\CB, ENM_CallBacks)
          ; Fake Event absenden
          If *ClientD\CB\Connect And (Not *ClientD\FakeConnectionEventSent)
             _ENM_SendClientEvent(*ENM, *ClientD, *ClientD\CB\Connect, #PB_NetworkEvent_Connect)
            *ClientD\FakeConnectionEventSent = #True ; Event darf kein 2. mal ausgelöst werden!
          EndIf
        EndIf
        CBFound = #True ; Callback gefunden
      EndIf
      UnlockMutex(\GMutex)
    EndIf
  EndWith
  
  ; Server Callbacks ...
  With *ENM\Server
    If (*ENM\Type & #ENM_Server) And (Not CBFound)
      LockMutex(\GMutex)
      *ServerD = FindMapElement(\Servers(), Str(Connection))
      If *ServerD
        ; Callbacks einstellen
        If *CB
          CopyStructure(*CB, @*ServerD\CB, ENM_CallBacks)
        EndIf
      EndIf
      UnlockMutex(\GMutex)
    EndIf
  EndWith
EndProcedure

; zwischenebene für Eventprozessoren
Procedure _ENM_EventMiddleLayer(*ENME._ENM_Event)
  With *ENME
    \CB(\Server, \ConId)
    
    ; CB - Sperren Rücksetzen   
    Select \Type
      Case #PB_NetworkEvent_File
        \TS\RecFile = ElapsedMilliseconds() + (#ENM_Sys_ReceiveLockDelay)
        
      Case #PB_NetworkEvent_Data
        \TS\RecData = ElapsedMilliseconds() + (#ENM_Sys_ReceiveLockDelay)
        
      Case #PB_NetworkEvent_Disconnect
        ; Aufräumarbeiten:
        ; XData Map-Element löschen
        _ENM_DeleteConnectionData(\ENM, \ConId)
        ; TimeStamps löschen
        FreeMemory(\TS)
        
    EndSelect
    
    FreeMemory(*ENME)
  EndWith
EndProcedure

; Event Generieren
Macro _ENM_GenerateEvent(_Server)
  *ENMEvent = AllocateMemory(SizeOf(_ENM_Event)) ; Speicher für das Event
  *ENMEvent\CB = *CB  ; Callback
  *ENMEvent\ENM = *ENM ; ENM-Handle
  *ENMEvent\ConId = *Client\Id ; Connection - ID
  *ENMEvent\TS = *Client\ReceiveTS ; TimeStamps für Receive-Events
  *ENMEvent\Server = _Server ; Server falls nötig
  *ENMEvent\Type = Type ; Event Typ
EndMacro

; Queue nach Absetzen des Disconnect-Events stoppen.
Macro _ENM_EventTriggers()
  If Type = #PB_NetworkEvent_Disconnect
    JQ_StopQueue(*Client\JQ) ; Queue Stoppen
  EndIf
EndMacro

; Event an einen Client senden
Procedure _ENM_SendClientEvent(*ENM._ENM_Data, *Client._ENM_ClientData_Client, *CB, Type)
  Protected *ENMEvent._ENM_Event

  If *CB
    ; Event erstellen
    _ENM_GenerateEvent(#Null)
    
    ; Auf Event-Stack legen
    If Not JQ_Add(*Client\JQ, @_ENM_EventMiddleLayer(), *ENMEvent)
      FreeMemory(*ENMEvent)  
    Else
      ; Trigger auslösen
      _ENM_EventTriggers()
    EndIf
  EndIf
EndProcedure

; Event absetzen
Procedure _ENM_SendServerEvent(*ENM._ENM_Data, *Client._ENM_ClientData_Server, *CB, Type)
  Protected *ENMEvent._ENM_Event
  
  If *CB
    ; Event erstellen
    _ENM_GenerateEvent(*Client\Server)
    
    ; Auf Event-Stack legen
    If Not JQ_Add(*Client\JQ, @_ENM_EventMiddleLayer(), *ENMEvent)
      FreeMemory(*ENMEvent)  
    Else
      ; Trigger auslösen
      _ENM_EventTriggers()
    EndIf
  EndIf
EndProcedure

; Events abfragen und absenden
Procedure _ENM_ServerEventProcessor(*ENM._ENM_Data)
  Protected Event, Client, Server, *ServerD._ENM_Server, *ClientD._ENM_ClientData_Server
  Protected RecFile, RecData
  
  With *ENM\Server
    
    Repeat
      Thread_Std_CheckStop(*ENM\Server) ; Aus Stop prüfen
            
      If Not \Count ; Keine Verbindungen? => überspringen und CPU sparen :D
        CDelay(#ENM_Sys_NoEventDelay)
        Continue
      EndIf
      
      ; ServerEvent abfragen
      Event = NetworkServerEvent()
      
      If Event ; Netzwerk Event da
        ; Server abfragen
        Server = EventServer()
        ; Client abfragen
        Client = EventClient()
        
        ; Callbacks laden
        LockMutex(\GMutex)  
        *ServerD = FindMapElement(\Servers(), Str(Server)) 
        If *ServerD ; Server OK?
          
          ; Limit erreicht, Client sofort beenden!
          If *ServerD\ConnectionLimit And (Event = #PB_NetworkEvent_Connect) And (*ServerD\ConnectionLimit <= \ClientCount)
            CloseNetworkConnection(Client)
            UnlockMutex(\GMutex)
            Continue
          EndIf
            
          ; Client sicher anlegen
          *ClientD = FindMapElement(*ServerD\Clients(), Str(Client))
          If Not *ClientD
            *ClientD = AddMapElement(*ServerD\Clients(), Str(Client))
            *ClientD\ID = Client
          EndIf
          
          ;{ UDP - Workaround
          CompilerIf #ENM_SN_UDPWorkAround
            If (*ServerD\Type = #PB_Network_UDP) And (Not *ClientD\SecureID) And (Not *ClientD\Server) And (Event <> #PB_NetworkEvent_Connect)
              ; Wah.. Blödes WA! Alles muss man doppelt machen...
              ; Limit erreicht, Client sofort beenden!
              If *ServerD\ConnectionLimit And (*ServerD\ConnectionLimit <= MapSize(*ServerD\Clients()))
                DeleteMapElement(*ServerD\Clients())
                CloseNetworkConnection(Client)
                UnlockMutex(\GMutex)
                Continue
              EndIf
            
              ; Temporären UDP-Client erstellen
              *ClientD = *ServerD\Clients()
              _ENM_InitCon(*ClientD) ; Client initialisieren
              *ClientD\Server = Server ; Server speichern
              \Count + 1
              
              ; Client im UDP - Watcher eintragen
              ; Unlock hier nötig sonst gibts nen Deathlock.
              UnlockMutex(\GMutex)
              LockMutex(\UDP\GMutex)
              \UDP\Clients(Str(Client))\Client = *ClientD
              \UDP\Clients(Str(Client))\SecureID = *ClientD\SecureID
              UnlockMutex(\UDP\GMutex)
              LockMutex(\GMutex)
              
              ; Connect - Event für UDP vortäuschen
              CompilerIf #ENM_SN_UDPWorkAroundFakeConnect
                UnlockMutex(\GMutex)
                _ENM_SendServerEvent(*ENM,  *ClientD, *ServerD\CB\Connect, Event)
                LockMutex(\GMutex)
              CompilerEndIf
            EndIf
            ;}
            
          CompilerElse
            If (Not *ClientD\SecureID) And (Event <> #PB_NetworkEvent_Connect) ; Fehler
              DeleteMapElement(*ServerD\Clients(), Str(Client))
              UnlockMutex(\GMutex) 
              CDelay(#ENM_Sys_FalseEventFilterDelay)
              Continue
            EndIf
          CompilerEndIf
          
          ; solle für weniger falsche Event-Calls sorgen
          If *ClientD\ReceiveTS ; Gibt es schon Timestamps?
            RecFile = *ClientD\ReceiveTS\RecFile
            RecData = *ClientD\ReceiveTS\RecData
          EndIf
          
          ; Strukturen sind in Benutzung...
          *ServerD\Used = #True
          *ClientD\Used = #True
        Else
          UnlockMutex(\GMutex)
          CDelay(#ENM_Sys_NoEventDelay)
          Continue
        EndIf
        UnlockMutex(\GMutex)
        
        ; Events verarbeiten und Absenden
        Select Event 
          Case #PB_NetworkEvent_Data
            ; Darf das CB aufgerufen werden?
            If RecData And ((ElapsedMilliseconds()-RecData) >= #ENM_Sys_FalseEventFilter)
              *ClientD\ReceiveTS\RecData = #Null
              If *ClientD\Stream ; Stream-Modus
                SignalSemaphore(*ClientD\Stream) ; Stream signalisieren
              Else
                _ENM_SendServerEvent(*ENM, *ClientD, *ServerD\CB\Receive, Event)
              EndIf
            Else; Wartet um mehr falsche Events abzufangen
              CDelay(#ENM_Sys_FalseEventFilterDelay)
            EndIf
            
          Case #PB_NetworkEvent_File
            ; Darf das CB aufgerufen werden?
            If RecFile And ((ElapsedMilliseconds()-RecFile) >= #ENM_Sys_FalseEventFilter)
              *ClientD\ReceiveTS\RecFile = #Null ; Falsche Events abfangen
              _ENM_SendServerEvent(*ENM, *ClientD, *ServerD\CB\ReceiveFile, Event)
            Else ; Wartet um mehr falsche Events abzufangen
              CDelay(#ENM_Sys_FalseEventFilterDelay)
            EndIf
            
          Case #PB_NetworkEvent_Connect
            ; Client registrieren
            LockMutex(\GMutex)
            _ENM_InitCon(*ClientD, Client) ; Client Initialisieren
            *ClientD\Server = Server ; Server speichern
            \ClientCount + 1 ; Client-Zähler erhöhen
            UnlockMutex(\GMutex)
            
            _ENM_SendServerEvent(*ENM, *ClientD, *ServerD\CB\Connect, Event)
            
          Case #PB_NetworkEvent_Disconnect
            *ServerD\Used = #False
            *ClientD\Used = #False
            
            ; Client austragen
            LockMutex(\GMutex)
            ; Jetzt sind wir fertig...
            ; Ist der Client noch gültig?
            If *ClientD\SecureID
              *ClientD\SecureID = #Null ; SecureID nullsetzen
              ; Nur 1. mal absetzen!!!
              _ENM_SendServerEvent(*ENM, *ClientD, *ServerD\CB\Disconnect, Event)
               ; Stream-Modus beenden
              If *ClientD\Stream
                SignalSemaphore(*ClientD\Stream)
              EndIf
              ; JQ freigeben
              _ENM_QuitCon(*ENM, *ClientD\JQ, #Null, Client)
              ; MapEintrag löschen
              DeleteMapElement(*ServerD\Clients(), Str(Client))
              ; Ein Client weniger
              \ClientCount - 1
            EndIf
            UnlockMutex(\GMutex)

        EndSelect
        ; Strukturen werden nicht mehr benutzt
        If Event <> #PB_NetworkEvent_Disconnect
          *ServerD\Used = #False
          *ClientD\Used = #False
        EndIf
        
      Else
        CDelay(#ENM_Sys_NoEventDelay) ; Wir wollen ja nicht den kompletten Prozessor verheizen ...
      EndIf
    ForEver
    
  EndWith
EndProcedure

; Events abfragen und absenden
Procedure _ENM_ClientEventProcessor(*ENM._ENM_Data)
  Protected Event, Client, *ClientD._ENM_ClientData_Client
  Protected RecFile, RecData
  
  With *ENM\Client
    
    Repeat
      Thread_Std_CheckStop(*ENM\Client)
      
      If Not \Count  ; Keine Verbindungen? => überspringen und CPU sparen :D
        CDelay(#ENM_Sys_NoEventDelay)
        Continue
      EndIf
      
      LockMutex(\GMutex)
      ForEach \Clients() ; Alle Clients durchgehen
        *ClientD = \Clients() ; Client - Struktrur
        Client = *ClientD\ID ; Client ID
        
        ; Wird gerade gelöscht...
        If Not *ClientD\SecureID
          Continue
        EndIf
        
        ; NetzwerkEvent abfragen
        Event = NetworkClientEvent(Client)
        
        If Event ; Netzwerk Event da
          
          ; Struktur wird benutzt
          *ClientD\Used = #True
          UnlockMutex(\GMutex)
          
          ; solle für weniger falsche Event-Calls sorgen
          If *ClientD\ReceiveTS ; Nur zur Sicherheit ...
            RecFile = *ClientD\ReceiveTS\RecFile
            RecData = *ClientD\ReceiveTS\RecData
          EndIf
          
          ; Events verarbeiten und absenden
          Select Event 
            Case #PB_NetworkEvent_Data
              ; Darf das CB aufgerufen werden?
              If RecData And ((ElapsedMilliseconds()-RecData) >= #ENM_Sys_FalseEventFilter)
                *ClientD\ReceiveTS\RecData = #Null
                If *ClientD\Stream ; Stream-Modus
                  SignalSemaphore(*ClientD\Stream) ; Stream signalisieren
                Else
                  _ENM_SendClientEvent(*ENM, *ClientD, *ClientD\CB\Receive, Event)
                EndIf
              Else ; Wartet um mehr falsche Events abzufangen
                CDelay(#ENM_Sys_FalseEventFilterDelay)
              EndIf
              
            Case #PB_NetworkEvent_File
              ; Darf das CB aufgerufen werden?
              If RecFile And ((ElapsedMilliseconds()-RecFile) >= #ENM_Sys_FalseEventFilter)
                *ClientD\ReceiveTS\RecFile = #Null
                _ENM_SendClientEvent(*ENM, *ClientD, *ClientD\CB\ReceiveFile, Event)
              Else ; Wartet um mehr falsche Events abzufangen
                CDelay(#ENM_Sys_FalseEventFilterDelay)
              EndIf
              
            Case #PB_NetworkEvent_Disconnect
              ; Client austragen
              LockMutex(\GMutex)
              ; Client wird nicht mehr benutzt
              *ClientD\Used = #False
              ; Ist der Client noch gültig?
              If *ClientD\SecureID
                *ClientD\SecureID = #Null ; SecureID Nullsetzen
                ; Nur 1. mal absetzen!!!
                _ENM_SendClientEvent(*ENM, *ClientD, *ClientD\CB\Disconnect, Event)
                 ; Stream-Modus beenden
                If *ClientD\Stream
                  SignalSemaphore(*ClientD\Stream)
                EndIf
                ; JQ freigeben
                _ENM_QuitCon(*ENM, *ClientD\JQ, #Null, Client)
                ; MapEintrag löschen
                DeleteMapElement(\Clients(), Str(Client))
                ; Ein Client weniger
                \Count - 1
              EndIf
              UnlockMutex(\GMutex)
              
          EndSelect
          ; Used zurücksetzen
          If Event <> #PB_NetworkEvent_Disconnect
            *ClientD\Used = #False
          EndIf
          
          LockMutex(\GMutex)
        EndIf   
      Next
      UnlockMutex(\GMutex)
      
      ; Wichtig! Einkoppeln von neuen Verbindungen klappt so viel schneller!
      CDelay(#ENM_Sys_NoEventDelay)
    ForEver
    
  EndWith
EndProcedure

;{ UDP - WA Watcher
CompilerIf #ENM_SN_UDPWorkAround
  Procedure _ENM_UDPWatcher(*ENM._ENM_Data)
    Protected Time, *ServerD._ENM_Server, *ClientD._ENM_ClientData_Server
    Protected Client, SecureID
    
    With *ENM\Server\UDP
      Repeat
        ; Beendet
        Thread_Std_CheckStop(*ENM\Server\UDP)
        
        LockMutex(\GMutex)
        ForEach \Clients()
          Time = ElapsedMilliseconds()
          *ClientD = \Clients()\Client
          SecureID = \Clients()\SecureID
          Client = *ClientD\ID
          
          ; Seit MaxTime inaktiv
          If (*ClientD\SecureID <> SecureID) Or ((*ClientD\RecData And ((Time-*ClientD\RecData) >= #ENM_SN_UDPWorkAroundInactiveKill)) And (*ClientD\RecFile And ((Time-*ClientD\RecFile) >= #ENM_SN_UDPWorkAroundInactiveKill)))
            If (*ClientD\SecureID <> SecureID)
              DeleteMapElement(\Clients(), Str(Client))
              Continue
            EndIf
            
            LockMutex(*ENM\Server\GMutex)
            *ServerD = @*ENM\Server\Servers(Str(*ClientD\Server))
            
            ; Disconnect - Fake
            CompilerIf #ENM_SN_UDPWorkAroundFakeConnect
              If *ServerD\CB
                UnlockMutex(*ENM\Server\GMutex)
                _ENM_SendServerEvent(*ENM, *ServerD, Client, *ServerD\CB\Disconnect, #PB_NetworkEvent_Disconnect)
                LockMutex(*ENM\Server\GMutex)
              EndIf
            CompilerEndIf
            
           ; Secure ID laden
           SecureID = *ClientD\SecureID
           *ClientD\SecureID = #Null
           
            If *ClientD\Used ; Wird benutzt, warten...
              UnlockMutex(*ENM\Server\GMutex)
              While *ClientD\Used
                CDelay(1)
              Wend
              LockMutex(*ENM\Server\GMutex)
            EndIf
            
            ; Client austragen
            DeleteMapElement(\Clients(), Str(Client))
            
            ; Connection beenden
            If SecureID
              If *ClientD\Stream ; Stream-Modus beenden
                SignalSemaphore(*ClientD\Stream)
              EndIf
              _ENM_QuitCon(*ENM, *ClientD\JQ, Client, *ClientD\Server, Client)
              *ENM\Server\ClientCount - 1 ; Ein Client weniger
            EndIf
            
            UnlockMutex(*ENM\Server\GMutex)  
          EndIf 
        Next
        UnlockMutex(\GMutex)
        
        CDelay(5)
      ForEver
    EndWith
    
  EndProcedure
CompilerEndIf
;}

; ------------------------------------------------------------------------------------
;- Public
; ------------------------------------------------------------------------------------

; ENM Initialisieren
Procedure ENM_Init(Type = #ENM_Full)
  Protected *ENM._ENM_Data = AllocateMemory(SizeOf(_ENM_Data))
  Shared ENM
  
  InitializeStructure(*ENM, _ENM_Data)
  
  With *ENM
    ; VTable anlegen
    \VTable = ?_ENM_VTable
    
    ; Server starten
    \Type = Type
    If \Type & #ENM_Server
      CompilerIf #ENM_SN_UDPWorkAround
        Thread_Std_Init(\Server\UDP, _ENM_UDPWatcher(), *ENM)
      CompilerEndIf
      Thread_Std_Init(\Server, _ENM_ServerEventProcessor(), *ENM)
    EndIf
    
    ; Client erstellen
    If \Type & #ENM_Client
      Thread_Std_Init(\Client, _ENM_ClientEventProcessor(), *ENM)
    EndIf
    
    ; Garbage Collectoren
    CompilerIf #ENM_Sys_AsyncFree
      \GBCollector = JQ_Init()
    CompilerEndIf
    
    ; Connection-Closer
    \ConCloser\ConCloser = JQ_Init()
    \ConCloser\GMutex = CreateMutex()
    ; XData Mutex
    \XData\GMutex = CreateMutex()
  EndWith
  
  ; Eintragen...
  PokeI(@ENM, *ENM)
  
EndProcedure

Procedure ENM_DeInit()
  Protected *ENM._ENM_Data = PeekI(@ENM)

  With *ENM
    If \Type & #ENM_Server
      ; Event - Verwaltung beenden
      Thread_Std_Stop(\Server)
      JQ_WaitUntilFinish(\ConCloser\ConCloser, #False)

      ; Alle Server beenden
      If \Server\Count
        ForEach \Server\Servers()
          _ENM_CloseServer(*ENM, \Server\Servers()\ID)
        Next
      EndIf

      ; UDP-WA beenden
      CompilerIf #ENM_SN_UDPWorkAround
        Thread_Std_Stop(\Server\UDP)
        Thread_Std_DeInit(\Server\UDP)
      CompilerEndIf
    EndIf
    
    ; Clients beenden
    If \Type & #ENM_Client
      ; Alle Clients beenden
      
      ; Event - Verwaltung beenden
      Thread_Std_Stop(\Client)
      JQ_WaitUntilFinish(\ConCloser\ConCloser, #False)

      ; Alle Connections beenden
      If \Client\Count
        ForEach \Client\Clients()
          _ENM_CloseConnection(*ENM, \Client\Clients()\ID)
        Next
      EndIf
      
    EndIf

    ; Garbage freigeben
    ; Connection-Closer
    JQ_WaitUntilFinish(\ConCloser\ConCloser)
    JQ_DeInit(\ConCloser\ConCloser)
    FreeMutex(\ConCloser\GMutex)
    
    ; XData Mutex
    FreeMutex(\XData\GMutex)
    
    CompilerIf #ENM_Sys_AsyncFree
      ; Object-Remover
      JQ_WaitUntilFinish(\GBCollector)
      JQ_DeInit(\GBCollector)
    CompilerEndIf
    ; Muss nach dem GBC sein, da er diese Mutexen verwenden kann!!
    If \Type & #ENM_Client
      ; Mutex freigeben
      Thread_Std_DeInit(\Client)
    EndIf
   
    If \Type & #ENM_Server
      ; Mutex freigeben
      Thread_Std_DeInit(\Server)
    EndIf
    
    ; ENM-Pointer leeren
    PokeI(@ENM, #Null)
    
    ; Struktur freigeben
    ClearStructure(*ENM, _ENM_Data)
    FreeMemory(*ENM)
  EndWith
  
EndProcedure

; ------------------------------------------------------------------------------------

CompilerIf #ENM_Sys_UseMistakePreventer
  ; Fängt ab hier alle falschen direkten Procedure-Aufrufe ab
  XIncludeFile "ENM_MistakePreventer.pbi"
CompilerEndIf


; VTable für das Interface
DataSection
  _ENM_VTable:
  Data.i @_ENM_OpenServer()
  Data.i @_ENM_CloseServer()
  Data.i @_ENM_LimitServerConnections()
  Data.i @_ENM_OpenConnection()
  Data.i @_ENM_OpenProxyConnection()
  Data.i @_ENM_CloseConnection()
  Data.i @_ENM_SetCallbacks()
  Data.i @_ENM_GetConnectionData()
  Data.i @_ENM_SetConnectionData()
  ; aus ENM_Pakets.pbi
  Data.i @_ENM_SendPaket()
  Data.i @_ENM_ReceivePaket()
  ; Aus ENM_Stream.pbi
  Data.i @_ENM_Stream_OpenStream()
  ; Aus ENM_Cache.pbi
  Data.i @_ENM_Cache_New()
  ; Aus ENM_ProtocolBuilder.pbi
  Data.i @_ENM_Protocol_NewBuilder()
  ; Aus ENM_ProtocolBlueprint.pbi
  Data.i @_ENM_Protocol_NewBlueprint()
  ; Aus ENM_ProtocolTransceiver.pbi
  Data.i @_ENM_Protocol_NewTransceiver()
  ; Aus ENM_Authenticate.pbi
  Data.i @_ENM_Auth_New()
  ; Aus ENM_Logger.pbi
  Data.i @_ENM_Logger_New()
  ; Aus ENM_LoggerFileBackend.pbi
  Data.i @_ENM_LoggerBackend_NewFileLogger()
EndDataSection



; IDE Options = PureBasic 4.60 Beta 3 (Windows - x64)
; CursorPosition = 58
; Folding = AY---+-
; EnableXP
; CPU = 1