﻿; ------------------------------------------------------------------------------------
; Befehlsbeschreibung für den Easy Network Manager
; ------------------------------------------------------------------------------------
;
;-------------------------------------------------------------------------------------
;- ENM Interface
;-------------------------------------------------------------------------------------
;
; ####################################################################################
;- ENM\OpenServer(Port.l, Type.l = #PB_Network_TCP)
;
; Parameter:    Port   : Port auf dem der Server auf Verbindungen warten soll
;               Type.l : TCP/UDP - Server. Default ist TCP
;
; Description:  Startet einen Server
;
; Return:       Server-Handle wenn Erfolg, sonst #Null
; ####################################################################################
;
; ####################################################################################
;- ENM\CloseServer(Server)
;
; Parameter:    Server : Sever-ID
;
; Description:  Beendet einen Server
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENM\LimitServerConnections(Server, Count = #Null)
;
; Parameter:    Server : Sever-ID
;               Count  : Anzahl Verbindungen auf die Begrenzt wird (#Null = Disable)
;
; Description:  Maximale Anzahl der Verbindungen die an einem Server aktiv sein
;               dürfen.
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENM\OpenConnection(ServerName$, Port.l, Type = #PB_Network_TCP)
;
; Parameter:    ServerName$ : HostName/IP des Servers zu dem verbunden werden soll
;               Port        : Port auf den der Server läuft
;               Type        : TCP/UDP - Verbindung. Default ist TCP
;
; Description:  Stellt eine Netzwerkverbindung zu einem Server her
;
; Return:       Connection-ID
; ####################################################################################
;
; ####################################################################################
;- ENM\OpenProxyConnection(ServerName$, Port.l, ProxyType, ProxyHost$, ProxyPort, User$ = "", Pass$ = "", Timeout = 5000)
;
; Parameter:    ServerName$ : HostName/IP des Servers zu dem verbunden werden soll
;               Port        : Port auf den der Server läuft
;               ProxyType   : Proxy-Typ (#ENM_Proxy_SOCKS4/#ENM_Proxy_SOCKS5)
;               ProxyHost$  : HostName/IP des Proxy-Servers
;               ProxyPort   : Port auf dem der Proxy läuft
;               User$       : UserName für Authentifizierung am Proxy
;               Pass$       : Passwort zur Authentifizierung am Proxy [SOCKS5]
;               Timeout     : Timeour für die Proxy Verbindung
;
; Description:  Stellt eine Netzwerkverbindung zu einem Server über einen Proxy her
;
; Return:       Connection-ID
; ####################################################################################
;
; ####################################################################################
;- ENM\CloseConnection(Connection, Server = #Null)
;
; Parameter:    Connection : Connection-ID
;               Server     : #Null wenn es eine Verbing ist die zu einem Server
;                            verbindet, Server-ID wenn sie zu einem Client verbindet.
;
; Description:  Schließt eine Netzwerk-Verbindung
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENM\SetCallbacks(Connection, *CB.ENM_Callbacks)
;
; Parameter:    Connection : Server-ID, Connection-ID
;               *CB               : Pointer auf Callback-Struktur
;
; Description:  Setzt die Callbacks für eine Verbindung/einen Server. Wenn als 
;               Connection #ENM_Default_Callbacks verwendet wird, werden diese Callbacks
;               als Default für alle kommenden Verbindungen gesetzt.
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENM\GetConnectionData(Connection)
;
; Parameter:    Connection : Connection-ID
;
; Description:  Liest die Extra-Daten für eine Verbindung
;
; Return:       Xtra-Data
; ####################################################################################
;
; ####################################################################################
;- ENM\SetConnectionData(Connection, XData)
;
; Parameter:    Connection : Connection-ID
;               XData      : Extra-Daten
;
; Description:  Schreibt die Extra-Daten für eine Verbindung
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENM\SendPaket(Connection, *Mem, MemSize, Compression = #Null)
;
; Parameter:    Server      : Connection: Connection-ID
;               *Mem        : Speicher der gesendet werden soll
;               MemSize     : Größe des Speichers
;               Compression : Kompressionstufe für das Paket (0-9)
;
; Description:  Sendet ein Paket an einen Server/Client
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENM\ReceivePakets(Connection)
;
; Parameter:    Server: Connection: Connection-ID
;
; Description:  Empfäng Pakete von einer Verbindung
;
; Return:       ENM_Pakets Interface
; ####################################################################################
;
; ####################################################################################
;- ENM\OpenStream(Connection, Server = #Null)
;
; Parameter:    Connection : Connection-ID
;               Server     : #Null wenn es eine Verbing ist die zu einem Server verbindet,
;                            Server-ID wenn sie zu einem Client verbindet.
;
; Description:  Öffnet einen Daten-Stream
;
; Return:       ENM_Stream Interface
; ####################################################################################
;
; ####################################################################################
;- ENM\OpenCache(MaxSize = #Null)
;
; Parameter:    MaxSize : Maximalgröße des Cache, #Null = Unlimitiert
;
; Description:  Öffnet einen neuen Memory-Cache
;
; Return:       ENM_Cache Interface
; ####################################################################################
;
; ####################################################################################
;- ENM\OpenProtocolBuilder(Name$, Version.a)
;
; Parameter:    Name$   : Name des Protokolls
;               Version : Version des Protokolls (0-255)
;
; Description:  Öffnet einen neuen Protokol-Builder zum erstellen eines Paketes
;
; Return:       ENM_Protcol_Builder Interface
; ####################################################################################
;
; ####################################################################################
;- ENM\OpenProtocolBlueprint(Blueprint$)
;
; Parameter:    Blueprint$ : Protokol-Blueprint
;
; Description:  Öffnet einen neuen Protokol-Blueprint-Token zum erstellen von
;               Protokol-Transceivern
;
; Return:       ENM_Protcol_Blueprint Interface
; ####################################################################################
;
; ####################################################################################
;- ENM\OpenProtocolTransceiver(Blueprint)
;
; Parameter:    Blueprint : ENM_Protocol_Blueprint Token
;
; Description:  Öffnet einen neuen Protokol-Tranceiver zum Übertragen und Empfangen
;               der Daten eines Protokols
;
; Return:       ENM_Protcol_Transceiver Interface
; ####################################################################################
;
; ####################################################################################
;- ENM\OpenAuthenticationSystem(MasterPassword$, MaximumLoginCount = 1, TokenTimeToLife = 604800)
;
; Parameter:    MasterPassword$   : Masterpassword$ für den Admin-Zugriff
;               MaximumLoginCount : Default-Wert für die maximale Anzahl
;                                   gleichzeitiger Logins eines Clients
;               TokenTimeToLife   : Default-Wert für die maximale Lebensdauer eines
;                                   Zugriffs-Token
;
; Description:  Erstellt ein neues Authentication-System zum Verwalten von User-
;               Anmeldungen. Daten, ...
;
; Return:       ENM_AuthenticationSystem Interface
; ####################################################################################
;
; ####################################################################################
;- ENM\OpenLogger()
;
; Parameter:    -
;
; Description:  Öffnet einen neuen Logger um Ereignisse und Aktionen aufzuzeichnen 
;
; Return:       ENM_Logger Interface
; ####################################################################################
;
; ####################################################################################
;- ENM\OpenLoggerFileBackend(LogFile$)
;
; Parameter:    LogFile$: Pfad zur Log-Datei
;
; Description:  Öffnet ein neues Logger-Backend für einen Logger
;
; Return:       ENM_Logger_Backend Interface falls Erfolg, sonst #False
; ####################################################################################
;
;-------------------------------------------------------------------------------------
;- ENM_Pakets Interface
;-------------------------------------------------------------------------------------
;
; ####################################################################################
;- ENMP\GetNext()
;
; Parameter:    -
;
; Description:  Nächstes Paket laden
;
; Return:       #True falls vorhanden, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMP\GetMem()
;
; Parameter:    -
;
; Description:  SpeicherAddresse des aktuellen Paketes
;
; Return:       Speicheradresse
; ####################################################################################
;
; ####################################################################################
;- ENMP\GetMemSize()
;
; Parameter:    -
;
; Description:  SpeicherGröße des aktuellen Paketes
;
; Return:       SpeicherGröße
; ####################################################################################
;
; ####################################################################################
;- ENMP\Count()
;
; Parameter:    -
;
; Description:  Anzahl der ampfangenen Pakete
;
; Return:       Anzahl Pakete
; ####################################################################################
;
; ####################################################################################
;- ENMP\Free()
;
; Parameter:    -
;
; Description:  Pakete und Interface freigeben
;
; Return:       #Null
; ####################################################################################
; ------------------------------------------------------------------------------------
;
;-------------------------------------------------------------------------------------
;- ENM_Stream Interface
;-------------------------------------------------------------------------------------
;
; ####################################################################################
;- ENMS\WaitForEvent()
;
; Parameter:    -
;
; Description:  Wartet auf ein Stream-Event
;
; Return:       Stream-Event
; ####################################################################################
;
;
; ####################################################################################
;- ENMS\SendPaket(*Mem, MemSize, Compression = #Null)
;
; Parameter:    *Mem        : Speicher der gesendet werden soll
;               MemSize     : Größe des Speichers
;               Compression : Kompressionstufe für den Speicher (0-9)
;
; Description:  Sende Speicher über den Stream
;
; Return:       #True wenn Erfolg, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMS\R_GetMem()
;
; Parameter:    -
;
; Description:  SpeicherAddresse des gesamten Streams
;
; Return:       Speicheradresse
; ####################################################################################
;
; ####################################################################################
;- ENMS\R_GetReceivedMemSize()
;
; Parameter:    -
;
; Description:  SpeicherGröße des gesamten verfügbaren/emfangenen Streams
;
; Return:       SpeicherGröße
; ####################################################################################
;
; ####################################################################################
;- ENMS\R_GetTotalMemSize()
;
; Parameter:    -
;
; Description:  SpeicherGröße des gesamten Streams
;
; Return:       SpeicherGröße
; ####################################################################################
;
; ####################################################################################
;- ENMS\R_GetCurMem()
;
; Parameter:    -
;
; Description:  SpeicherAddresse des aktuellen Stream-Pakets
;
; Return:       Speicheradresse
; ####################################################################################
;
; ####################################################################################
;- ENMS\R_GetCurMemSize()
;
; Parameter:    -
;
; Description:  SpeicherGröße des aktuellen Stream-Pakets
;
; Return:       SpeicherGröße
; ####################################################################################
;
; ####################################################################################
;- ENMS\S_GetTotalMemSize()
;
; Parameter:    -
;
; Description:  SpeicherGröße des gesamten Streams der gesendet wird (Sende-Seitig)
;
; Return:       SpeicherGröße
; ####################################################################################
;
; ####################################################################################
;- ENMS\S_GetSentMemSize()
;
; Parameter:    -
;
; Description:  SpeicherGröße des bereits gesendeten Streams (Sende-Seitig)
;
; Return:       SpeicherGröße
; ####################################################################################
;
; ####################################################################################
;- ENMS\Close()
;
; Parameter:    -
;
; Description:  Löst ein #ENM_Stream_Closed Event an der Gegenseite aus
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMS\Reset()
;
; Parameter:    -
;
; Description:  Setzt den Speicherbuffer manuell zurück. Wir sonst bei
;               #ENM_Stream_Receive_FirstPart und #ENM_Stream_Receive_SinglePart
;               ausgelöst
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMS\Free()
;
; Parameter:    -
;
; Description:  Gibt das Stream-Objekt frei
;
; Return:       #Null
; ####################################################################################
;
;-------------------------------------------------------------------------------------
;- ENM_Cache Interface
;-------------------------------------------------------------------------------------
;
; ####################################################################################
;- ENMC\Add(Name$, *Mem, MemSize, MaxAge = #Null, CopyMem = #True)
;
; Parameter:    Name$   : Name für den Cache - Eintrag
;               *Mem    : Speicher der gecached werden soll
;               MemSize : Größe des Speichers
;               MaxAge  : Maximale Gültigkeit des Eintrages in Sekunden
;               CopyMem : Soll der Speicher kopiert werden oder nur der Pointer kopier?
;
; Description:  Fügt einen Eintrag zum Cache hinzu
;
; Return:       #True wenn Erfolg, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMC\Get(Name$)
;
; Parameter:    Name$ : Name des Cache - Eintrags
;
; Description:  Läd einen Eintrag aus dem Cache
;
; Return:       ENM_CacheEntry - Interface wenn Erfolg, sonst #Null
; ####################################################################################
;
; ####################################################################################
;- ENMC\Flush()
;
; Parameter:    -
;
; Description:  Löscht alle Einträge in einem Cache
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMC\Free()
;
; Parameter:    -
;
; Description:  Gibt den Cache frei
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMC\GetSize()
;
; Parameter:    -
;
; Description:  Aktuelle Größe des Cache
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMC\CleanUp()
;
; Parameter:    -
;
; Description:  Löscht veraltete Einträge manuell aus dem Cache
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMC\Config(MaxSize = #Null, MinimumEntrySize = 1*1024,  CompressTimeLimit = 60, CompressLevel = 9)
;
; Parameter:    MaxSize           : Maximalgröße des Cache
;               MinimumEntrySize  : Minimalgröße eines Entrages 0 = deaktiviert
;               CompressTimeLimit : Ab wie vielen Sekunden Nichtverwendung soll ein
;                                   Eintrag im Cache bei der CleanUp() Procedure 
;                                   komprimiert werden? 0 = deaktiviert
;               CompressLevel:      Wie stark soll ein inaktiver Eintrag komprimiert
;                                   werden? 0 = deaktiviert.
;
; Description:  Rekonfiguriert den Cache. Aufruf ohne Parameter: Standard Config
;
; Return:       #Null
; ####################################################################################
;
;-------------------------------------------------------------------------------------
;- ENM_CacheEntry Interface
;-------------------------------------------------------------------------------------
;
; ####################################################################################
;- ENMCE\GetMem()
;
; Parameter:    -
;
; Description:  Holt den Speicher des Cache Eintrags
;
; Return:       Eintrags Speicher
; ####################################################################################
;
; ####################################################################################
;- ENMCE\GetMemSize()
;
; Parameter:    -
;
; Description:  Holt die Speichergröße des Cache Eintrags
;
; Return:       Eintrags Speichergröße
; ####################################################################################
;
; ####################################################################################
;- ENMCE\Release()
;
; Parameter:    -
;
; Description:  Markiert den Eintrag als freigegeben und gibt das Objekt frei
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMCE\Delete()
;
; Parameter:    -
;
; Description:  Löscht den Cache-Eintrag und gibt das Objekt frei
;
; Return:       #Null
; ####################################################################################
;
;-------------------------------------------------------------------------------------
;- ENM_Protcol_Builder Interface
;-------------------------------------------------------------------------------------
;
; ####################################################################################
;- ENMPB\AddByte(Name$, DefaultValue.a = #Null)
;
; Parameter:    Name$ : Name des Eintrages
;               DefaultValue: Default Wert
;
; Description:  Legt einen neuen Byte - Eintrag im Protokol an
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMPB\AddLong(Name$, DefaultValue.l = #Null)
;
; Parameter:    Name$ : Name des Eintrages
;               DefaultValue: Default Wert
;
; Description:  Legt einen neuen Long - Eintrag im Protokol an
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMPB\AddQuad(Name$, DefaultValue.q = #Null)
;
; Parameter:    Name$ : Name des Eintrages
;               DefaultValue: Default Wert
;
; Description:  Legt einen neuen Quad - Eintrag im Protokol an
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMPB\AddFloat(Name$, DefaultValue.f = #Null)
;
; Parameter:    Name$ : Name des Eintrages
;               DefaultValue: Default Wert
;
; Description:  Legt einen neuen Float - Eintrag im Protokol an
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMPB\AddDouble(Name$, DefaultValue.d = #Null)
;
; Parameter:    Name$ : Name des Eintrages
;               DefaultValue: Default Wert
;
; Description:  Legt einen neuen Double - Eintrag im Protokol an
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMPB\AddString(Name$, DefaultValue.s = #Null)
;
; Parameter:    Name$ : Name des Eintrages
;               DefaultValue: Default Wert
;
; Description:  Legt einen neuen Byte - Eintrag im Protokol an
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMPB\AddMem(Name$)
;
; Parameter:    Name$ : Name des Eintrages
;
; Description:  Legt einen neuen Memory - Eintrag im Protokol an
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMPB\ExportBlueprint.s()
;
; Parameter:    -
;
; Description:  Export den Blueprint als String
;
; Return:       Blueprint
; ####################################################################################
;
; ####################################################################################
;- ENMPB\Free()
;
; Parameter:    -
;
; Description:  Gibt das Builder-Objekt frei
;
; Return:       #Null
; ####################################################################################
;
;-------------------------------------------------------------------------------------
;- ENM_Protcol_Blueprint Interface
;-------------------------------------------------------------------------------------
;
; ####################################################################################
;- ENMPBP\Free()
;
; Parameter:    -
;
; Description:  Gibt das Blueprint-Objekt frei
;
; Return:       #Null
; ####################################################################################
;
;-------------------------------------------------------------------------------------
;- ENM_Protcol_Transceiver Interface
;-------------------------------------------------------------------------------------
;
; ####################################################################################
;- ENMPT\T_SetByte(Name$, Value.a)
;
; Parameter:    Name$ : Name des Eintrages
;               Value : Wert der gesetzt wird
;
; Description:  Setzt den Protokol-Eintrag Name$ auf den Wert Value
;
; Return:       #True falls Ok, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMPT\T_SetLong(Name$, Value.l)
;
; Parameter:    Name$ : Name des Eintrages
;               Value : Wert der gesetzt wird
;
; Description:  Setzt den Protokol-Eintrag Name$ auf den Wert Value
;
; Return:       #True falls Ok, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMPT\T_SetQuad(Name$, Value.q)
;
; Parameter:    Name$ : Name des Eintrages
;               Value : Wert der gesetzt wird
;
; Description:  Setzt den Protokol-Eintrag Name$ auf den Wert Value
;
; Return:       #True falls Ok, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMPT\T_SetFloat(Name$, Value.f)
;
; Parameter:    Name$ : Name des Eintrages
;               Value : Wert der gesetzt wird
;
; Description:  Setzt den Protokol-Eintrag Name$ auf den Wert Value
;
; Return:       #True falls Ok, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMPT\T_SetDouble(Name$, Value.d)
;
; Parameter:    Name$ : Name des Eintrages
;               Value : Wert der gesetzt wird
;
; Description:  Setzt den Protokol-Eintrag Name$ auf den Wert Value
;
; Return:       #True falls Ok, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMPT\T_SetString(Name$, Value.s)
;
; Parameter:    Name$ : Name des Eintrages
;               Value : Wert der gesetzt wird
;
; Description:  Setzt den Protokol-Eintrag Name$ auf den Wert Value
;
; Return:       #True falls Ok, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMPT\T_SetMem(Name$, *Mem, MemSize)
;
; Parameter:    Name$   : Name des Eintrages
;               *Mem    : Speicher-Adresse
;               MemSize : Speicher-Größe
;
; Description:  Setzt den Protokol-Eintrag Name$ auf den Wert *Mem
;
; Return:       #True falls Ok, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMPT\T_Export(*MemSize.Integer, UsedOnly = #True)
;
; Parameter:    *MemSize : Zeiger auf eine Integer-Variable in die die
;                          Speichergröße geschrieben wird.
;               UsedOnly : Send-Only-Used Optimierung aktivieren, sollte man so lassen
;
; Description:  Exportiert die gesetzten Protokol-Daten in einen Speicher-Block der
;               mit ENMPT\R_Import() wieder geladen werden kann.
;
; Return:       Speicheradresse
; ####################################################################################
;
; ####################################################################################
;- ENMPT\T_Reset(FullReset = #False)
;
; Parameter:    FullReset: Volles Reset, Speicherschonender aber langsamer.
;                          Kein Unterschied im Verhalten des Objekts.
;
; Description:  Setzt den Transmitter-Teil auf die default Werte zurück
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMPT\R_GetByte.a(Name$)
;
; Parameter:    Name$ : Name des Eintrages
;
; Description:  Läd den Wert eine Protokol-Eintrages
;
; Return:       Eintrags-Wert
; ####################################################################################
;
; ####################################################################################
;- ENMPT\R_GetLong.l(Name$)
;
; Parameter:    Name$ : Name des Eintrages
;
; Description:  Läd den Wert eine Protokol-Eintrages
;
; Return:       Eintrags-Wert
; ####################################################################################
;
; ####################################################################################
;- ENMPT\R_GetQuad.q(Name$)
;
; Parameter:    Name$ : Name des Eintrages
;
; Description:  Läd den Wert eine Protokol-Eintrages
;
; Return:       Eintrags-Wert
; ####################################################################################
;
; ####################################################################################
;- ENMPT\R_GetFloat.f(Name$)
;
; Parameter:    Name$ : Name des Eintrages
;
; Description:  Läd den Wert eine Protokol-Eintrages
;
; Return:       Eintrags-Wert
; ####################################################################################
;
; ####################################################################################
;- ENMPT\R_GetDouble.d(Name$)
;
; Parameter:    Name$ : Name des Eintrages
;
; Description:  Läd den Wert eine Protokol-Eintrages
;
; Return:       Eintrags-Wert
; ####################################################################################
;
; ####################################################################################
;- ENMPT\R_GetString.s(Name$)
;
; Parameter:    Name$ : Name des Eintrages
;
; Description:  Läd den Wert eine Protokol-Eintrages
;
; Return:       Eintrags-Wert
; ####################################################################################
;
; ####################################################################################
;- ENMPT\R_GetMem(Name$, *MemSize.Integer)
;
; Parameter:    Name$    : Name des Eintrages
;               *MemSize : Zeiger auf eine Integer-Variable in die die
;                          Speichergröße geschrieben wird.
;
; Description:  Läd den Wert eine Protokol-Eintrages
;
; Return:       Eintrags-Wert
; ####################################################################################
;
; ####################################################################################
;- ENMPT\R_Import(*Mem, MemSize)
;
; Parameter:    *Mem    : Speicher-Adresse
;               MemSize : Speicher-Größe
;
; Description:  Läd ProtokollInformationen aus einem Speicherblock der mit
;               ENMPT\T_Export() erstellt wurde.
;
; Return:       #True falls Erfolg, sonst Fehler
; ####################################################################################
;
; ####################################################################################
;- ENMPT\R_Reset(FullReset = #False)
;
; Parameter:    FullReset: Volles Reset, Speicherschonender aber langsamer.
;                          Kein Unterschied im Verhalten des Objekts.
;
; Description:  Setzt den Receiver-Teil auf die default Werte zurück
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMPT\Free()
;
; Parameter:    -
;
; Description:  Gibt das Transceiver-Objekt frei
;
; Return:       #Null
; ####################################################################################
;
;-------------------------------------------------------------------------------------
;- ENM_AuthenticationSystem Interface
;-------------------------------------------------------------------------------------
;
; ####################################################################################
;- ENMAS\AddUser(UserName$, Password$, Permissions = #ENM_Auth_User)
;
; Parameter:    UserName$   : Benutzername
;               Password$   : Passwort, kann auch bereits gehasht sein
;               Permissions : Berechtigungsstufe für den Uer
;
; Description:  Legt einen neuen User im System an
;
; Return:       #True falls Erfolg, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMAS\Login(UserName$, PasswordHash$, *LoginResult.Integer = #Null)
;
; Parameter:    UserName$     : Benutzername
;               PasswordHash$ : PasswortHash
;               *LoginResult  : Genaues Login-Ergebiss/Fehlermeldungen
;
; Description:  Loggt einen User im System ein
;
; Return:       ENM_Authentication_System_User Interface falls Erfolg, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMAS\LoginPerToken(AccessToken$, *LoginResult.Integer = #Null)
;
; Parameter:    AccessToken$ : Zugriffstoken
;               *LoginResult : Genaues Login-Ergebiss/Fehlermeldungen
;
; Description:  Loggt einen User via Zugriffs-Token im System ein
;
; Return:       ENM_Authentication_System_User Interface falls Erfolg, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMAS\Exporter(Exporter.ENM_Auth_ExportImportCallback)
;
; Parameter:    Exporter : Pointer auf ein Export-Callback
;
; Description:  Ruft das Export-CB für jeden User im System auf
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMAS\Importer(Importer.ENM_Auth_Importer)
;
; Parameter:    Importer : Pointer auf ein Import-Callback
;
; Description:  Ruft das Import-CB auf das jeden User wieder ins System einträgt
;
; Return:       #True falls Erfolg, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMAS\Panic()
;
; Parameter:    -
;
; Description:  Keine Logins mehr möglich, alle Sessions werden ungültig!
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMAS\Free()
;
; Parameter:    -
;
; Description:  Gibt das Authentication-Object frei
;
; Return:       #Null
; ####################################################################################
;
;-------------------------------------------------------------------------------------
;- ENM_AuthenticationSystem_User Interface
;-------------------------------------------------------------------------------------
;
; ####################################################################################
;- ENMASU\IsSessionValid()
;
; Parameter:    -
;
; Description:  Überprüft ob die der Benutzer noch gültig eingeloggt ist
;
; Return:       #True falls Erfolg, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMASU\GenerateAccessToken.s()
;
; Parameter:    -
;
; Description:  Erstellt einen zeitliche begenzten Zugriffs-Token für den Client
;
; Return:       Access-Token$
; ####################################################################################
;
; ####################################################################################
;- ENMASU\DeleteAccessToken()
;
; Parameter:    -
;
; Description:  Löscht den Zugriffstoken für den Client
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMASU\GetData()
;
; Parameter:    -
;
; Description:  Läd die User-spezifischen Daten
;
; Return:       Benutzer - Data
; ####################################################################################
;
; ####################################################################################
;- ENMASU\SetData(Data)
;
; Parameter:    Data : Die Daten die gesetzt werden sollen
;
; Description:  Setzt die User-spezifischen Daten
;
; Return:       #True falls Erfolg, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMASU\GetName.s()
;
; Parameter:    -
;
; Description:  Ermittelt den Benutzer-Name
;
; Return:       Benutzer-Name$
; ####################################################################################
;
; ####################################################################################
;- ENMASU\GetPermissions()
;
; Parameter:    -
;
; Description:  Läd die Berechtigungen für den jeweiligen Benutzer
;
; Return:       Benutzer - Permissions
; ####################################################################################
;
; ####################################################################################
;- ENMASU\Logout(KeepToken = #True)
;
; Parameter:    KeepToken : Eventuell generierter Zugriffs-Token bleibt gültig
;
; Description:  Logt den Benutzer aus dem System aus
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENMASU\GetMasterAccess(MasterPasswordHash$)
;
; Parameter:    MasterPasswordHash$ : Hash des Master-Passwortes
;
; Description:  Fordert Master(Admin)-Zugriff an
;
; Return:       #True falls Erolg, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMASU\IsMaster()
;
; Parameter:    -
;
; Description:  Überprüft ob der Benutzer Master-Zugriff hat
;
; Return:       #True falls Erolg, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMASU\LockUser(UserName$, Lock = #True)
;
; Parameter:    UserName$ : Benutzer der gesperrt werden soll
;               Lock      : #True = sperren, #False = entsperren
;
; Description:  Sperrt/Entsperrt einen Benutzer. Man kann sich selbst nicht sperren!
;               Benötigt Master-Zugriff!
;
; Return:       #True falls Erolg, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMASU\DeleteUser(UserName$)
;
; Parameter:    UserName$ : Benutzer der gelöscht werden soll
;
; Description:  Löscht einen Benutzer. Man kann sich selbst nicht löschen!
;               Benötigt Master-Zugriff!
;
; Return:       #True falls Erolg, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMASU\SetPermissions(UserName$, Permissions)
;
; Parameter:    UserName$   : BenutzerName
;               Permissions : Berechtigungen die gesetzt werden sollen
;
; Description:  Setzt die Zugriffsrechte für einen Benutzer
;               Benötigt Master-Zugriff!
;
; Return:       #True falls Erolg, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMASU\SetMaximumLoginCount(UserName$, MaximumLoginCount)
;
; Parameter:    UserName$         : BenutzerName
;               MaximumLoginCount : Maximale Anzahl gleichzeitiger Logins
;
; Description:  Wie oft darf sich ein User gleichzeitig einloggen?
;               Benötigt Master-Zugriff!
;
; Return:       #True falls Erolg, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMASU\SetTokenTimeToLife(UserName$, TokenTimeToLife)
;
; Parameter:    UserName$       : BenutzerName
;               TokenTimeToLife : Wie lange ist ein Zugriffstoken gültig?
;
; Description:  Setzt die gültigkeitsdauer eines Zugriffstoken
;               Benötigt Master-Zugriff!
;
; Return:       #True falls Erolg, sonst #False
; ####################################################################################
;
; ####################################################################################
;- ENMASU\ForceLogin(UserName$)
;
; Parameter:    UserName$ : BenutzerName
;
; Description:  Loggt sich ohne jede Überprüfung in einen Benutzer ein
;               Benötigt Master-Zugriff!
;
; Return:       ENM_AuthenticationSystem_User Interface, sonst #False
; ####################################################################################
;
;-------------------------------------------------------------------------------------
;- ENM_Logger Interface
;-------------------------------------------------------------------------------------
;
; ####################################################################################
;- ENML\AddBackend(LoggerBackend.ENM_Logger_Backend)
;
; Parameter:    LoggerBackend : Logger - Backend
;
; Description:  Fügt ein Logger-Backend zum Logger hinzu
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENML\AddBackend(Caller$, LogLevel, LogMessage$)
;
; Parameter:    Caller$     : Aufrufende Procedure/System
;               LogLevel    : Wie wichtig ist der Eintrag? 
;               LogMessage$ : Log-Meldung
; 
; Description:  Übergibt eine Meldung an den Logger, dieser meldet diese an sämtliche
;               Logger-Backends weiter
;
; Return:       #Null
; ####################################################################################
;
; ####################################################################################
;- ENML\Free()
;
; Parameter:    -
;
; Description:  Gibt das Logger-Objekt frei
;
; Return:       #Null
; ####################################################################################
;
;-------------------------------------------------------------------------------------
;- ENM_Logger_Backend Interface
;-------------------------------------------------------------------------------------
;
; ####################################################################################
;- ENMLB\Free()
;
; Parameter:    -
;
; Description:  Gibt das LoggerBackend-Objekt frei
;
; Return:       #Null
; ####################################################################################
; IDE Options = PureBasic 4.60 Beta 3 (Windows - x64)
; CursorPosition = 1165
; FirstLine = 1129
; EnableXP
; CPU = 1