﻿; ------------------------------------------------------------------------------------
; Easy Network Manager - Logger Module
; PB 4.5+ OS: Windows/Linux/MacOS Architecture: x86/x64
; Author: Alexander Aigner, cxAlex @ purebasic.fr/german
; ------------------------------------------------------------------------------------

; EnableExplicit

; Log-Level
Enumeration 1
  #Logger_Info
  #Logger_Warning
  #Logger_Error
  #Logger_Critical
EndEnumeration

; Referenz-Backend für Interface
XIncludeFile "ENM_LoggerReferenceBackend.pbi"
; File-Backend
XIncludeFile "ENM_LoggerFileBackend.pbi"

Interface ENM_Logger
  AddBackend(LoggerBackend.ENM_Logger_Backend) ; Logger Backend hinzufügen
  Log(Caller$, LogLevel, LogMessage$) ; Eintrag loggen
  Free() ; Logger freigeben
EndInterface

; Logger-Daten:
Structure _ENM_Logger_Data
  *VTable ; VTable
  *ENM._ENM_Data
  List Backends._ENM_Logger_Backend_Private() ; Backends
  BackendsMutex.i ; Backend-Schutzmutex
  JQ.i ; JobQueue
EndStructure

; Eintrags-Daten
Structure _ENM_Logger_Entry_Data
  Backend._ENM_Logger_Backend_Private
  Caller$ ; Aufrufende Instanz
  LogLevel.i ; Log-Level
  LogTime.i ; Log-Zeit
  LogMessage$ ; Meldung
EndStructure

; Middle - Layer
Procedure __ENM_Logger_MiddleLayer(*ENMLED._ENM_Logger_Entry_Data)
  With *ENMLED
    ; Backend mit Daten aufrufen
    \Backend\AddLogEntry(\Caller$, \LogLevel, \LogTime, \LogMessage$)
    ; Daten freigeben
    ClearStructure(*ENMLED, _ENM_Logger_Entry_Data)
    FreeMemory(*ENMLED)
  EndWith
EndProcedure

Procedure _ENM_Logger_New(*ENM._ENM_Data)
  Protected *ENML._ENM_Logger_Data = AllocateMemory(SizeOf(_ENM_Logger_Data))
  InitializeStructure(*ENML, _ENM_Logger_Data)
  
  With *ENML
    \ENM = *ENM
    \VTable = ?_ENM_Logger_VTable ; VTable setzen
    \JQ = JQ_Init() ; Logger-Queue
    \BackendsMutex = CreateMutex() ; Schutzmutex
  EndWith
  
  ProcedureReturn *ENML
EndProcedure

Procedure _ENM_Logger_AddBackend(*ENML._ENM_Logger_Data, LoggerBackend._ENM_Logger_Backend_Private)
  With *ENML
    ; Start Logging signalisieren:
    LoggerBackend\StartLogging()
    ; Backend am Ende eintragen
    LockMutex(\BackendsMutex)
    LastElement(\Backends()) : AddElement(\Backends()) : \Backends() = LoggerBackend
    UnlockMutex(\BackendsMutex)
  EndWith
EndProcedure

Procedure _ENM_Logger_Log(*ENML._ENM_Logger_Data, Caller$, LogLevel, LogMessage$)
  Protected RefEntry._ENM_Logger_Entry_Data
  Protected *Entry._ENM_Logger_Entry_Data
  
  ; Aufrufs-Parameter setzen
  With RefEntry
    \Caller$ = Caller$
    \LogLevel = LogLevel
    \LogTime = Date()
    \LogMessage$ = LogMessage$
  EndWith
  
  With *ENML
    LockMutex(\BackendsMutex)
    ; Für jedes backend aufrufen
    ForEach \Backends()
      ; Backend einstellen
      *Entry = AllocateMemory(SizeOf(_ENM_Logger_Entry_Data))
      InitializeStructure(*Entry, _ENM_Logger_Entry_Data)
      CopyStructure(@RefEntry, *Entry, _ENM_Logger_Entry_Data)
      *Entry\Backend = \Backends()
      ; Auf Queue pushen
      JQ_Add(\JQ, @__ENM_Logger_MiddleLayer(), *Entry)
    Next
    UnlockMutex(\BackendsMutex)
  EndWith
EndProcedure

Procedure _ENM_Logger_Free(*ENML._ENM_Logger_Data)
  With *ENML
    ; Warten bis alle Log-Aufrufe erledigt sind
    JQ_WaitUntilFinish(\JQ)
    ; Stop Logging Signalisieren:
    LockMutex(\BackendsMutex)
    ForEach \Backends()
      \Backends()\StopLogging()  
    Next
    UnlockMutex(\BackendsMutex)
    ; Daten freigeben:
    FreeMutex(\BackendsMutex)
    ClearStructure(*ENML, _ENM_Logger_Data)
    FreeMemory(*ENML)
  EndWith
EndProcedure

CompilerIf #ENM_Sys_AsyncFree
  ; Freigabe für den Async-Manager
  Procedure _ENM_Logger_Free_Async(*ENML._ENM_Logger_Data)
    With *ENML
      If ENM
        JQ_Add(\ENM\GBCollector, @_ENM_Logger_Free(), *ENML)
      Else
        _ENM_Logger_Free(*ENML)
      EndIf
    EndWith
  EndProcedure
CompilerEndIf

; VTable:
DataSection
  _ENM_Logger_VTable:
  Data.i @_ENM_Logger_AddBackend()
  Data.i @_ENM_Logger_Log()
  CompilerIf #ENM_Sys_AsyncFree
    Data.i @_ENM_Logger_Free_Async()
  CompilerElse
    Data.i @_ENM_Logger_Free()
  CompilerEndIf
EndDataSection




; IDE Options = PureBasic 4.60 Beta 3 (Windows - x64)
; CursorPosition = 13
; Folding = --
; EnableThread
; EnableXP
; CPU = 1
; EnablePurifier