﻿; ------------------------------------------------------------------------------------
; Easy Network Manager - Logger Module - Referenc Backend
; PB 4.5+ OS: Windows/Linux/MacOS Architecture: x86/x64
; Author: Alexander Aigner, cxAlex @ purebasic.fr/german
; ------------------------------------------------------------------------------------

; Level in Strings umsetzen
Procedure.s __ENM_LoggerBackend_LevelToString(Level)
  Select Level
    Case #Logger_Info          : ProcedureReturn "Info"
    Case #Logger_Warning       : ProcedureReturn "Warning"
    Case #Logger_Error         : ProcedureReturn "Error"
    Case #Logger_Critical      : ProcedureReturn "Critical"
    Default                    : ProcedureReturn "Unknown"
  EndSelect
EndProcedure

; Logger-Backend erstellen:
Procedure _ENM_LoggerBackend_NewFileLogger(*ENM, LoggerFile$)
  Protected *ENMLB._ENM_LoggerBackend_Data = AllocateMemory(SizeOf(_ENM_LoggerBackend_Data))
  
  With *ENMLB
    \VTable = ?_ENM_LoggerBackend_File_VTable
    ; Log erstellen
    \LoggerData = OpenFile(#PB_Any, LoggerFile$)
    If Not \LoggerData ; Konnte nicht erstellt werden
      FreeMemory(*ENMLB)
      *ENMLB = #Null
    Else ; ans Ende der Datei springen
      FileSeek(\LoggerData, Lof(\LoggerData))
      ; Alles direkt schreiben, kein Cache. Zur Sicherheit
      FileBuffersSize(\LoggerData, #Null)
    EndIf
  EndWith
  
  ProcedureReturn *ENMLB
EndProcedure

; Logger - Proceduren:

; Initialisieren
Procedure _ENM_LoggerBackend_File_StartLogging(*ENMLB._ENM_LoggerBackend_Data)
  With *ENMLB
    ; FileFormat schreiben
    WriteStringFormat(\LoggerData, #PB_UTF8)
    WriteStringN(\LoggerData, FormatDate("[%dd.%mm.%yyyy %hh:%ii:%ss] ", Date()) + "LOG: START LOGGING", #PB_UTF8)
    WriteStringN(\LoggerData, FormatDate("[%dd.%mm.%yyyy %hh:%ii:%ss] ", Date()) + "LOG: ENCODING: UTF8", #PB_UTF8)
    WriteStringN(\LoggerData, FormatDate("[%dd.%mm.%yyyy %hh:%ii:%ss] ", Date()) + "LOG: FORMAT VERSION: 1.0", #PB_UTF8)  
  EndWith
EndProcedure

Procedure _ENM_LoggerBackend_File_AddLogEntry(*ENMLB._ENM_LoggerBackend_Data, Caller$, LogLevel, LogTime, LogMessage$)
  Protected LogString$
  With *ENMLB
    ; Log-Eintrag schreiben und sicher flushen
    LogString$ = FormatDate("[%dd.%mm.%yyyy %hh:%ii:%ss] ", LogTime) + __ENM_LoggerBackend_LevelToString(LogLevel) + ": " + Caller$ + ": " + LogMessage$
    WriteStringN(\LoggerData, LogString$, #PB_UTF8)
  EndWith
EndProcedure

Procedure _ENM_LoggerBackend_File_StopLogging(*ENMLB._ENM_LoggerBackend_Data)
  With *ENMLB
    WriteStringN(\LoggerData, FormatDate("[%dd.%mm.%yyyy %hh:%ii:%ss] ", Date()) + "LOG: STOP LOGGING", #PB_UTF8)
    WriteStringN(\LoggerData, "", #PB_UTF8)
  EndWith
EndProcedure

Procedure _ENM_LoggerBackend_File_Free(*ENMLB._ENM_LoggerBackend_Data)
  With *ENMLB
    FlushFileBuffers(\LoggerData)
    CloseFile(\LoggerData)
    FreeMemory(*ENMLB)
  EndWith
EndProcedure

; Data Section:
DataSection
  _ENM_LoggerBackend_File_VTable:
  Data.i @_ENM_LoggerBackend_File_Free()
  Data.i @_ENM_LoggerBackend_File_StartLogging()
  Data.i @_ENM_LoggerBackend_File_AddLogEntry()
  Data.i @_ENM_LoggerBackend_File_StopLogging()
EndDataSection
; IDE Options = PureBasic 4.60 Beta 3 (Windows - x64)
; CursorPosition = 63
; FirstLine = 42
; Folding = --
; EnableXP
; CPU = 1