﻿; ------------------------------------------------------------------------------------
; Easy Network Manager - Paket Verwaltung
; PB 4.5+ OS: Windows/Linux/MacOS Architecture: x86/x64
; Author: Alexander Aigner, cxAlex @ purebasic.fr/german
; ------------------------------------------------------------------------------------

; Kontroll - Interface
Interface ENM_Pakets 
  GetNext()
  GetMem()
  GetMemSize()
  Count()
  Free()
EndInterface

; Paket Header
Structure _ENM_PaketHeader
  Sign1.a ; Prüfbyte 1
  Size.l ; Speichergröße
  Sign2.a ; Prüfbyte 2
  Sign4.a ; Prüfbyte 4
  cSize.l ; wahre Größe
  Sign3.a ; Prüfbyte 3
EndStructure

; Prüf - Macros
; Kriterien:
; - Sign1 ist eine Zufallszahl zwischen 1-255
; - Sign2 ist das Einserkomplement von Sign1. (Sign1!Sign2 = $FF)
; - Sign3 ist eine Zusammensetzung aus Sign1 und Sign2
; - Sign4 ist das Einserkomplement von Sign3 (Sign3!Sign4 = $FF)
Macro _ENM_Paket_GenerateSign(_PaketMem)
  _PaketMem#\Sign1 = Random(254) + 1
  _PaketMem#\Sign2 = _PaketMem#\Sign1!$FF
  _PaketMem#\Sign3 = (_PaketMem#\Sign1&$AA)!(_PaketMem#\Sign2&$55)
  _PaketMem#\Sign4 = _PaketMem#\Sign3!$FF
EndMacro

Macro _ENM_Paket_CheckSign(_PaketMem)
  (((_PaketMem#\Size>#Null) And (_PaketMem#\cSize>=#Null)) And ((_PaketMem#\Sign1!_PaketMem#\Sign2) = $FF) And ((_PaketMem#\Sign3!_PaketMem#\Sign4) = $FF) And (_PaketMem#\Sign3 = (_PaketMem#\Sign1&$AA)!(_PaketMem#\Sign2&$55)))
EndMacro

; Paket - Daten
Structure _ENM_Paket_Data
  *Mem
  MemSize.i
EndStructure

; Pakete - Verwaltung
Structure _ENM_Pakets_Data
  *VTable
  *ENM._ENM_Data
  List Pakets._ENM_Paket_Data()
EndStructure

; Proceduren
Procedure _ENM_Paket_GetNext(*ENMP._ENM_Pakets_Data)
  With *ENMP
    ProcedureReturn NextElement(\Pakets())
  EndWith
EndProcedure

; Proceduren
Procedure _ENM_Paket_Count(*ENMP._ENM_Pakets_Data)
  With *ENMP
    ProcedureReturn ListSize(\Pakets())
  EndWith
EndProcedure

Procedure _ENM_Paket_GetMem(*ENMP._ENM_Pakets_Data)
  With *ENMP
    ProcedureReturn \Pakets()\Mem
  EndWith
EndProcedure

Procedure _ENM_Paket_GetMemSize(*ENMP._ENM_Pakets_Data)
  With *ENMP
    ProcedureReturn \Pakets()\MemSize
  EndWith
EndProcedure

Procedure _ENM_Paket_Free(*ENMP._ENM_Pakets_Data)
  With *ENMP
    ResetList(\Pakets())
    ForEach \Pakets()
      If \Pakets()\Mem
        FreeMemory(\Pakets()\Mem)
      EndIf
    Next
    ClearStructure(*ENMP, _ENM_Pakets_Data)
    FreeMemory(*ENMP)
  EndWith
EndProcedure

CompilerIf #ENM_Sys_AsyncFree
  ; Procedure für den Async-Manager
  Procedure _ENM_Paket_Free_Async(*ENMP._ENM_Pakets_Data)
    With *ENMP
      If ENM
        JQ_Add(*ENMP\ENM\GBCollector, @_ENM_Paket_Free(), *ENMP)
      Else
        _ENM_Paket_Free(*ENMP)
      EndIf
    EndWith
  EndProcedure
CompilerEndIf

; Neues Paket
Procedure _ENM_NewPaket()
  Protected *Paket._ENM_Pakets_Data = AllocateMemory(SizeOf(_ENM_Pakets_Data))
  InitializeStructure(*Paket, _ENM_Pakets_Data)
  *Paket\VTable = ?_ENM_Paket_VTable
  ProcedureReturn *Paket
EndProcedure   

; Paket senden
Procedure _ENM_SendPaket(*ENM._ENM_Data, Connection, *Mem, MemSize, Compression = #Null)
  Protected PaketMemSize = SizeOf(_ENM_PaketHeader) + MemSize
  Protected *PaketMem._ENM_PaketHeader, Rest, GRest
  Protected packed.zlib_Info, *pMem, TotalResends
  
  If (MemSize <= #Null) Or (*Mem <= #Null) ; Fehler ...
    ProcedureReturn #Null
  EndIf
  
  ; Paket-Mem anlegen
  *PaketMem= AllocateMemory(PaketMemSize)
  
  ; Paket Header schreiben
  _ENM_Paket_GenerateSign(*PaketMem)
  *PaketMem\Size = MemSize
  
  If Not Compression
    ; Daten kopieren
    CopyMemory(*Mem, *PaketMem+SizeOf(_ENM_PaketHeader), MemSize)
  Else ; Kompression
    *pMem = zLib_compress(*Mem, MemSize, Compression, @packed)
    If packed\DataOk ; Kompression ok
      ; Header anpassen
      *PaketMem\Size = packed\MemSize
      *PaketMem\cSize = MemSize
      
      ; Speicher kopieren
      CopyMemory(*pMem,  *PaketMem + SizeOf(_ENM_PaketHeader), packed\MemSize)
      ; Neue Speichergröße einstellen
      PaketMemSize = SizeOf(_ENM_PaketHeader) + packed\MemSize
      ; Zwischenspeicher freigeben
      FreeMemory(*pMem)
      
    Else ; "Normal" kopieren
      CopyMemory(*Mem, *PaketMem+SizeOf(_ENM_PaketHeader), MemSize)
    EndIf
    
  EndIf
  ; Senden
  Repeat
    Rest = SendNetworkData(Connection, *PaketMem + GRest, PaketMemSize - GRest)
    GRest + Rest
    If GRest = PaketMemSize ; Alles gesendet
      Break
    ElseIf Rest = #Null ; Fehler!
      If TotalResends < #ENM_Paket_MaxResends
        TotalResends + 1
      Else
        Break
      EndIf
    EndIf
    Delay(#ENM_Paket_ReloadDelay)
  ForEver
  
  ; Speicher freigebn
  FreeMemory(*PaketMem)
  ProcedureReturn ((GRest = PaketMemSize) Or #Null)
EndProcedure

; Reload probieren
Macro _ENM_Paket_TryReload()
  If (nRLCount < #ENM_Paket_MaxReloads) Or Rest
    
    If Rest
      nRLCount = 0
      RLCount = 0
      CompilerIf #ENM_Paket_DynamicReload ; Dynamisches Reload?
        If #ENM_Paket_ReloadDelay < Ping ; Kleiner Ping: bringt nix!
          Delay(#ENM_Paket_ReloadDelay + Ping)
        Else
          Delay(#ENM_Paket_ReloadDelay)
        EndIf
      CompilerElse
        Delay(#ENM_Paket_ReloadDelay)
      CompilerEndIf
    Else
      If RLCount < #ENM_Paket_MaxReloadDelayMultiplicator
        RLCount + 1
      EndIf
      nRLCount + 1
      CompilerIf #ENM_Paket_DynamicReload ;Dynamisches Reload?
        If #ENM_Paket_ZeroReloadDelay*RLCount < Ping ; ; Kleiner Ping: bringt nix!
          Delay((#ENM_Paket_ZeroReloadDelay * RLCount) + Ping)
        Else
          Delay(#ENM_Paket_ZeroReloadDelay * RLCount)
        EndIf
      CompilerElse
        Delay(#ENM_Paket_ZeroReloadDelay * RLCount)
      CompilerEndIf
    EndIf
    ; Goto .... aber ich finde nichts besseres bei der Verschachtelung :P
    ; Continue haut irgendwie nicht hin ...
    Goto ___ENM_Paket_ReLoad
  Else
    Break
  EndIf
EndMacro

; Paket empfangen
Procedure _ENM_ReceivePaket(*ENM._ENM_Data, Connection)
  Protected ReceiveBlockSize, BlockCount, GesMemSize, *PaketMem._ENM_PaketHeader, Rest, Pos, nRLCount, RLCount, *MemBase
  Protected *Paket._ENM_Pakets_Data = _ENM_NewPaket(), *uMem, Ping
  
  If ConnectionID(Connection)
    ; Läd die Pingzeit des Client für den dynamischen Reload
    Ping = GetClientPing(Connection)
    
    *Paket\ENM = *ENM ; ENM speichern
    
    ReceiveBlockSize =  #ENM_Paket_ReceiveBlockSize ; Empfangsblock-größe
    BlockCount = #Null ; Anzahl Blöcke
    GesMemSize = #Null ; GesamtSpeicher-größe
    *PaketMem = #Null ; Speicher
    *MemBase = #Null  ; Speicher - Grundadresse
    
    If Ping > -1 ; Ping OK
      Repeat
        ___ENM_Paket_ReLoad:      
        Repeat ; Daten einlesen
          *MemBase = ReAllocateMemory(*MemBase, (BlockCount + 1)*ReceiveBlockSize)
          Rest = ReceiveNetworkData(Connection, *MemBase + GesMemSize, ReceiveBlockSize)
          GesMemSize + Rest
          BlockCount + 1
        Until Rest<>ReceiveBlockSize
        
        *PaketMem = *MemBase; Nötig zum freigeben + Nachladen
        *PaketMem + Pos ; Nötig Falls Nachgeladen werden muss
        
        If Not GesMemSize ; sieht nach Fehler aus ...
          Break
        ElseIf GesMemSize >= SizeOf(_ENM_PaketHeader) ; Haben wir wenigstens einen gültigen Header?
          
          While GesMemSize-Pos > 0 ; Speicher durchgehen
            If _ENM_Paket_CheckSign(*PaketMem); Prüfen ...
              If (*PaketMem\Size + Pos + SizeOf(_ENM_PaketHeader)) <= GesMemSize ; Passen die Angaben?
                
                ; Neues Paket erstellen
                AddElement(*Paket\Pakets())
                If *PaketMem\cSize ; Kompression
                  ; Daten entpacken
                  *uMem = zLib_uncompress(*PaketMem + SizeOf(_ENM_PaketHeader), *PaketMem\Size, *PaketMem\cSize)
                  If *uMem ; entpacken Ok
                    ; Paket schreiben
                    *Paket\Pakets()\MemSize = *PaketMem\cSize
                    *Paket\Pakets()\Mem = *uMem
                    
                    ;  Weiterzählen
                    Pos + *PaketMem\Size + SizeOf(_ENM_PaketHeader) ; Position anpassen
                    *PaketMem + SizeOf(_ENM_PaketHeader) + *PaketMem\Size ; Pointer anpassen
                    
                  EndIf
                Else ; keine Kompression
                  ; Paket schreiben
                  *Paket\Pakets()\MemSize = *PaketMem\Size
                  *Paket\Pakets()\Mem = AllocateMemory(*Paket\Pakets()\MemSize)
                  
                  *PaketMem + SizeOf(_ENM_PaketHeader) ; Auf Dateposition gehen
                  CopyMemory(*PaketMem, *Paket\Pakets()\Mem, *Paket\Pakets()\MemSize) ; Daten abspeichern
                  
                  ; Weiterzählen
                  *PaketMem + *Paket\Pakets()\MemSize ; Adresse Anpassen
                  Pos + *Paket\Pakets()\MemSize + SizeOf(_ENM_PaketHeader) ; Position anpassen
                EndIf
                
              Else
                _ENM_Paket_TryReload()
              EndIf
            Else
              Break ; Signatur - Error, die Daten - Integrität ist nicht mehr gegeben!
            EndIf
          Wend
        Else ; Reload probieren
          _ENM_Paket_TryReload()
        EndIf
      Until #True
    EndIf  
    
    If Not ListSize(*Paket\Pakets()) ; Nichts empfangen? Freigeben.
      ClearStructure(*Paket, _ENM_Pakets_Data)
      FreeMemory(*Paket)
      *Paket = #Null
    Else
      ResetList(*Paket\Pakets()) ; Liste auf Anfang setzen
    EndIf
    
    If *MemBase
      FreeMemory(*MemBase) ; Speicher freigebn
    EndIf
  Else
    FreeMemory(*Paket)
    *Paket = #Null
  EndIf
  ProcedureReturn *Paket ; Pakete zurückgeben
EndProcedure

; VTable:
DataSection
  _ENM_Paket_VTable:
  Data.i @_ENM_Paket_GetNext()
  Data.i @_ENM_Paket_GetMem()
  Data.i @_ENM_Paket_GetMemSize()
  Data.i @_ENM_Paket_Count()
  CompilerIf #ENM_Sys_AsyncFree
    Data.i @_ENM_Paket_Free_Async()
  CompilerElse
    Data.i @_ENM_Paket_Free()
  CompilerEndIf
EndDataSection


; IDE Options = PureBasic 4.60 Beta 3 (Windows - x64)
; CursorPosition = 101
; FirstLine = 78
; Folding = ---
; EnableXP
; CPU = 1