﻿; ------------------------------------------------------------------------------------
; Easy Network Manager - Protokol Builder
; PB 4.5+ OS: Windows/Linux/MacOS Architecture: x86/x64
; Author: Alexander Aigner, cxAlex @ purebasic.fr/german
; ------------------------------------------------------------------------------------

; Common-Funktionen
XIncludeFile "ENM_ProtocolCommons.pbi"

; ProtocolBuilder - Daten
Structure _ENM_Protocol_Builder Extends _ENM_Protocol_Default
  EntryId.a
EndStructure

; Kontroll - Interface
Interface ENM_Protocol_Builder
  ; Add - Proceduren
  AddByte(Name$, DefaultValue.a = #Null)
  AddLong(Name$, DefaultValue.l = #Null)
  AddQuad(Name$, DefaultValue.q = #Null)
  AddFloat(Name$, DefaultValue.f = #Null)
  AddDouble(Name$, DefaultValue.d = #Null)
  AddString(Name$, DefaultValue.s = #NULL$)
  AddMem(Name$)
  ; Export
  ExportBlueprint.s()
  ; Free
  Free()
EndInterface


; Neuen Builder erstellen
Procedure _ENM_Protocol_NewBuilder(*ENM._ENM_Data, Name$, Version.a)
  Protected *ENMPB._ENM_Protocol_Builder = AllocateMemory(SizeOf(_ENM_Protocol_Builder))
  InitializeStructure(*ENMPB, _ENM_Protocol_Builder)
  
  With *ENMPB
    \VTable = ?_ENM_Protocol_Builder_VTable ; VTable
    \ENM = *ENM ; ENM
    \Name$ = Name$ ; Name
    \Version = Version ; Version
  EndWith  
    
  ProcedureReturn *ENMPB
EndProcedure

; Builder freigeben
Procedure _ENM_Protocol_Builder_Free(*ENMPB._ENM_Protocol_Builder)
  With *ENMPB
    ClearStructure(*ENMPB, _ENM_Protocol_Builder)
    FreeMemory(*ENMPB)
  EndWith
EndProcedure

CompilerIf #ENM_Sys_AsyncFree
  ; Procedure für den Async-Manager
  Procedure _ENM_Protocol_Builder_Free_Async(*ENMPD._ENM_Protocol_Default)
    With *ENMPD
      If ENM
        JQ_Add(\ENM\GBCollector, @_ENM_Protocol_Builder_Free(), *ENMPD)
      Else
        _ENM_Protocol_Builder_Free(*ENMPD)
      EndIf
    EndWith
  EndProcedure
CompilerEndIf

; Einträge schreiben

; Default Set-Macro
Macro __ENM_Protocol_Builder_AddEntry_X(_Name, _Entry, _Type)
  Procedure _ENM_Protocol_Builder_AddEntry_#_Name(*ENMPB._ENM_Protocol_Builder, Name$, DefaultValue._Entry)
    Name$ = LCase(Name$)
    ; Array vergrößeren
    ReDim *ENMPB\Entrys(*ENMPB\EntryCnt) : *ENMPB\EntryCnt + 1 
    *ENMPB\Entrys(*ENMPB\EntryId)\Name$ = Name$ ; Name setzen
    *ENMPB\Entrys(*ENMPB\EntryId)\Type = _Type ; Typ setzen
    *ENMPB\Entrys(*ENMPB\EntryId)\_Entry = DefaultValue ; Defaultwert setzen
    *ENMPB\EntryId + 1 ; Entry-ID erhöhen
  EndProcedure
EndMacro

; EInträge für verschiedene Typen erstellen
__ENM_Protocol_Builder_AddEntry_X(Byte, a, #PB_Ascii)
__ENM_Protocol_Builder_AddEntry_X(Long, l, #PB_Long)
__ENM_Protocol_Builder_AddEntry_X(Quad, q, #PB_Quad)
__ENM_Protocol_Builder_AddEntry_X(Float, f, #PB_Float)
__ENM_Protocol_Builder_AddEntry_X(Double, d, #PB_Double)
__ENM_Protocol_Builder_AddEntry_X(String, s, #PB_String)

Procedure _ENM_Protocol_Builder_AddEntry_Mem(*ENMPB._ENM_Protocol_Builder, Name$)
  With *ENMPB
    Name$ = LCase(Name$)
    ; Array vergrößeren
    ReDim *ENMPB\Entrys(*ENMPB\EntryCnt) : *ENMPB\EntryCnt + 1 
    \Entrys(\EntryId)\Name$ = Name$
    \Entrys(\EntryId)\Type = #PB_Mem
    \EntryId + 1
  EndWith
EndProcedure

; Protokol exportieren
Procedure$ _ENM_Protocol_Builder_ExportBlueprint(*ENMPB._ENM_Protocol_Builder)
  Protected ProtocolLine$, i
  Protected Proto.StringBuilder = StringBuilder_New()
  With *ENMPB
    \EntryCnt - 1
    ProtocolLine$ = "PN: " + \Name$ + #CRLF$ ; Protokol - Name
    Proto\Add(@ProtocolLine$)
    ProtocolLine$ = "PV: " + Str(\Version) + #CRLF$ ; Protokol - Version
    Proto\Add(@ProtocolLine$)
    For i = 0 To \EntryCnt ; Alle Einträge + Typen + DefaultValue
      ProtocolLine$ = \Entrys(i)\Name$ + " I: " + Str(i) + " T: " + __ENM_Protocol_Type2String(\Entrys(i)\Type) + " D: " +  __ENM_Protocol_DefVal2String(\Entrys(i)) + #CRLF$
      Proto\Add(@ProtocolLine$)  
    Next
    ProtocolLine$ = Proto\GetString()
    Proto\Free()
  EndWith
  ProcedureReturn ProtocolLine$
EndProcedure

; VTable für den Builder:
DataSection
  _ENM_Protocol_Builder_VTable:
  Data.i @_ENM_Protocol_Builder_AddEntry_Byte()
  Data.i @_ENM_Protocol_Builder_AddEntry_Long()
  Data.i @_ENM_Protocol_Builder_AddEntry_Quad()
  Data.i @_ENM_Protocol_Builder_AddEntry_Float()
  Data.i @_ENM_Protocol_Builder_AddEntry_Double()
  Data.i @_ENM_Protocol_Builder_AddEntry_String()
  Data.i @_ENM_Protocol_Builder_AddEntry_Mem()
  Data.i @_ENM_Protocol_Builder_ExportBlueprint()
  CompilerIf #ENM_Sys_AsyncFree
    Data.i @_ENM_Protocol_Builder_Free_Async()
  CompilerElse
    Data.i @_ENM_Protocol_Builder_Free()
  CompilerEndIf   
EndDataSection

; IDE Options = PureBasic 4.60 Beta 3 (Windows - x64)
; CursorPosition = 112
; FirstLine = 99
; Folding = --
; EnableXP
; CPU = 1