﻿; ------------------------------------------------------------------------------------
; Easy Network Manager - Protokol Grundstrukturen
; PB 4.5+ OS: Windows/Linux/MacOS Architecture: x86/x64
; Author: Alexander Aigner, cxAlex @ purebasic.fr/german
; ------------------------------------------------------------------------------------

; EnableExplicit

#_ENM_Protocol_Magic = $FF
#_ENM_Protocol_MapSize = #ENM_ProtoBP_MapSize ; Die MapBlockGröße kleinhalten, spart Speicher und die Vergrößerungs-Performance
;                                               ist in diesem Fall irrelevant.

; Hilfs-Konstante
CompilerIf Defined(PB_Mem, #PB_Constant) = #False
  Enumeration
    #PB_Mem = $FE
  EndEnumeration
CompilerEndIf

; Protokoll - Eintrag (Variant)
Structure _ENM_Protocol_Entry
  Type.a ; Eintrags-Typ
  MemSize.i ; Speicher-Größe
  Name$ ; Eintrags-Name
  Used.a ; Eintrag wurde benutzt
  StructureUnion
    a.a
    l.l
    q.q
    f.f
    d.d
    *Mem
  EndStructureUnion
  s.s
EndStructure

Structure _ENM_Protocol_Default
  *VTable ; VTable
  *ENM._ENM_Data ; ENM - Instanz
  Name$ ; Protokol-Name
  Version.a ; Protokol-Version
  EntryCnt.a ; Eintrags-Zähler
  ; Beste Performance für den Transceiver, auch wenns etwas umständlicher in Builder & Blueprint wird
  ; außerdem speicherschonender
  Array Entrys._ENM_Protocol_Entry(0)
EndStructure

; Blueprint
Structure _ENM_Protocol_Blueprint Extends _ENM_Protocol_Default
  cpyMutex.i ; Kopier-Mutex
  PNCrc.l ; ProtokolName - CRC
  Map BackIndex.a(#_ENM_Protocol_MapSize) ; Rückindex
EndStructure

Structure __ENM_Protocol_ToMemory_Variant
  StructureUnion
    a.a
    l.l
    q.q
    f.f
    d.d
  EndStructureUnion
EndStructure

; Hilfsprocedure: String in Typ Umwandeln
Procedure __ENM_Protocol_String2Type(String$)
  Protected Type
  
  Select LCase(Trim(String$))
    Case "byte" : Type = #PB_Ascii
    Case "long": Type = #PB_Long
    Case "quad": Type = #PB_Quad
    Case "float": Type = #PB_Float
    Case "double": Type = #PB_Double
    Case "string": Type = #PB_String
    Case "memory": Type = #PB_Mem
  EndSelect
  
  ProcedureReturn Type
EndProcedure

; Hilfsprocedure: Typ in String Umwandeln
Procedure$ __ENM_Protocol_Type2String(Type)
  Protected String$
  
  Select Type
    Case #PB_Ascii  : String$ = "BYTE"
    Case #PB_Long   : String$ = "LONG"
    Case #PB_Quad   : String$ = "QUAD"
    Case #PB_Float  : String$ = "FLOAT"
    Case #PB_Double : String$ = "DOUBLE"
    Case #PB_String : String$ = "STRING"
    Case #PB_Mem    : String$ = "MEMORY"
  EndSelect
  
  ProcedureReturn String$
EndProcedure

; Hilfprocedure: Default-Value in String umwandeln
Procedure$ __ENM_Protocol_DefVal2String(*Entry._ENM_Protocol_Entry)
  Protected String$
  
  Select *Entry\Type
    Case #PB_Ascii  : String$ = Str(*Entry\a)
    Case #PB_Long   : String$ = Str(*Entry\l)
    Case #PB_Quad   : String$ = Str(*Entry\q)
    Case #PB_Float  : String$ = StrF(*Entry\f)
    Case #PB_Double : String$ = StrD(*Entry\d)
    Case #PB_String : String$ = *Entry\s
    Case #PB_Mem    : String$ = ""
  EndSelect
  
  ProcedureReturn String$
EndProcedure

; Parst einen Protokol-Bauplan.
Procedure _ENM_Protocol_Parser(Protocol$, *Proto._ENM_Protocol_Blueprint)
  Protected Line$, LineCount, RtVar, i, Error, t_pos, d_pos, i_pos, Name$, Type.a, Index
  
  With *Proto
    ; Index zurücksetzen:
    \EntryCnt = 0
    ; Auf Einheitlichen Separator bringen
    Protocol$ = ReplaceString(Protocol$, Chr(13) + Chr(10), Chr(10))
    Protocol$ = ReplaceString(Protocol$, Chr(10) + Chr(13), Chr(10))
    Protocol$ = ReplaceString(Protocol$, Chr(13), Chr(10))
    ; Doppelte Leerzeichen finden und ersetzen
    While FindString(Protocol$, Chr(32) + Chr(32))
      Protocol$ = ReplaceString(Protocol$, Chr(32) + Chr(32), Chr(32))
    Wend
    ; Leerzeilen finden und eleminieren
    While FindString(Protocol$, Chr(10) + Chr(32)) Or FindString(Protocol$, Chr(10) + Chr(10)) Or FindString(Protocol$, Chr(32) + Chr(10))
      Protocol$ = ReplaceString(Protocol$, Chr(10) + Chr(10), Chr(10))
      Protocol$ = ReplaceString(Protocol$, Chr(10) + Chr(32), Chr(10))
      Protocol$ = ReplaceString(Protocol$, Chr(32) + Chr(10), Chr(10))
    Wend  
    
    ; Zeilen zählen
    LineCount = CountString(Protocol$, Chr(10))    
    
    ; Letztes Zeichen <> Separator => + eine Zeile
    If Right(Protocol$, 1) <> Chr(10)
      LineCount + 1
    EndIf
    
    If LineCount >= 2 ; Minimum an Zeilen
      For i = 1 To LineCount
        Line$ = StringField(Protocol$, i, Chr(10))
        Select i
          Case 1 ; Protokoll - Name
            If Left(Line$, 4) = "PN: " ; Syntax ok?
              *Proto\Name$ = LCase(Mid(Line$, 5)) ; Name speichern
              *Proto\PNCrc = CRC32Fingerprint(@*Proto\Name$, StringByteLength(*Proto\Name$))
            Else
              Error = #True
              Break
            EndIf
            
          Case 2 ; Protokoll - Version
            If Left(Line$, 4) = "PV: " ; Syntax ok?
              *Proto\Version = Val(Mid(Line$, 5)) ; Version speichern
            Else
              Error = #True
              Break
            EndIf
            
          Default ; Einträge
            i_pos = FindString(Line$, " I: ") ; Index-Position
            t_pos = FindString(Line$, " T: ", i_pos) ; Type-Position
            d_Pos = FindString(Line$, " D:", t_pos) ; Default - Position
            If t_pos And d_pos And i_pos
              Name$ = LCase(Left(Line$, i_pos-1)) 
              i_pos + 4
              Index = Val(Trim(Mid(Line$, i_pos, t_pos-i_pos)))
              t_pos + 4
              Type = __ENM_Protocol_String2Type(Mid(Line$, t_pos, d_pos-t_pos))
              d_pos + 4
              If (Name$ <> "") And Type And (Index <= \EntryCnt) ; Name und Typ und Index ok?
                ; Array vergrößern
                ReDim \Entrys(\EntryCnt)
                \EntryCnt + 1
                 ; RückIndex speichern
                 AddMapElement(\BackIndex(), Name$) : \BackIndex() = Index
                \Entrys(Index)\Name$ = Name$
                \Entrys(Index)\Type = Type
                Select Type ; Default-Value laden
                  Case #PB_Byte   : \Entrys(Index)\a = Val(Mid(Line$, d_pos))
                  Case #PB_Long   : \Entrys(Index)\l = Val(Mid(Line$, d_pos))
                  Case #PB_Quad   : \Entrys(Index)\q = Val(Mid(Line$, d_pos))
                  Case #PB_Float  : \Entrys(Index)\f = ValF(Mid(Line$, d_pos))
                  Case #PB_Double : \Entrys(Index)\d = ValD(Mid(Line$, d_pos))
                  Case #PB_String : \Entrys(Index)\s = Mid(Line$, d_pos)
                  Case #PB_Mem
                EndSelect
              Else
                Error = #True
                Break
              EndIf
            Else
              Error = #True
              Break
            EndIf
        EndSelect
      Next
      \EntryCnt-1
    Else
      Error = #True
    EndIf
    
    If Error ; Es gab Fehler beim Parsen...
      ReDim \Entrys(0)
      \Name$ = ""
      \Version = #Null
    Else ; Alles ok!
      RtVar = #True
    EndIf
  EndWith
  
  ProcedureReturn RtVar
EndProcedure


Macro __ENM_Protocol_DefineEntry()
  CompilerIf Defined(*Entry, #PB_Variable) = #Null
    Protected *Entry._ENM_Protocol_Entry
  CompilerEndIf
  *Entry = *Proto\Entrys(i)
EndMacro

; Vergrößert den Puffer für das Protokol
; und versucht je nach Vergrößerungsart die Ideale Strategie zu wählen
Macro __ENM_Protocol_ToMemory_ResizeMem(_MemSize, _MemMode = #False)
  CompilerIf _MemMode ; Memory-Vergrößerungsmodus
    If (BlockCount*BlockSize)<_MemSize
      BlockCount = BlockCount + Round((_MemSize-(BlockCount*BlockSize))/BlockSize, #PB_Round_Up)
      *Mem = ReAllocateMemory(*Mem, BlockCount*BlockSize)
    EndIf
  CompilerElse ; Normaler Modus
    While (BlockCount*BlockSize)<_MemSize
      BlockCount+1
      *Mem = ReAllocateMemory(*Mem, BlockCount*BlockSize)
    Wend
  CompilerEndIf
EndMacro

; Standart - Eintrag anlegen
Macro __ENM_Protocol_ToMemory_AddEntry(_Type, _Entry)
  __ENM_Protocol_DefineEntry()
  __ENM_Protocol_ToMemory_ResizeMem(*MemSize\i + 2*SizeOf(Ascii) + SizeOf(_Type)) ; Speicher vergrößern
  PokeA(*Mem + *MemSize\i + 0*SizeOf(Ascii), i) ; Eintrags-ID schreiben
  PokeA(*Mem + *MemSize\i + 1*SizeOf(Ascii), #PB_#_Type) ; Typ schreiben
  Poke#_Entry(*Mem + *MemSize\i + 2*SizeOf(Ascii), *Entry\_Entry) ; Eintrag schreiben
  *MemSize\i + 2*SizeOf(Ascii) + SizeOf(_Type) ; speichergröße speichern
EndMacro

; String - Eintrag anlegen
Macro __ENM_Protocol_ToMemory_AddEntryString()
  __ENM_Protocol_DefineEntry()
  Protected sbl = StringByteLength(*Entry\s) ; StringByteLenght
  __ENM_Protocol_ToMemory_ResizeMem(*MemSize\i + 2*SizeOf(Ascii) + sbl + SizeOf(Character)) ; Speicher vergrößern
  PokeA(*Mem + *MemSize\i + 0*SizeOf(Ascii), i) ; Eintrags-id schreiben
  PokeA(*Mem + *MemSize\i + 1*SizeOf(Ascii), #PB_String) ; Typ schreiben
  PokeS(*Mem + *MemSize\i + 2*SizeOf(Ascii), *Entry\s, sbl) ; String schreiben
  *MemSize\i + 2*SizeOf(Ascii) + sbl + SizeOf(Character); speichergröße speichern
EndMacro

; Memory - Eintrag abspeichern
Macro __ENM_Protocol_ToMemory_AddEntryMem()
  __ENM_Protocol_DefineEntry()
  __ENM_Protocol_ToMemory_ResizeMem(*MemSize\i + 2*SizeOf(Ascii) + SizeOf(Long) + *Entry\MemSize, #True) ; Speicher vergrößern
  PokeA(*Mem + *MemSize\i + 0*SizeOf(Ascii), i) ; Eintrags-ID schreiben
  PokeA(*Mem + *MemSize\i + 1*SizeOf(Ascii), #PB_Mem) ; Eintrags-Typ schreiben
  PokeL(*Mem + *MemSize\i + 2*SizeOf(Ascii), *Entry\MemSize) ; Speicher-Länge schreiben
  If *Proto\Entrys(i)\MemSize ; Speicher schreiben
    CopyMemory(*Entry\Mem, *Mem + *MemSize\i + 2*SizeOf(Ascii) + SizeOf(Long), *Entry\MemSize)
  EndIf
  *MemSize\i + 2*SizeOf(Ascii) + SizeOf(Long) + *Entry\MemSize ; speichergröße speichern
EndMacro

; Eintrag aus speicher laden
Macro __ENM_Protocol_ImportFromMemory_ReadEntry(_Type, _Entry)
  __ENM_Protocol_DefineEntry()
  *Entry\_Entry = *Mem\_Entry ; Eintrag lesen
  *Mem + SizeOf(_Type) ; Größe weiterzählen
EndMacro

; String aus Speicher laden
Macro __ENM_Protocol_ImportFromMemory_ReadString()
  __ENM_Protocol_DefineEntry()
  *Entry\s = PeekS(*Mem) ; String lesen
  *Mem + StringByteLength(*Entry\s) + SizeOf(Character)
EndMacro

Macro __ENM_Protocol_ImportFromMemory_ReadMem()
  __ENM_Protocol_DefineEntry()
  Size = PeekL(*Mem) ; Speicher-Größe lesen
  *Mem + SizeOf(Long)  ; Größe weiterzählen
  If Size
    *Entry\Mem = AllocateMemory(Size) ; Speicher laden
    *Entry\MemSize = Size ; größe Speichern
    CopyMemory(*Mem, *Entry\Mem, Size) ; Speicher kopieren
    *Mem + Size  ; Größe weiterzählen
  EndIf
EndMacro

; Protokoll - Map in speicher exportieren zum versand über's Netzwerk
Procedure _ENM_Protocol_ToMemory(*Proto._ENM_Protocol_Blueprint, *MemSize.Integer, UsedOnly, MaxUsed, FirstUsed)
  Protected *Mem, i, TotalUsed, BlockCount, BlockSize
  
  ; Blockweises vergrößern
  BlockCount = 0
  BlockSize = 512
  *MemSize\i = #Null
  
  With *Proto
    
    ; HEADER
    __ENM_Protocol_ToMemory_ResizeMem(2*SizeOf(Ascii) + SizeOf(Long)) ; Speicher anlegen
    ; Magic schreiben
    PokeA(*Mem + *MemSize\i, #_ENM_Protocol_Magic)
    *MemSize\i + SizeOf(Ascii)
    ; Namens-CRC schreiben
    PokeL(*Mem + *MemSize\i, *Proto\PNCrc)
    *MemSize\i + SizeOf(Long)
    ; Version schreiben
    PokeA(*Mem + *MemSize\i, \Version)
    *MemSize\i + SizeOf(Ascii)
    
    ; Einträge schreiben
    For i = FirstUsed To \EntryCnt
      ; Optimierungen:
      If UsedOnly
        ; Mehr Felder werden nicht benutzt, fertig ...
        If TotalUsed >= MaxUsed
          Break
        EndIf
        ; Dieses Feld wird nicht benutzt, überspringen ...
        If Not \Entrys(i)\Used ; Used-Only Modus
          Continue
        EndIf
        TotalUsed + 1 ; Wirkliche Anzahl Einträge + 1
      EndIf
      
      Select \Entrys(i)\Type
        Case #PB_Ascii  : __ENM_Protocol_ToMemory_AddEntry(Ascii, a)
        Case #PB_Long   : __ENM_Protocol_ToMemory_AddEntry(Long, l)
        Case #PB_Quad   : __ENM_Protocol_ToMemory_AddEntry(Quad, q)
        Case #PB_Float  : __ENM_Protocol_ToMemory_AddEntry(Float, f)
        Case #PB_Double : __ENM_Protocol_ToMemory_AddEntry(Double, d)
        Case #PB_String : __ENM_Protocol_ToMemory_AddEntryString()
        Case #PB_Mem    : __ENM_Protocol_ToMemory_AddEntryMem()
      EndSelect
    Next
  EndWith
  
  ; 0-Terminieren und auf wahre Größe bringen
  *Mem = ReAllocateMemory(*Mem, *MemSize\i + SizeOf(Ascii))
  *MemSize\i + SizeOf(Ascii)
  
  ; Und fertig
  ProcedureReturn *Mem
EndProcedure

; Speicher in Protokol-Map umlegen
Procedure _ENM_Protocol_ImportFromMemory(*Proto._ENM_Protocol_BluePrint, *Mem.__ENM_Protocol_ToMemory_Variant, MemSize, *ErrorNeedReset.Ascii)
  Protected PNCrc.l, PVersion.a, Type, Size, RtVar, *BMem = *Mem, i, EntryCnt
  
  With *Proto
    ; Minimalgröße und Signatur Check
    If (MemSize > 8) And *Mem\a = #_ENM_Protocol_Magic ; Magic Ok?
      *Mem + SizeOf(Ascii)
      PNCrc = *Mem\l ; Namens-CRC lesen
      *Mem + SizeOf(Long)
      PVersion = *Mem\a ; Version lesen
      *Mem + SizeOf(Ascii)
      ; CRC und Version Check
      If (\PNCrc = PNCrc) And (\Version = PVersion) ; Protokol Ok?
        RtVar = #True
        Repeat
          i = *Mem\a ; Index lesen
          *Mem + SizeOf(Ascii)
          Type = *Mem\a ; Typ lesen
          *Mem + SizeOf(Ascii)
          EntryCnt+1 ; Eintragszähler erhöhen
          If (i <= \EntryCnt) And (\Entrys(i)\Type = Type) ; Protokoll ok?
            Select Type ; Wert lesen
              Case #PB_Ascii  : __ENM_Protocol_ImportFromMemory_ReadEntry(Ascii, a)
              Case #PB_Long   : __ENM_Protocol_ImportFromMemory_ReadEntry(Long, l)
              Case #PB_Quad   : __ENM_Protocol_ImportFromMemory_ReadEntry(Quad, q)
              Case #PB_Float  : __ENM_Protocol_ImportFromMemory_ReadEntry(Float, f)
              Case #PB_Double : __ENM_Protocol_ImportFromMemory_ReadEntry(Double, d)
              Case #PB_String : __ENM_Protocol_ImportFromMemory_ReadString()
              Case #PB_Mem    : __ENM_Protocol_ImportFromMemory_ReadMem()
              Default         : RtVar = #False : Break
            EndSelect
          Else
            RtVar = #False : Break
          EndIf
        Until Not *Mem\a ; 0-Terminierung
        If Not RtVar And EntryCnt>1; Error, muss Daten zurücksetzten
          *ErrorNeedReset\a = #True
        EndIf
      EndIf
    EndIf
  EndWith
  
  ProcedureReturn RtVar
EndProcedure

; IDE Options = PureBasic 4.60 Beta 3 (Windows - x64)
; CursorPosition = 38
; FirstLine = 27
; Folding = ---
; EnableXP
; CPU = 1