﻿; ------------------------------------------------------------------------------------
; Easy Network Manager - Protokol Transceiver
; PB 4.5+ OS: Windows/Linux/MacOS Architecture: x86/x64
; Author: Alexander Aigner, cxAlex @ purebasic.fr/german
; ------------------------------------------------------------------------------------

; Protokol - Blueprint
XIncludeFile "ENM_ProtocolBlueprint.pbi"

; Blueprint dublizieren
#_ENM_Protocol_Transceiver_BlueprintDublicate = #ENM_ProtoT_DublicateBlueprint

Structure _ENM_Protocol_Transceiver_TransmitterStatistics
  TotalUsed.a ; Totale Anzahl benutzter Felder
  FirstUsed.a ; Erstes benutztes Feld
EndStructure

Structure _ENM_Protocol_Transceiver_ReceiverStatistics
EndStructure

; Transceiver - Structure
Structure _ENM_Protocol_Transceiver
  *VTable ; VTable
  *ENM._ENM_Data ; ENM
  StructureUnion ; Empfänger
    BluePrint._ENM_Protocol_BluePrint_Ex ; Parent - Blueprint
    *BluePrintRaw._ENM_Protocol_BluePrint
  EndStructureUnion
  
  StructureUnion ; Empfänger
    *Receiver._ENM_Protocol_BluePrint
    ReceiverCrtl._ENM_Protocol_BluePrint_Ex
  EndStructureUnion
  
  StructureUnion ; Sender
    *Transmitter._ENM_Protocol_BluePrint
    TransmitterCrtl._ENM_Protocol_BluePrint_Ex
  EndStructureUnion
  
  ; Statistik zur Optimierung
  TransmitterStatistics._ENM_Protocol_Transceiver_TransmitterStatistics
  ReceiverStatistics._ENM_Protocol_Transceiver_ReceiverStatistics
EndStructure

; Kontrol - Interface
Interface ENM_Protocol_Transceiver
  ; Schreibe-Proceduren
  T_SetByte(Name$, Value.a)
  T_SetLong(Name$, Value.l)
  T_SetQuad(Name$, Value.q)
  T_SetFloat(Name$, Value.f)
  T_SetDouble(Name$, Value.d)
  T_SetString(Name$, Value.s)
  T_SetMem(Name$, *Mem, MemSize)
  ; Export und Reset
  T_Export(*MemSize.Integer, UsedOnly = #True)
  T_Reset(Total = #False)
  ; Lese-Proceduren
  R_GetByte.a(Name$)
  R_GetLong.l(Name$)
  R_GetQuad.q(Name$)
  R_GetFloat.f(Name$)
  R_GetDouble.d(Name$)
  R_GetString.s(Name$)
  R_GetMem(Name$, *MemSize.Integer)
  ; Import und Reset
  R_Import(*Mem, MemSize)
  R_Reset(Total = #False)
  ; Freigeben
  Free()
EndInterface

; Transmitter anlengen falls nötig
Macro _ENM_Protocol_Transceiver_CheckTransmitter()
  If Not *ENMPTC\Transmitter
    *ENMPTC\Transmitter = *ENMPTC\BluePrint\Dublicate() ; Neuen Transmitter anlegen
    *ENMPTC\TransmitterStatistics\FirstUsed = 255 ; Erstes benutztes Feld
  EndIf
EndMacro

; Receiver anlengen falls nötig
Macro _ENM_Protocol_Transceiver_CheckReceiver()
  If Not *ENMPTC\Receiver
    *ENMPTC\Receiver = *ENMPTC\BluePrint\Dublicate()
  EndIf
EndMacro

; Neuen Transceiver anlegen
Procedure _ENM_Protocol_NewTransceiver(*ENM._ENM_Data, BluePrint._ENM_Protocol_Blueprint_Ex)
  Protected *ENMPTC._ENM_Protocol_Transceiver = AllocateMemory(SizeOf(_ENM_Protocol_Transceiver))
  
  With *ENMPTC
    \VTable = ?_ENM_Protocol_Transceiver_VTable
    CompilerIf #_ENM_Protocol_Transceiver_BlueprintDublicate
      \BluePrint = BluePrint\Dublicate() ; Bauplan lokal dublizieren
    CompilerElse
      \BluePrint = BluePrint ; als geteiltes Objekt verwenden
    CompilerEndIf
    \ENM = *ENM
  EndWith
  
  ProcedureReturn *ENMPTC
EndProcedure

; Receiver Rücksetzen
Procedure _ENM_Protocol_Transceiver_ResetReceiver(*ENMPTC._ENM_Protocol_Transceiver, Total)
  With *ENMPTC
    If \Receiver ; Es gibt eine Receiver
      If Total ; Totales Reset, Speicherschonender
        \ReceiverCrtl\Free() ; Freigeben
        \Receiver = #Null ; Nullsetzen
      Else ; Eigendlich muss ja nur das Array kopiert werden...
        \BluePrint\Lock()
        CopyArray(\BluePrintRaw\Entrys(), \Receiver\Entrys())
        \BluePrint\Unlock()
      EndIf
      ClearStructure(@\ReceiverStatistics, _ENM_Protocol_Transceiver_ReceiverStatistics) ; Statistik zurücksetzen
    EndIf
  EndWith
EndProcedure

; Transmitter Rücksetzen
Procedure _ENM_Protocol_Transceiver_ResetTransmitter(*ENMPTC._ENM_Protocol_Transceiver, Total)
  With *ENMPTC
    If \Transmitter ; Es gibt eine Transmitter
      If Total ; Totales Reset, Speicherschonender
        \TransmitterCrtl\Free() ; Freigeben
        \Transmitter = #Null ; Nullsetzen
      Else ; eigendlich muss ja nur das Array kopiert werden...
        \BluePrint\Lock()
        CopyArray(\BluePrintRaw\Entrys(), \Transmitter\Entrys())
        \BluePrint\Unlock()
      EndIf
      ClearStructure(@\TransmitterStatistics, _ENM_Protocol_Transceiver_TransmitterStatistics) ; Statistik zurücksetzen
      \TransmitterStatistics\FirstUsed = 255 ; erstes benutztes Feld
    EndIf
  EndWith
EndProcedure

; Transceiver freigeben
Procedure _ENM_Protocol_Transceiver_Free(*ENMPTC._ENM_Protocol_Transceiver)
  _ENM_Protocol_Transceiver_ResetReceiver(*ENMPTC, #True)
  _ENM_Protocol_Transceiver_ResetTransmitter(*ENMPTC, #True)
  CompilerIf #_ENM_Protocol_Transceiver_BlueprintDublicate
    *ENMPTC\Blueprint\Free() ; Freigeben Falls nötig
  CompilerEndIf
  FreeMemory(*ENMPTC)
EndProcedure

CompilerIf #ENM_Sys_AsyncFree
  ; Procedure für den Async-Manager
  Procedure _ENM_Protocol_Transceiver_Free_Async(*ENMPT._ENM_Protocol_Transceiver)
    With *ENMPT
      If ENM
        JQ_Add(\ENM\GBCollector, @_ENM_Protocol_Transceiver_Free(), *ENMPT)
      Else
        _ENM_Protocol_Transceiver_Free(*ENMPT)
      EndIf
    EndWith
  EndProcedure
CompilerEndIf

; Standart - Schreibeprocedure für Transmitter
Macro __ENM_Protocol_Transceiver_SetX(_Type, _Entry, _CType)
  Procedure _ENM_Protocol_Transceiver_Transmitter_Set#_Type(*ENMPTC._ENM_Protocol_Transceiver, Name$, Value._Entry)
  Protected *Entry._ENM_Protocol_Entry, Index
    
    _ENM_Protocol_Transceiver_CheckTransmitter()
    
    Name$ = LCase(Name$)
    If FindMapElement(*ENMPTC\Transmitter\BackIndex(), Name$) ; Es gibt den Eintrag :D
      Index = *ENMPTC\Transmitter\BackIndex()
      *Entry = *ENMPTC\Transmitter\Entrys(Index) ; Eintrag laden
      If *Entry\Type = _CType ; Passt der Typ
        ; Für die Optimierungen...
        If (Not *Entry\Used) And (*Entry\_Entry <> Value)
          If Index < *ENMPTC\TransmitterStatistics\FirstUsed ; Neues 1. Feld
            *ENMPTC\TransmitterStatistics\FirstUsed = Index
          EndIf
          *Entry\Used = #True ; Benutzt
          *ENMPTC\TransmitterStatistics\TotalUsed + 1 ; Total benutzte Felder + 1
        EndIf
        *Entry\_Entry = Value ; Wert eintragen
        
        ProcedureReturn #True
      EndIf
    EndIf
  EndProcedure
EndMacro

; Schreibproceduren erstellen
__ENM_Protocol_Transceiver_SetX(Byte, a, #PB_Ascii)
__ENM_Protocol_Transceiver_SetX(Long, l, #PB_Long)
__ENM_Protocol_Transceiver_SetX(Quad, q, #PB_Quad)
__ENM_Protocol_Transceiver_SetX(Float, f, #PB_Float)
__ENM_Protocol_Transceiver_SetX(Double, d, #PB_Double)
__ENM_Protocol_Transceiver_SetX(String, s, #PB_String)

; Memory setzen
Procedure _ENM_Protocol_Transceiver_Transmitter_SetMem(*ENMPTC._ENM_Protocol_Transceiver, Name$, *Mem, MemSize)
  Protected *Entry._ENM_Protocol_Entry, Index
  
  _ENM_Protocol_Transceiver_CheckTransmitter()
  
  Name$ = LCase(Name$)
  If FindMapElement(*ENMPTC\Transmitter\BackIndex(), Name$) ; Es gibt den Eintrag :D
    Index = *ENMPTC\Transmitter\BackIndex()
    *Entry = *ENMPTC\Transmitter\Entrys(Index) ; Eintrag laden
    If *Entry\Type = #PB_Mem ; Passt der Typ?
      ; Für die Optimierungen...
      If Not *Entry\Used
        If Index < *ENMPTC\TransmitterStatistics\FirstUsed ; Neues 1. Feld
          *ENMPTC\TransmitterStatistics\FirstUsed = Index
        EndIf
        *Entry\Used = #True ; Used setzen
        *ENMPTC\TransmitterStatistics\TotalUsed + 1 ; Total benutzte Felder + 1
      EndIf 
      ; Speicher kopieren
      *Entry\Mem = AllocateMemory(MemSize)
      *Entry\MemSize = MemSize
      CopyMemory(*Mem, *Entry\Mem, MemSize)
      ; Rückgabe Ok
      ProcedureReturn #True
    EndIf
  EndIf
EndProcedure

; Transmitter exportieren
Procedure _ENM_Protocol_Transceiver_Transmitter_Export(*ENMPTC._ENM_Protocol_Transceiver, *MemSize.Integer, UsedOnly)
  Protected RtVar
  
  _ENM_Protocol_Transceiver_CheckTransmitter()
  
  ; Speicher exportieren
  ProcedureReturn _ENM_Protocol_ToMemory(*ENMPTC\Transmitter, *MemSize, UsedOnly, *ENMPTC\TransmitterStatistics\TotalUsed, *ENMPTC\TransmitterStatistics\FirstUsed)
EndProcedure


; Standart - Leseprocedure für den Receicer
Macro __ENM_Protocol_Transceiver_GetX(_Type, _Entry, _CType)
  Procedure._Entry _ENM_Protocol_Transceiver_Receiver_Get#_Type(*ENMPTC._ENM_Protocol_Transceiver, Name$)
    Protected *Entry._ENM_Protocol_Entry
    
    _ENM_Protocol_Transceiver_CheckReceiver()
    
    Name$ = LCase(Name$)
    If FindMapElement(*ENMPTC\Receiver\BackIndex(), Name$) ; Es gibt den Eintrag :D
      *Entry = *ENMPTC\Receiver\Entrys(*ENMPTC\Receiver\BackIndex()) ; Eintrag laden
      If *Entry\Type = _CType ; Typ prüfen
        ProcedureReturn *Entry\_Entry
      EndIf
    EndIf
  EndProcedure
EndMacro

; Leseproceduren erstellen
__ENM_Protocol_Transceiver_GetX(Byte, a, #PB_Ascii)
__ENM_Protocol_Transceiver_GetX(Long, l, #PB_Long)
__ENM_Protocol_Transceiver_GetX(Quad, q, #PB_Quad)
__ENM_Protocol_Transceiver_GetX(Float, f, #PB_Float)
__ENM_Protocol_Transceiver_GetX(Double, d, #PB_Double)
__ENM_Protocol_Transceiver_GetX(String, s, #PB_String)

; Memory-Procedure
Procedure _ENM_Protocol_Transceiver_Receiver_GetMem(*ENMPTC._ENM_Protocol_Transceiver, Name$, *MemSize.Integer)
  Protected *Entry._ENM_Protocol_Entry
  _ENM_Protocol_Transceiver_CheckReceiver()
  
  Name$ = LCase(Name$)
  If FindMapElement(*ENMPTC\Receiver\BackIndex(), Name$) ; Es gibt den Eintrag :D
    *Entry = *ENMPTC\Receiver\Entrys(*ENMPTC\Receiver\BackIndex()) ; Eintrag laden
    If *Entry\Type = #PB_Mem ; Typ prüfen
      *MemSize\i = *Entry\MemSize
      ProcedureReturn *Entry\Mem
    EndIf
  EndIf
  
EndProcedure

; Transmitter exportieren
Procedure _ENM_Protocol_Transceiver_Receiver_Import(*ENMPTC._ENM_Protocol_Transceiver, *Mem, MemSize)
  Protected RtVar, NeedReset.a
  
  _ENM_Protocol_Transceiver_CheckReceiver()
  
  ; Speicher exportieren
  RtVar = _ENM_Protocol_ImportFromMemory(*ENMPTC\Receiver, *Mem, MemSize, @NeedReset)
  If NeedReset ; Rücksetzten nötig, falsche Daten!
    _ENM_Protocol_Transceiver_ResetReceiver(*ENMPTC, #False)
  EndIf
  ProcedureReturn RtVar
EndProcedure

; VTable für den Transceiver
DataSection
  _ENM_Protocol_Transceiver_VTable:
  ; Transmitter:
  Data.i @_ENM_Protocol_Transceiver_Transmitter_SetByte()
  Data.i @_ENM_Protocol_Transceiver_Transmitter_SetLong()
  Data.i @_ENM_Protocol_Transceiver_Transmitter_SetQuad()
  Data.i @_ENM_Protocol_Transceiver_Transmitter_SetFloat()
  Data.i @_ENM_Protocol_Transceiver_Transmitter_SetDouble()
  Data.i @_ENM_Protocol_Transceiver_Transmitter_SetString()
  Data.i @_ENM_Protocol_Transceiver_Transmitter_SetMem()
  Data.i @_ENM_Protocol_Transceiver_Transmitter_Export()
  Data.i @_ENM_Protocol_Transceiver_ResetTransmitter()
  
  ; Receiver:
  Data.i @_ENM_Protocol_Transceiver_Receiver_GetByte()
  Data.i @_ENM_Protocol_Transceiver_Receiver_GetLong()
  Data.i @_ENM_Protocol_Transceiver_Receiver_GetQuad()
  Data.i @_ENM_Protocol_Transceiver_Receiver_GetFloat()
  Data.i @_ENM_Protocol_Transceiver_Receiver_GetDouble()
  Data.i @_ENM_Protocol_Transceiver_Receiver_GetString()
  Data.i @_ENM_Protocol_Transceiver_Receiver_GetMem()
  Data.i @_ENM_Protocol_Transceiver_Receiver_Import()
  Data.i @_ENM_Protocol_Transceiver_ResetReceiver()
  
  CompilerIf #ENM_Sys_AsyncFree
    Data.i @_ENM_Protocol_Transceiver_Free_Async()
  CompilerElse
    Data.i @_ENM_Protocol_Transceiver_Free()
  CompilerEndIf
EndDataSection
; IDE Options = PureBasic 4.60 Beta 3 (Windows - x64)
; CursorPosition = 7
; Folding = ---
; EnableXP
; CPU = 1