﻿; ------------------------------------------------------------------------------------
; Easy Proxy Connector
; PB 4.5+ OS: Windows/Linux/MacOS Architecture: x86/x64
; Author: Alexander Aigner, cxAlex @ purebasic.fr/german
; ------------------------------------------------------------------------------------

Enumeration 1
  #ENM_Proxy_SOCKS4
  #ENM_Proxy_SOCKS5
EndEnumeration

; Ported from EProxy: http://www.purebasic.fr/german/viewtopic.php?t=23357

; Version
#SOCKS_Version = $05

; Methoden
; $03 - $7F IANA Assigned
; $80 - $FE Reserved for Private Methodes
#SOCKS_Method_NoAuthenticationRequired = $00
#SOCKS_Method_GSSAPI = $01
#SOCKS_Method_UserNamePassword = $02
#SOCKS_Method_NoAcceptableMethod = $00

; Socks Kommandos
#SOCKS_CMD_Connect = $01
#SOCKS_CMD_Bind = $02
#SOCKS_CMD_UDPAssociate = $03

; Adresstype
#SOCKS_ATYP_IPv4 = $01
#SOCKS_ATYP_DomainName = $03
#SOCKS_ATYP_IPv6 = $04
#SOCKS_ATYP_SOCKS4 = $05

; Antworten
; $09 - $FF IANA Assigned
#SOCKS_V4_REP_Succeeded = $5A
#SOCKS_REP_Succeeded = $00
#SOCKS_REP_GeneralSocksServerFailure = $01
#SOCKS_REP_ConnectionNotAllowedByRuleset = $02
#SOCKS_REP_NetworkUnreachable = $03
#SOCKS_REP_HostUnreachable = $04
#SOCKS_REP_ConnectionRefused = $05
#SOCKS_REP_TTLExpired = $06
#SOCKS_REP_CommandNotSupported = $07
#SOCKS_REP_AddressTypeNotSupported = $08

; Authentication
#SOCKS_AUTH_Ok = $00
#SOCKS_AUTH_Failure = $01

Structure SOCKS_V4_Ver
  ver.a
  rsv.a
  port.w
  ip.l
EndStructure

; Socks CMD
Structure SOCKS_Cmd
  ver.a ; Version, muss $05 sein
  cmd.a ; Kommando
  rsv.a ; Reserved muss $00 sein
  atyp.a ; Addresstyp
EndStructure

; Octed - Struktur
Structure _SOCKS_Port_Oct
  high.a
  low.a
EndStructure

; Socks Reply
Structure SOCKS_Reply
  ver.a ; Version, muss $05 sein
  rep.a ; Antwort
  rsv.a ; Reserved muss $00 sein
  atyp.a ; Addresstyp
  addr.l ; Addresse
  port.w ; Port
EndStructure

; Erste Anfrage
Structure SOCKS_Ver
  ver.a ; Version, muss $05 sein
  StructureUnion
    nMethods.a ; Anzahl methoden
    Method.a ; Methode
  EndStructureUnion
  Methods.a[255] ; Auswahl an Methoden
EndStructure

; Port in Octed - Schreibweise
Procedure.w _Port2Octed(Port)
  Protected Oct.w
  Protected t8Port._SOCKS_Port_Oct
    
  t8Port\high = Int(Port/256)
  t8Port\low = Port-(256*t8Port\high)
  Oct = PeekW(@t8Port)
  
  ProcedureReturn Oct
EndProcedure

; Octed in Port Schreibweise
Procedure _Octed2Port(Oct)
  Protected *t8Port._SOCKS_Port_Oct
  Protected Port
  
  *t8Port = @Oct
  Port = *t8Port\high*256 + *t8Port\low
  
  ProcedureReturn Port
EndProcedure

Procedure _IsIP(ip.s)
  Protected count, ret, i, a$, va
  ;ReturnValues :
  ;0 = no ip
  ;1 = IPv4 - IP
  count = CountString(ip, ".")
  If count = 3
    ret = 1
  Else
    ProcedureReturn 0
  EndIf
  For i = 0 To count
    a$ = StringField(ip, i + 1, ".")
    va = Val(a$)
    If a$ = "" Or Str(va)<>a$ Or va>255 Or va<0
      ProcedureReturn 0
    EndIf
  Next i
  ProcedureReturn ret
EndProcedure

Macro _SOCKS_Receive(_Mem, _MemSize)
  t = ElapsedMilliseconds()
  OverTime = #False
  Repeat
    Delay(5)
    If NetworkClientEvent(ConId) = #PB_NetworkEvent_Data
      ReceiveNetworkData(ConId, _Mem, _MemSize)
      Break
    EndIf
    If (ElapsedMilliseconds()-t) >= timeout
      OverTime = #True
    EndIf
  Until OverTime
EndMacro

; Verbindung über einen SOCKS54 Proxy herstellen
Procedure _SOCKS4_Connect(proxy_host$, proxy_port, dest_host$, dest_port, User$ = "", timeout = 5000)
  Protected ConId, CmdSize, *Cmd.SOCKS_V4_Ver, t, OverTime, RtVar
  
  ; Verbindung zum Proxy herstellen
  ConId = OpenNetworkConnection(proxy_host$, proxy_port)
  
  If ConId
    CmdSize = SizeOf(SOCKS_V4_Ver) + SizeOf(User$) + SizeOf(Byte)
    *Cmd = AllocateMemory(CmdSize)
    If _isIP(dest_host$)
      *Cmd\ver = $04 ; Socks V4
      *Cmd\rsv = 1 ; ?
      *Cmd\port = _Port2Octed(dest_port) ; Port
      ; IP
      *Cmd\ip = MakeIPAddress(Val(StringField(dest_host$, 1, ".")), Val(StringField(dest_host$, 2, ".")), Val(StringField(dest_host$, 3, ".")), Val(StringField(dest_host$, 4, ".")))
      ; User
      PokeS(*Cmd + SizeOf(SOCKS_V4_Ver), User$)
      
      SendNetworkData(ConId, *Cmd, CmdSize) ; Connect senden
      _SOCKS_Receive(*CMD, CmdSize) ; Auf Antwort warten
  
      
      If (Not OverTime) And *Cmd\ver = #Null And *Cmd\rsv = #SOCKS_V4_REP_Succeeded ; Alles ok?
        RtVar = ConId ; Verbindung hergstellt
      Else
        CloseNetworkConnection(ConId)
      EndIf
    EndIf
    FreeMemory(*CMD)
  EndIf
  
  ProcedureReturn RtVar
EndProcedure

; Verbindung über einen SOCKS5 Proxy herstellen
Procedure _SOCKS5_Connect(proxy_host$, proxy_port, dest_host$, dest_port, User$ = "", Pass$ = "", timeout = 5000)
  Protected ConId, Ver.SOCKS_Ver, *Cmd.SOCKS_Cmd, t, RtVar, CmdSize, Repl.SOCKS_Reply, OverTime
  
  ; Verbindung zum Proxy herstellen
  ConId = OpenNetworkConnection(proxy_host$, proxy_port)
  
  If ConId
    CmdSize = StringByteLength(User$) + StringByteLength(Pass$) + SizeOf(SOCKS_Cmd)
    *Cmd = AllocateMemory(CmdSize)
    
    Ver\Ver = #SOCKS_Version
    
    ; Phase 1: Anmelden
    If Not User$ ; Kein User, Offener Proxy
      Ver\NMethods = 1
      Ver\Methods[0] = #SOCKS_Method_NoAuthenticationRequired
    Else ; Offen oder Auth
      Ver\NMethods = 2
      Ver\Methods[0] = #SOCKS_Method_NoAuthenticationRequired
      Ver\Methods[1] = #SOCKS_Method_UserNamePassword
    EndIf
    
    SendNetworkData(ConId, @Ver, SizeOf(Word) + Ver\NMethods*SizeOf(Byte))
    _SOCKS_Receive(@Ver, SizeOf(Word)) ; Auf Antwort warten
    
    If Not (OverTime Or Ver\Ver <> #SOCKS_Version) ; alles Ok bisher
      Select Ver\Method
        Case #SOCKS_Method_NoAuthenticationRequired
          Ver\Method = #SOCKS_AUTH_Ok
          ; Nichts mehr zu tun, der Server verbindet ohne Passwort ...  
          
        Case #SOCKS_Method_UserNamePassword ; Der Server will ein Passwort
          ; Phase 2: Auth
          *Cmd\ver = #SOCKS_Version
          *Cmd\cmd = StringByteLength(User$)
          CopyMemory(@User$, *Cmd + 2*SizeOf(Byte), StringByteLength(User$))
          PokeA(*Cmd + 2*SizeOf(Byte) + StringByteLength(User$), StringByteLength(Pass$))
          CopyMemory(@Pass$, *Cmd + 2*SizeOf(Byte) + StringByteLength(User$), StringByteLength(Pass$))
          
          SendNetworkData(ConId, *Cmd, CmdSize) ; Auth senden
          _SOCKS_Receive(@Ver, SizeOf(Word)) ; Auf Antwort warten
          
          If (OverTime Or Ver\ver <> #SOCKS_Version) ; Fehler
            Ver\Method = #SOCKS_AUTH_FAILURE
          EndIf
          
      EndSelect
      
      If Ver\Method = #SOCKS_AUTH_Ok ; Jetzt kommen wir zum Connect
        ; Wir führen einfach eine Verbindung per DomainName durch, dadurch umgehen wir auch DNS - Filter
        CmdSize = SizeOf(SOCKS_Cmd) + 3*SizeOf(Byte) + StringByteLength(dest_host$)
        *CMD = ReAllocateMemory(*CMD, CmdSize)
        
        *CMD\ver = #SOCKS_Version
        *CMD\cmd = #SOCKS_CMD_Connect
        *CMD\rsv = #Null
        *CMD\atyp = #SOCKS_ATYP_DomainName
        PokeA(*CMD + SizeOf(SOCKS_Cmd), StringByteLength(dest_host$))
        CopyMemory(@dest_host$, *CMD + SizeOf(SOCKS_Cmd) + SizeOf(Byte), StringByteLength(dest_host$))
        PokeW(*CMD + SizeOf(SOCKS_Cmd) + SizeOf(Byte) + StringByteLength(dest_host$), _Port2Octed(dest_port))
        
        SendNetworkData(ConId, *Cmd, CmdSize) ; Connect senden
        _SOCKS_Receive(@Repl, SizeOf(SOCKS_Reply)) ; Auf Antwort warten
        
        If Not (OverTime Or Repl\Ver <> #SOCKS_Version)
          If Repl\rep = #SOCKS_REP_Succeeded
            RtVar = #True
          EndIf
        EndIf
      EndIf
    EndIf
    
    FreeMemory(*Cmd)
    If Not RtVar
      CloseNetworkConnection(ConId)
      ConId = #Null
    Else
      RtVar = ConId
    EndIf
  EndIf
    
  ProcedureReturn RtVar
EndProcedure


; Verbindet mit einem unterstützten Proxy
Procedure _ENM_ProxyConnect(dest_host$, dest_port, proxy_type, proxy_host$, proxy_port, User$ = "", Pass$ = "", timeout = 5000)
  Protected RtVar
  
  ; Proxy-Typ switchen, in Zukunft hier erweitern
  Select proxy_type
    Case #ENM_Proxy_SOCKS5
      RtVar = _SOCKS5_Connect(proxy_host$, proxy_port, dest_host$, dest_port, User$, Pass$, timeout)
    Case #ENM_Proxy_SOCKS4
      RtVar = _SOCKS4_Connect(proxy_host$, proxy_port, dest_host$, dest_port, User$, timeout)
  EndSelect
  
  ProcedureReturn RtVar
EndProcedure
; IDE Options = PureBasic 4.60 Beta 3 (Windows - x64)
; CursorPosition = 277
; FirstLine = 246
; Folding = --
; EnableXP
; CPU = 1