﻿; ------------------------------------------------------------------------------------
; Easy Network Manager - Stream Verwaltung
; PB 4.5+ OS: Windows/Linux/MacOS Architecture: x86/x64
; Author: Alexander Aigner, cxAlex @ purebasic.fr/german
; ------------------------------------------------------------------------------------

; Variablen Refractor, Send/Receive besser trennen

; Interface
Interface ENM_Stream
  WaitForEvent() ; Wartet auf ein Event
  SendMem(*Mem, MemSize, Compression = #Null)
  R_GetMem() ; kompletten Speicher laden
  R_GetReceivedMemSize() ; komplette empfangene Speichergröße
  R_GetCurMem() ; Aktuelle Speicheradresse
  R_GetCurMemSize() ; Aktuelle speichergröße
  R_GetTotalMemSize() ; Totale Speichergröße des aktuellen Streams
  S_GetTotalMemSize() ; Totale Send Speichergröße
  S_GetSentMemSize() ; Gesendete Speichergröße
  Close() ; Stream-Closed Event an der Gegenseite auslösen
  Reset() ; Setzt den bisher empfangenen Speicher zurück
  Free() ; Freigeben
EndInterface

; Stream - Verwaltung
Structure _ENM_Stream_Data
  *VTable ; VTable
  *Con._ENM_ClientData_ServerClient ; Connection
  *ENM._ENM_Data ; übergeordnetes ENM - Objekt
  GMem.i ; Wachsender Speicher
  Event.i ; Stream - Event
  ForceEvent.i ; Event einspielen
  Comp.i ; Stream - Kompression
  CPartSize.i ; Aktuelle Part-Größe
  *SendMem ; zu sendender Speicher
  ; Kann sich intern ändern/Partgrößen - änderung!
  SendMemSize.i ; SendeSpeicher größe
  SendMemParts.i ; Anzahl Parts
  SendMemcPart.i ; Aktueller Part
  ; Für den Empfänger, aktueller Part
  SendMemCurMem.i ; Aktueller Speicher
  SendMemCurMemSize.i ; Aktuelle Speichergröße
  ; Für die Statistik-Funktionen
  TotalReceiveMemSize.i ; Totale Empfangsgröße (Empfänger)
  TotalSendMemSize.i ; Sendespeicher Größe
  CurrentSendMemSize.i ; Gesendete Speichergröße
EndStructure

Enumeration 1
  #ENM_Stream_Receive_Part
  #ENM_Stream_Receive_FirstPart
  #ENM_Stream_Receive_LastPart
  #ENM_Stream_Receive_SinglePart
  #ENM_Stream_Send_Part
  #ENM_Stream_Send_LastPart
  #ENM_Stream_Closed
  #ENM_Stream_ConnectionClosed
  #ENM_Stream_Error
  #_ENM_Stream_OrderNextPart
EndEnumeration

; Stream Header
Structure _ENM_StreamHeader
  Type.a ; StreamType
EndStructure

Structure _ENM_StreamHeader_FirstPart Extends _ENM_StreamHeader
  TotalSize.l ; Anzahl Parts
EndStructure

; SplittPart
Structure _SplittPart
  OffSet.i
  Size.i
EndStructure

; Ermittel Anzahl Parts
Procedure _ENM_Stream_CountParts(_GesSize, _PartSize)
  Protected Parts = Round(_GesSize/_PartSize, #PB_Round_Down)
  
  If Not Parts
    Parts = 1
  EndIf
  
  ProcedureReturn Parts
EndProcedure

Declare _ENM_Stream_Reset(*ENMS._ENM_Stream_Data)

; Berechnet die Position und Größe eines Parts
Procedure _ENM_Stream_CalcPart(Size.i, Parts.i, CurrentPart.i, *SN._SplittPart)
  Protected PartSize.i
  
  PartSize = Size/Parts
  
  *SN\OffSet = (CurrentPart-1)*PartSize
  If Parts = CurrentPart
    *SN\Size = Size-*SN\OffSet
  Else
    *SN\Size = PartSize
  EndIf
  
EndProcedure

; Öffnet einen Stream
Procedure _ENM_Stream_OpenStream(*ENM._ENM_Data, Client.i, Server.i = #Null)
  Protected *ENMS._ENM_Stream_Data
  
  If Server ; ServerModus
    If Not *ENM\Type & #ENM_Server
      ProcedureReturn #Null
    EndIf
    
    With *ENM\Server
      LockMutex(\GMutex)
      ; Connection finden
      If FindMapElement(\Servers(), Str(Server))
        If FindMapElement(\Servers()\Clients(), Str(Client))
          ; Stream erstellen
          If Not \Servers()\Clients()\Stream
            *ENMS = AllocateMemory(SizeOf(_ENM_Stream_Data))
            *ENMS\Con = \Servers()\Clients()
          EndIf
        EndIf
      EndIf
      UnlockMutex(\GMutex)
    EndWith
    
    
  Else ; ClientModus
    If Not *ENM\Type & #ENM_Client
      ProcedureReturn #Null
    EndIf
    
    With *ENM\Client
      LockMutex(\GMutex)
      If FindMapElement(\Clients(), Str(Client))
        ; Stream erstellen
        If Not \Clients()\Stream
          *ENMS = AllocateMemory(SizeOf(_ENM_Stream_Data))
          *ENMS\Con = \Clients()
        EndIf
      EndIf
      UnlockMutex(\GMutex)
    EndWith
    
  EndIf
  
  If *ENMS ; Rest erstellen
    *ENMS\VTable = ?_ENM_Stream_VTable ; VTable
    *ENMS\ENM = *ENM ; ENM mitschleifen schadet nie...
    *ENMS\Con\Stream = CreateSemaphore() ; Connection in den Stream-Modus versetzen
    *ENMS\Con\ReceiveTS\RecData = ElapsedMilliseconds() ; Empfangen erlaubt
    *ENMS\CPartSize = #ENM_Stream_MaximumPartSize ; Partgröße setzen
  EndIf
  
  ProcedureReturn *ENMS
EndProcedure

; Stream freigeben
Procedure _ENM_Stream_Free(*ENMS._ENM_Stream_Data)
  With *ENMS
    ; Semaphore freigeben
    FreeSemaphore(\Con\Stream)
    \Con\Stream = #Null
    ; GMem freigeben
    If \GMem
      GMem_Free(\GMem)
    EndIf
    ; Speicher freigeben
    FreeMemory(*ENMS)
  EndWith
EndProcedure

CompilerIf #ENM_Sys_AsyncFree
  ; Freigabe für den Async-Manager
  Procedure _ENM_Stream_Free_Async(*ENMS._ENM_Stream_Data)
    With *ENMS
      If ENM
        JQ_Add(\ENM\GBCollector, @_ENM_Stream_Free(), *ENMS)
      Else
        _ENM_Stream_Free(*ENMS)
      EndIf
    EndWith
  EndProcedure
CompilerEndIf

; Auf Event warten
Procedure _ENM_Stream_WaitForEvent(*ENMS._ENM_Stream_Data)
  Protected *Stream._ENM_StreamHeader_FirstPart, MemSize, SP._SplittPart
  Protected SPakets.ENM_Pakets, SendStream._ENM_StreamHeader
  Protected *SendMem._ENM_StreamHeader, OffSetSize, RtVar
  
  With *ENMS
    Repeat
      \Event = #Null
      WaitSemaphore(\Con\Stream) ; Auf Event warten
      If \ForceEvent ; Event einspielen
        \Event = \ForceEvent
        \ForceEvent = #Null
        Break
      EndIf
      
      
      If Not \Con\SecureID
        \Event = #ENM_Stream_ConnectionClosed
      Else
        ; Pakete laden
        SPakets = ENM\ReceivePakets(\Con\ID)
        ; Empfang rücksetzen
        \Con\ReceiveTS\RecData = ElapsedMilliseconds()
        If SPakets ; Pakete ok.
          While SPakets\GetNext() ; Alle durchgehen
            \Event = #Null
            ; Stream laden
            *Stream = SPakets\GetMem()
            MemSize = SPakets\GetMemSize()
            Select *Stream\Type
              Case #ENM_Stream_Receive_SinglePart, #ENM_Stream_Receive_FirstPart, #ENM_Stream_Receive_Part, #ENM_Stream_Receive_LastPart ; Part empfangen
                ; Reseten wenn nötig
                If (*Stream\Type = #ENM_Stream_Receive_FirstPart) Or (*Stream\Type = #ENM_Stream_Receive_SinglePart)
                  _ENM_Stream_Reset(*ENMS) ; Reset
                  \TotalReceiveMemSize = *Stream\TotalSize ; Totale größe Speichern
                  OffSetSize = SizeOf(_ENM_StreamHeader_FirstPart)
                Else
                  OffSetSize = SizeOf(_ENM_StreamHeader)
                EndIf
                
                ; Größe der Raw-Daten berechnen
                \SendMemCurMemSize = MemSize - OffSetSize
                
                ; Stream vergrößern
                GMem_WriteData(\GMem, *Stream + OffSetSize, \SendMemCurMemSize)
                
                ; Aktuellen Part eintragen:
                \SendMemCurMem = GMem_GetMem(\GMem) + (GMem_GetMemSize(\GMem) - \SendMemCurMemSize)
                
                ; Nächsten Part ordern
                If *Stream\Type <> #ENM_Stream_Receive_LastPart ; War schon der letzte Part...
                  SendStream\Type = #_ENM_Stream_OrderNextPart
                  ENM\SendPaket(\Con\ID, @SendStream, SizeOf(_ENM_StreamHeader))  
                EndIf
                
              Case #_ENM_Stream_OrderNextPart ; Andere Seiter ordert Part
                If \SendMem
                  
                  RtVar = #Null
                  While Not RtVar
                    ; Part berechnen
                    _ENM_Stream_CalcPart(\SendMemSize, \SendMemParts, \SendMemcPart, @SP)
                    
                    ; Paket erstellen
                    *SendMem = AllocateMemory(SizeOf(_ENM_StreamHeader) + SP\Size)
                    CopyMemory(\SendMem + SP\OffSet, *SendMem + SizeOf(_ENM_StreamHeader), SP\Size)
                    
                    ; Letzter Part!
                    If \SendMemParts = \SendMemcPart
                      \Event = #ENM_Stream_Send_LastPart
                      *SendMem\Type = #ENM_Stream_Receive_LastPart
                      \SendMem = #Null
                    Else ; Nächster Part
                      \Event = #ENM_Stream_Send_Part
                      *SendMem\Type = #ENM_Stream_Receive_Part
                      \SendMemcPart + 1
                    EndIf
                    
                    ; Paket senden
                    RtVar = ENM\SendPaket(\Con\ID, *SendMem, SizeOf(_ENM_StreamHeader) + SP\Size, \Comp)
                    ; Paket freigeben
                    FreeMemory(*SendMem)
                    
                    If Not RtVar
                      If \CPartSize>=1280 ; Noch gr
                        \CPartSize*(2/3)
                        ; Speicher neu definieren
                        \SendMem + SP\OffSet
                        \SendMemSize - SP\OffSet
                        \SendMemParts = _ENM_Stream_CountParts(\SendMemSize, \CPartSize)
                        \SendMemcPart = 1
                      Else ; Fehler, beenden
                        \Event = #ENM_Stream_Error
                        Break 2
                      EndIf
                    Else ; Gesendet erhöhen...
                      \CurrentSendMemSize + SP\Size
                    EndIf
                    
                  Wend
                  
                EndIf
                
              Case #ENM_Stream_Closed
                Break
                ; Stream beendet
            EndSelect
            
            ; Letztes Event merken
            If Not \Event
              \Event = *Stream\Type
            EndIf
          Wend
          ; Pakete freigeben
          SPakets\Free()
        EndIf
      EndIf
    Until \Event ; Warten bis ein Event kommt
    
    ; Event zurückgeben
    ProcedureReturn \Event
  EndWith
EndProcedure

; Stream senden
Procedure _ENM_Stream_SendMemory(*ENMS._ENM_Stream_Data, *Mem, MemSize, Compression = #Null)
  Protected *SendMem._ENM_StreamHeader_FirstPart, SP._SplittPart, RtVar
  
  With *ENMS
    If (Not \SendMem) And \Con\SecureID
      ; Daten speichern
      \SendMem = *Mem ; zu sendender Speicher
      \SendMemSize = MemSize ; Speichergröße
      \TotalSendMemSize = MemSize ; Speichergröße (ändert sicht nicht bei Part-Resize!)
      \SendMemParts = _ENM_Stream_CountParts(\SendMemSize, \CPartSize) ; Anzahl zu sendender Parts
      \SendMemcPart = 1 ; 1. Part
      \Comp = Compression ; Kompression
      
      ; Part berechnen
      _ENM_Stream_CalcPart(\SendMemSize, \SendMemParts, \SendMemcPart, @SP)
      
      ; Paket erstellen
      *SendMem = AllocateMemory(SizeOf(_ENM_StreamHeader_FirstPart) + SP\Size)
      CopyMemory(\SendMem + SP\OffSet, *SendMem + SizeOf(_ENM_StreamHeader_FirstPart), SP\Size)
      
      ; Größe speichern:
      *SendMem\TotalSize = MemSize
      
      ; Letzter = 1. Part!
      If \SendMemParts = \SendMemcPart
        *SendMem\Type = #ENM_Stream_Receive_SinglePart
        \SendMem = #Null
      Else
        *SendMem\Type = #ENM_Stream_Receive_FirstPart
        \SendMemcPart + 1
      EndIf
      
      ; Paket senden
      RtVar = ENM\SendPaket(\Con\ID, *SendMem, SizeOf(_ENM_StreamHeader_FirstPart) + SP\Size, \Comp)  
      ; Paket freigeben
      FreeMemory(*SendMem)
      
      If (Not RtVar) And (\CPartSize>=1280) ; schon seeeehr langsam...
        \CPartSize*(2/3) ; Partsize verkleinern
        RtVar = _ENM_Stream_SendMemory(*ENMS, *Mem, MemSize, Compression) ; Neuer Sendeversuch mit kleinerer Partsize
      ElseIf \SendMemParts = \SendMemcPart ; SinglePart!
        \ForceEvent = #ENM_Stream_Send_LastPart
        SignalSemaphore(\Con\Stream)
      Else
        \CurrentSendMemSize = SP\Size ; schon gesendet...
      EndIf
      ; Weitere Pakete werden in der Event-Schleife geordert
      ProcedureReturn RtVar
    EndIf
  EndWith
EndProcedure

; Speicher laden
Procedure _ENM_Stream_R_GetMem(*ENMS._ENM_Stream_Data)
  With *ENMS
    ProcedureReturn GMem_GetMem(\GMem)
  EndWith
EndProcedure

; Speichergröße laden
Procedure _ENM_Stream_R_GetMemSize(*ENMS._ENM_Stream_Data)
  With *ENMS
    ProcedureReturn GMem_GetMemSize(\GMem)
  EndWith
EndProcedure

; aktuellen Speicher laden
Procedure _ENM_Stream_R_GetCurMem(*ENMS._ENM_Stream_Data)
  With *ENMS
    ProcedureReturn \SendMemCurMem
  EndWith
EndProcedure

; aktuelle Speichergröße laden
Procedure _ENM_Stream_R_GetCurMemSize(*ENMS._ENM_Stream_Data)
  With *ENMS
    ProcedureReturn \SendMemCurMemSize
  EndWith
EndProcedure

; totale Speichergröße laden
Procedure _ENM_Stream_R_GetTotalMemSize(*ENMS._ENM_Stream_Data)
  With *ENMS
    ProcedureReturn \TotalReceiveMemSize
  EndWith
EndProcedure

; totale Speichergröße laden
Procedure _ENM_Stream_S_GetTotalMemSize(*ENMS._ENM_Stream_Data)
  With *ENMS
    ProcedureReturn \TotalSendMemSize
  EndWith
EndProcedure

; totale Speichergröße laden
Procedure _ENM_Stream_S_GetSentMemSize(*ENMS._ENM_Stream_Data)
  With *ENMS
    ProcedureReturn \CurrentSendMemSize
  EndWith
EndProcedure

Procedure _ENM_Stream_Close(*ENMS._ENM_Stream_Data)
  Protected SendStream._ENM_StreamHeader
  With *ENMS
    SendStream\Type = #ENM_Stream_Closed
    ENM\SendPaket(\Con\ID, @SendStream, SizeOf(_ENM_StreamHeader))
  EndWith
EndProcedure

; Speicher zurücksetzen
Procedure _ENM_Stream_Reset(*ENMS._ENM_Stream_Data)
  With *ENMS
    If \GMem ; GMem Freigeben wenn nötig
      GMem_Free(\GMem)
    EndIf
    ; Neuer GMem, Neue Parts
    \GMem = GMem_New(#ENM_Stream_BufferBlockSize)
    ; Speichergrößen rücksetzen
    \SendMemCurMemSize = #Null
    \SendMemCurMem = #Null
    \TotalReceiveMemSize = #Null
    \CPartSize = #ENM_Stream_MaximumPartSize ; Partgröße setzen
  EndWith
EndProcedure

; VTable:
DataSection
  _ENM_Stream_VTable:
  Data.i @_ENM_Stream_WaitForEvent()
  Data.i @_ENM_Stream_SendMemory()
  Data.i @_ENM_Stream_R_GetMem()
  Data.i @_ENM_Stream_R_GetMemSize()
  Data.i @_ENM_Stream_R_GetCurMem()
  Data.i @_ENM_Stream_R_GetCurMemSize()
  Data.i @_ENM_Stream_R_GetTotalMemSize()
  Data.i @_ENM_Stream_S_GetTotalMemSize()
  Data.i @_ENM_Stream_S_GetSentMemSize()
  Data.i @_ENM_Stream_Close()
  Data.i @_ENM_Stream_Reset()
  CompilerIf #ENM_Sys_AsyncFree
    Data.i @_ENM_Stream_Free_Async()  
  CompilerElse
    Data.i @_ENM_Stream_Free()
  CompilerEndIf
EndDataSection
; IDE Options = PureBasic 4.60 Beta 3 (Windows - x64)
; CursorPosition = 13
; Folding = ---
; EnableXP
; CPU = 1