﻿; ------------------------------------------------------------------------------------
; Sichert die PB - Netzwerkbefehle ab
; ------------------------------------------------------------------------------------

#SafeNetwork_UDPWorkAround = #ENM_SN_UDPWorkAround

; Mapgröße
#SafeNetwork_MapSize = 2*#ENM_Sys_ConnectionSlots

; PrePing
#SafeNetwork_ConnectionPrePing = #ENM_SN_PingTimeout
#SafeNetwork_PingCacheTime = #ENM_SN_PingCacheTime
#SafeNetwork_WSABlockDelay = #ENM_SN_WSABlockDelay
#SafeNetwork_MaxWaitTime_Struct = 1000 ; Irgenwann muss Schluss sein...
#SafeNetwork_MaxWaitTime_Mutex = 5000 ; Irgenwann muss Schluss sein...

; UDP-WA Mutex setzen
Macro __UDPWa_SetMutex()
  CompilerIf #SafeNetwork_UDPWorkAround
    LockMutex(*Con\CounterMutex)
    If *Con\doDel
      Mutex = #False
    Else
      Mutex = \waUDPMutex
      *Con\MutexUsed + 1
    EndIf
    UnlockMutex(*Con\CounterMutex)
  CompilerEndIf
EndMacro
; Connection-Struktur laden
Macro __LoadConStruct(_Con = Connection)
  LockMutex(__NetworkProtector\GMutex)
  *Con = FindMapElement(__NetworkProtector\Connections(), Str(_Con))
  UnlockMutex(__NetworkProtector\GMutex)
EndMacro
; Connection - Struktur freigeben
Macro __DelConStruct(_Con)
  LockMutex(__NetworkProtector\GMutex)
  DeleteMapElement(__NetworkProtector\Connections(), Str(_Con))
  UnlockMutex(__NetworkProtector\GMutex)
EndMacro
; Connection - Struktur hinzufügen
Macro __AddConStruct(_Con, _Ping)
  LockMutex(__NetworkProtector\GMutex)
  *Con = AddMapElement(__NetworkProtector\Connections(), Str(_Con))
  UnlockMutex(__NetworkProtector\GMutex)
  *Con\Mutex = CreateMutex()
  *Con\ConId = ConnectionID(_Con)
  *Con\Ping = _Ping
  *Con\PingStamp = ElapsedMilliseconds()
EndMacro
; Warten bis die Connection - Struktur entfernt wurde
Macro __WaitUntilRemoved(_Con)
  CompilerIf Defined(__SN_Waiter, #PB_Variable) = #False
    Protected __SN_Waiter  
  CompilerEndIf
  __SN_Waiter = ElapsedMilliseconds()
  LockMutex(__NetworkProtector\GMutex)
  While FindMapElement(__NetworkProtector\Connections(), Str(_Con))
    UnlockMutex(__NetworkProtector\GMutex)
    Delay(1)
    LockMutex(__NetworkProtector\GMutex)
    If (ElapsedMilliseconds()-__SN_Waiter)>=#SafeNetwork_MaxWaitTime_Struct : Break : EndIf
  Wend
  UnlockMutex(__NetworkProtector\GMutex)
EndMacro
; Warten bis die Mutex nicht benutzt wird
Macro __WaitUntilNoMutexUse()
  CompilerIf Defined(__SN_Waiter, #PB_Variable) = #False
    Protected __SN_Waiter  
  CompilerEndIf
  __SN_Waiter = ElapsedMilliseconds()
  While *Con\MutexUsed
    Delay(1)
    If (ElapsedMilliseconds()-__SN_Waiter)>=#SafeNetwork_MaxWaitTime_Mutex : Break : EndIf
  Wend
EndMacro
; Mutex laden und
Macro __TryGetMutex()
  Mutex = *Con\Mutex ; Mutex laden
  If Mutex ; Mutex da?
    If *Con\doDel ; Wird gelöscht...
      Mutex = #False 
    Else ; Benutzung signalisieren
      *Con\MutexUsed + 1
    EndIf
  EndIf
EndMacro
; ... Freigeben
Macro __ReleaseMutex()
  *Con\MutexUsed - 1
EndMacro
; Mutex sicher freigeben
Macro __SafeFreeMutex(_Mutex)
  ; Letzten Lock - Zyklus abwarten
  If _Mutex
    LockMutex(_Mutex) : UnlockMutex(_Mutex)
    LockMutex(_Mutex) : UnlockMutex(_Mutex)
    LockMutex(_Mutex) : UnlockMutex(_Mutex)
    FreeMutex(_Mutex) ; Mutex freigeben
  EndIf
EndMacro
; Standard-Vorgang bei der Freigabe:
Macro __Std_FreeConnection(_Con)
  ; gibt es die Mutex?
  If Mutex
    ; Mutex austragen
    ; Warten bis die Mutex frei ist
    __WaitUntilNoMutexUse()
    ; Eintrag Freigeben
    __DelConStruct(_Con)
    ; Mutex Freigeben
    __SafeFreeMutex(Mutex)
  EndIf
EndMacro
; Standart-Vorgang zum Laden der Mutex:
Macro __Std_LoadMutex()
  __LoadConStruct() ; Struktur laden
  If *Con ; Con Ok?
    __TryGetMutex() ; Mutex laden
    If Not Mutex ; UDP-WA
      __UDPWa_SetMutex()
    EndIf
  EndIf
EndMacro

CompilerIf #PB_Compiler_OS = #PB_OS_Windows
  
  Procedure _WSA_Check()
    Protected RtVar = WSAGetLastError_()
    
    Select RtVar
      Case #WSAEWOULDBLOCK ; Block, warten ...
        Delay(#SafeNetwork_WSABlockDelay)
      Case #WSAEMSGSIZE ; MessageSize zu groß
      Case #WSAENETDOWN ; Network down
      Case #WSAENETUNREACH ; Network unreachable
      Default
    EndSelect
    
    ProcedureReturn RtVar
  EndProcedure
  
CompilerElse
  
  Macro _WSA_Check() ; Gibts ned...
  EndMacro
  
CompilerEndIf

;{ Client - #PB_NetworkEvent_Disconnect - Erkennung

CompilerIf #PB_Compiler_OS = #PB_OS_Linux ;{
  #FIONREAD     = $541B
  
  #__FD_SETSIZE = 1024
  #__NFDBITS    = 8 * SizeOf(LONG)
  
  Macro __FDELT(d)
    ((d) / #__NFDBITS)
  EndMacro
  
  Macro __FDMASK(d)
    (1 << ((d) % #__NFDBITS))
  EndMacro
  
  Structure FD_SET
    fds_bits.l[#__FD_SETSIZE / #__NFDBITS]
  EndStructure
  
  Procedure.i __FD_SET(d.i, *set.FD_SET)
    If d >= 0 And d < #__FD_SETSIZE
      *set\fds_bits[__FDELT(d)] | __FDMASK(d)
    EndIf
  EndProcedure
  
  Procedure.i __FD_ISSET(d.i, *set.FD_SET)
    If d >= 0 And d < #__FD_SETSIZE
      ProcedureReturn *set\fds_bits[__FDELT(d)] & __FDMASK(d)
    EndIf
  EndProcedure
  
  Procedure.i __FD_ZERO(*set.FD_SET)
    FillMemory(*set, SizeOf(FD_SET), 0, #PB_Byte)
  EndProcedure
  
  
  #FD_SETSIZE = #__FD_SETSIZE
  #NFDBITS    = #__NFDBITS
  
  Macro FD_SET(fd, fdsetp)
    __FD_SET(fd, fdsetp)
  EndMacro
  
  Macro FD_ISSET(fd, fdsetp)
    __FD_ISSET(fd, fdsetp)
  EndMacro
  
  Macro FD_ZERO(fdsetp)
    __FD_ZERO(fdsetp)
  EndMacro
  
  ; Returns the minimum value for NFDS
  Procedure.i _NFDS(*set.FD_SET)
    Protected I.i, J.i
    
    For I = SizeOf(FD_SET)/SizeOf(LONG) - 1 To 0 Step -1
      If *set\fds_bits[I]
        
        For J = (#__NFDBITS - 1) To 0 Step -1
          If *set\fds_bits[I] & (1 << J)
            ProcedureReturn I * #__NFDBITS + J + 1
          EndIf
        Next
        
      EndIf
    Next
    
    ProcedureReturn 0
  EndProcedure
  ;}
CompilerEndIf

CompilerIf #PB_Compiler_OS = #PB_OS_MacOS ;{
  #IOC_OUT  = $40000000 ;(__uint32_t)
  Macro _IOR(g,n,t)
    _IOC(#IOC_OUT, (g), (n), (t))
  EndMacro
  #IOCPARM_MASK = $1fff
  Macro _IOC(inout,group,num,len)
    ((inout) | (((len) & #IOCPARM_MASK) << 16) | ((group) << 8) | (num))
  EndMacro
  #FIONREAD = _IOR('f', 127, SizeOf(LONG))
  
  #__DARWIN_FD_SETSIZE = 1024
  #__DARWIN_NBBY       = 8
  #__DARWIN_NFDBITS    = SizeOf(LONG) * #__DARWIN_NBBY
  
  Structure FD_SET
    fds_bits.l[ (#__DARWIN_FD_SETSIZE + #__DARWIN_NFDBITS - 1) / #__DARWIN_NFDBITS ]
  EndStructure
  
  Procedure.i __DARWIN_FD_SET(fd.i, *p.FD_SET)
    If fd >= 0 And fd < #__DARWIN_FD_SETSIZE
      *p\fds_bits[fd / #__DARWIN_NFDBITS] | (1 << (fd % #__DARWIN_NFDBITS))
    EndIf
  EndProcedure
  
  Procedure.i __DARWIN_FD_ISSET(fd.i, *p.FD_SET)
    If fd >= 0 And fd < #__DARWIN_FD_SETSIZE
      ProcedureReturn *p\fds_bits[fd / #__DARWIN_NFDBITS] & (1 << (fd % #__DARWIN_NFDBITS))
    EndIf
  EndProcedure
  
  Procedure.i __DARWIN_FD_ZERO(*p.FD_SET)
    FillMemory(*p, SizeOf(FD_SET), 0, #PB_Byte)
  EndProcedure
  
  #FD_SETSIZE = #__DARWIN_FD_SETSIZE
  
  Macro FD_SET(n, p)
    __DARWIN_FD_SET(n, p)
  EndMacro
  
  Macro FD_ISSET(n, p)
    __DARWIN_FD_ISSET(n, p)
  EndMacro
  
  Macro FD_ZERO(p)
    __DARWIN_FD_ZERO(p)
  EndMacro
  
  ; Returns the minimum value for NFDS
  Procedure.i _NFDS(*p.FD_SET)
    Protected I.i, J.i
    
    For I = SizeOf(FD_SET)/SizeOf(LONG) - 1 To 0 Step -1
      If *p\fds_bits[I]
        
        For J = (#__DARWIN_NFDBITS - 1) To 0 Step -1
          If *p\fds_bits[I] & (1 << J)
            ProcedureReturn I * #__DARWIN_NFDBITS + J + 1
          EndIf
        Next
        
      EndIf
    Next
    
    ProcedureReturn 0
  EndProcedure
  ;}
CompilerEndIf

CompilerIf #PB_Compiler_OS = #PB_OS_Windows ;{
  ; #FIONREAD is already defined
  ; FD_SET is already defined
  
  Structure _fd_set
    fd_count.i
    fd_array.i[#FD_SETSIZE]
  EndStructure
  
  Macro fd_set
    _fd_set  
  EndMacro
  
  Macro FD_ZERO(set)
    set\fd_count = 0
  EndMacro
  
  Procedure.i FD_SET(fd.i, *set.FD_SET)
    If *set\fd_count < #FD_SETSIZE
      *set\fd_array[ *set\fd_count ] = fd
      *set\fd_count + 1
    EndIf
  EndProcedure
  
  Procedure.i FD_ISSET(fd.i, *set.FD_SET)
    Protected I.i
    For I = *set\fd_count - 1 To 0 Step -1
      If I < #FD_SETSIZE
        If *set\fd_array[I] = fd
          ProcedureReturn #True
        EndIf
      EndIf
    Next
    
    ProcedureReturn #False
  EndProcedure
  
  Procedure.i _NFDS(*set.FD_SET)
    ProcedureReturn *set\fd_count
  EndProcedure
  ;}
CompilerEndIf

CompilerIf Defined(TIMEVAL, #PB_Structure) = #False ;{
  Structure TIMEVAL
    tv_sec.l
    tv_usec.l
  EndStructure ;}
CompilerEndIf

;}

Structure __ConnectionInfo
  doDel.a ; Ich werde gelöscht :)
  Mutex.i ; Mutex
  ConId.i ; Connection Id
  MutexUsed.i ; Mutex wird benutzt
  Ping.i ; Ping
  PingStamp.i ; Zeitmarke für den Ping
EndStructure

Structure __vId_Holder
  Id.q
  exist.a
EndStructure

Structure __NetworkProtector_Data
  Map Connections.__ConnectionInfo(#SafeNetwork_MapSize)
  CompilerIf #SafeNetwork_UDPWorkAround
    waUDPMutex.i
  CompilerEndIf
  EventClient.i
  GMutex.i ; Globale Mutex
EndStructure

Global __NetworkProtector.__NetworkProtector_Data

; Erstellt die Verwaltung
Procedure __NetworkProtector_Init()
  Shared __NetworkProtector
  With __NetworkProtector
    CompilerIf #SafeNetwork_UDPWorkAround
      \waUDPMutex = CreateMutex() ; Workaround für UDP
    CompilerEndIf
    \GMutex = CreateMutex()
  EndWith
EndProcedure : __NetworkProtector_Init()

; Event - Client abstrahieren
Procedure __EventClient()
  Shared __NetworkProtector
  With __NetworkProtector
    ProcedureReturn \EventClient
  EndWith
EndProcedure

Procedure __NetworkServerEvent()
  Protected Event, Mutex, *Con.__ConnectionInfo
  Shared __NetworkProtector
  
  With __NetworkProtector
    Event = NetworkServerEvent()
    
    If Event   
      \EventClient = EventClient() ; EventClient laden
      
      If Event = #PB_NetworkEvent_Connect ; Connect - Event : Lokale Schutzmutex für jede Verbindung erzeugen
        ; Warten bis nicht mehr vorhanden
        __WaitUntilRemoved(\EventClient)
        ; Eintrag anlegen
        __AddConStruct(\EventClient, Ping(IPString(GetClientIP(\EventClient)), #SafeNetwork_ConnectionPrePing))
       
      ElseIf Event = #PB_NetworkEvent_Disconnect ; Disconnect - Event: Lokale Schutzmutex abbauen
        __LoadConStruct(\EventClient)
        If *Con
          Mutex = *Con\Mutex
          If Mutex
            ; Nur einer kann löschen...
            If *Con\doDel
              Mutex = #False
              Event = #False
            Else
              *Con\doDel = #True
              *Con\Mutex = #Null
            EndIf
          Else ; es gibt kein Event ...
            Event = #False
          EndIf
        Else ; es gibt kein Event...
          Event = #False
        EndIf
        
        ; Standartmäßig freigeben
        __Std_FreeConnection(\EventClient)
      EndIf
    EndIf
  EndWith
  
  ProcedureReturn Event
EndProcedure

Procedure __NetworkClientEvent(Connection)
  Protected Event, Mutex, *Con.__ConnectionInfo
  Protected hSocket.i, tv.timeval, readfds.fd_set, Length.i
  Shared __NetworkProtector
  
  With __NetworkProtector
    
    __LoadConStruct()
    If *Con
      Mutex = *Con\Mutex
      If Mutex
        If *Con\doDel
          Mutex = #False
        Else
          *Con\MutexUsed + 1
        EndIf
        hSocket = *Con\ConId
      EndIf
    EndIf
    
    If Mutex
      
      LockMutex(Mutex)
      
      If hSocket < 65536 And hSocket > 0
        Event = NetworkClientEvent(Connection)
      Else
        Event = #PB_NetworkEvent_Disconnect
      EndIf
      
      If Not Event ; Erweiterung um das #PB_NetworkEvent_Disconnect zu erkennen
        tv\tv_sec  = 0 ; Dont even wait, just query status
        tv\tv_usec = 0
        
        If hSocket > 65536 Or hSocket < 1 ; Gültiger bereich
          Event = #PB_NetworkEvent_Disconnect
        Else
          
          FD_ZERO(readfds)
          FD_SET(hSocket, readfds)
          
          ; Check if there is something new
          Event = select_(_NFDS(readfds), @readfds, #Null, #Null, @tv)
          If Event < 0 ; Seems to be an error
            Event = #PB_NetworkEvent_Disconnect
          ElseIf Event = 0 Or Not FD_ISSET(hSocket, readfds) ; No data available
            Event = #Null
          Else
            ; Check if data is available?
            CompilerIf #PB_Compiler_OS = #PB_OS_Windows
              Event = ioctlsocket_(hSocket, #FIONREAD, @Length)
            CompilerElse
              Event = ioctl_(hSocket, #FIONREAD, @Length)
            CompilerEndIf
            
            If Event Or Length = 0 ; Not successful to query for data available OR no data available ? This seems to be an error!
              Event = #PB_NetworkEvent_Disconnect
            Else
              Event = #Null
            EndIf
          EndIf
        EndIf
      EndIf
      
      UnlockMutex(Mutex)
      
      *Con\MutexUsed - 1
      
      If Event = #PB_NetworkEvent_Disconnect ; Disconnect - Event: Mutex austragen
        ; Nur einer kann löschen...
        If *Con\doDel
          Mutex = #False
          Event = #False
        Else
          If Mutex = *Con\Mutex
            *Con\doDel = #True
            *Con\Mutex = #Null
          Else
            Mutex = #False
            Event = #False
          EndIf
        EndIf

        ; Standartmäßig freigeben
        __Std_FreeConnection(Connection)
      EndIf
    Else
      Event = #PB_NetworkEvent_Disconnect
    EndIf
    
  EndWith
  
  ProcedureReturn Event
EndProcedure

Procedure __OpenNetworkConnection(Host$, Port, Type = #PB_Network_TCP)
  Protected Connection, Ping, *Con.__ConnectionInfo
  Shared __NetworkProtector
  
  With __NetworkProtector
    CompilerIf #SafeNetwork_ConnectionPrePing > #Null
      Ping = Ping(Host$, #SafeNetwork_ConnectionPrePing)
    CompilerElse
      Ping = #Null
    CompilerEndIf
    
    If Ping > -1 ; PrePing Ok
      Connection = OpenNetworkConnection(Host$, Port, Type)
      If Connection
        ; Wird gerade gelöscht
        __WaitUntilRemoved(Connection)
        ; Eintrag anlegen
        __AddConStruct(Connection, Ping)
      EndIf
    EndIf
  EndWith
  
  ProcedureReturn Connection
EndProcedure

Procedure __CloseNetworkConnection(Connection)
  Protected isOk, Mutex, *Con.__ConnectionInfo
  Protected sid
  Shared __NetworkProtector
  
  With __NetworkProtector
    __LoadConStruct()
    If *Con
      Mutex = *Con\Mutex
      If Mutex
        ; Löschflag setzen
        If *Con\doDel
          Mutex = #False
        Else
          *Con\doDel = #True
          *Con\Mutex = #Null
        EndIf
      EndIf
    EndIf
    
    If Mutex      
      ; sid für den WA
      sid = *Con\ConId
      
      ; Standartmäßig freigeben
      __Std_FreeConnection(Connection)
      
      ; Connection beenden 
      ; Windows - Work Around
      CompilerIf (#PB_Compiler_OS = #PB_OS_Windows) And (#ENM_Sys_WindowsCloseConnectionWA = #True)
        Protected Socket.LINGER
        Socket\l_onoff = 1
        Socket\l_linger = 0
        If setsockopt_(sid, #SOL_SOCKET, #SO_LINGER, @Socket, SizeOf(Linger)) = #ERROR_SUCCESS
          isOk = ((closesocket_(sid) = #ERROR_SUCCESS) Or #Null)
        EndIf
      CompilerElse
        isOk = CloseNetworkConnection(Connection)
      CompilerEndIf
    EndIf
  EndWith
  
  ProcedureReturn isOk
EndProcedure

Procedure __ConnectionID(Connection)
  Protected isOk, *Con.__ConnectionInfo
  Shared __NetworkProtector
  
  With __NetworkProtector
    __LoadConStruct()
    If *Con
      ; Ist die Mutex da, wird gerade gelöscht, ...
      isOk = (*Con\Mutex Or #Null)*(*Con\ConId*(1-(*Con\doDel Or #Null))) ; Jaja, sehr komplexe Sache... ^^
    EndIf
  EndWith
  
  ProcedureReturn isOk
EndProcedure

Procedure __SendNetworkData(Connection, *Mem, MemSize)
  Protected isOk, Mutex, *Con.__ConnectionInfo
  Shared __NetworkProtector
  
  With __NetworkProtector
    ; Versucht die Mutex zu laden...
    __Std_LoadMutex()
    
    If Mutex
      LockMutex(Mutex)
      isOk = SendNetworkData(Connection, *Mem, MemSize)
      UnlockMutex(Mutex)
      If isOk < 0
        _WSA_Check() ; Fehler behandeln
        isOk = #Null
      EndIf
    
      ; Mutex Benutzung zurücksetzen
      __ReleaseMutex()
    EndIf
  EndWith
  
  ProcedureReturn isOk
EndProcedure

Procedure __SendNetworkString(Connection, String$)
  Protected isOk, Mutex, *Con.__ConnectionInfo
  Shared __NetworkProtector
  
  With __NetworkProtector
    ; Versucht die Mutex zu laden...
    __Std_LoadMutex()
    
    If Mutex
      LockMutex(Mutex)
      isOk = SendNetworkString(Connection, String$)
      UnlockMutex(Mutex)
      
      ; Mutex Benutzung zurücksetzen
      __ReleaseMutex()
    EndIf
  EndWith
  
  ProcedureReturn isOk
EndProcedure

Procedure __SendNetworkFile(Connection, File$)
  Protected isOk, Mutex, *Con.__ConnectionInfo
  Shared __NetworkProtector
  
  With __NetworkProtector
    ; Versucht die Mutex zu laden...
    __Std_LoadMutex()

    If Mutex
      LockMutex(Mutex)
      isOk = SendNetworkFile(Connection, File$)
      UnlockMutex(Mutex)
      
      ; Mutex Benutzung zurücksetzen
      __ReleaseMutex()
    EndIf
  EndWith
  
  ProcedureReturn isOk
EndProcedure

Procedure __GetClientIP(Connection)
  Protected isOk, Mutex, *Con.__ConnectionInfo
  Shared __NetworkProtector
  
  With __NetworkProtector
    ; Versucht die Mutex zu laden...
    __Std_LoadMutex()

    If Mutex
      LockMutex(Mutex)
      isOk = GetClientIP(Connection)
      UnlockMutex(Mutex)
      
      ; Mutex Benutzung zurücksetzen
      __ReleaseMutex()
    EndIf
  EndWith
  
  ProcedureReturn isOk
EndProcedure

Procedure __GetClientPort(Connection)
  Protected isOk, Mutex, *Con.__ConnectionInfo
  Shared __NetworkProtector
  
  With __NetworkProtector    
    ; Versucht die Mutex zu laden...
    __Std_LoadMutex()
    
    If Mutex
      LockMutex(Mutex)
      isOk = GetClientPort(Connection)
      UnlockMutex(Mutex)
      
      ; Mutex Benutzung zurücksetzen
      __ReleaseMutex()
    EndIf
  EndWith
  
  ProcedureReturn isOk
EndProcedure

; Ping
Procedure GetClientPing(Connection)
  Protected isOk, Mutex, *Con.__ConnectionInfo
  Shared __NetworkProtector
  
  CompilerIf #SafeNetwork_ConnectionPrePing > #Null
    
    With __NetworkProtector
      ; Versucht die Mutex zu laden...
    __Std_LoadMutex()
      
      If Mutex
        If (*Con\PingStamp+#SafeNetwork_PingCacheTime)<ElapsedMilliseconds() ; Ping neuladen
          LockMutex(Mutex)
          *Con\Ping = Ping(IPString(GetClientIP(Connection)), #SafeNetwork_ConnectionPrePing)
          *Con\PingStamp = ElapsedMilliseconds()
          UnlockMutex(Mutex)
        EndIf
        isOk = *Con\Ping

        ; Mutex Benutzung zurücksetzen
        __ReleaseMutex()
      EndIf
    EndWith
  CompilerElse
    isOk = #Null
  CompilerEndIf
  
  ProcedureReturn isOk
EndProcedure

Procedure __ReceiveNetworkData(Connection, *Mem, MemSize)
  Protected isOk, Mutex, *Con.__ConnectionInfo
  Shared __NetworkProtector
  
  With __NetworkProtector
    ; Versucht die Mutex zu laden...
    __Std_LoadMutex()

    If Mutex
      LockMutex(Mutex)
      isOk = ReceiveNetworkData(Connection, *Mem, MemSize)
      UnlockMutex(Mutex)
      If isOk = -1
        isOk = #Null
      EndIf
      
      ; Mutex Benutzung zurücksetzen
      __ReleaseMutex()
    EndIf
  EndWith
  
  ProcedureReturn isOk
EndProcedure


; PB - Befehle überschreiben

Macro EventClient()
  __EventClient()
EndMacro

Macro NetworkServerEvent()
  __NetworkServerEvent()  
EndMacro

Macro NetworkClientEvent(Connection)
  __NetworkClientEvent(Connection)  
EndMacro

Macro OpenNetworkConnection(ServerName, Port, Type = #PB_Network_TCP)
  __OpenNetworkConnection(ServerName, Port, Type)  
EndMacro

Macro CloseNetworkConnection(Connection)
  __CloseNetworkConnection(Connection)  
EndMacro

Macro ConnectionID(Connection)
  __ConnectionID(Connection)  
EndMacro

Macro SendNetworkData(Connection, Mem, MemSize)
  __SendNetworkData(Connection, Mem, MemSize)  
EndMacro

Macro SendNetworkString(Connection, String)
  __SendNetworkString(Connection, String)  
EndMacro

Macro SendNetworkFile(Connection, File)
  __SendNetworkFile(Connection, File)  
EndMacro

Macro ReceiveNetworkData(Connection, Mem, MemSize)
  __ReceiveNetworkData(Connection, Mem, MemSize)
EndMacro

Macro GetClientIP(Connection)
  __GetClientIP(Connection)
EndMacro

Macro GetClientPort(Connection)
  __GetClientIP(Connection)
EndMacro

; IDE Options = PureBasic 4.60 Beta 3 (Windows - x64)
; CursorPosition = 590
; FirstLine = 18
; Folding = AA9-+-v-----
; EnableXP
; CPU = 1